Package: HiTC
Type: Package
Title: High Throughput Chromosome Conformation Capture analysis
Description: The HiTC package was developed to explore high-throughput 'C' data such as 5C or Hi-C. Dedicated R classes as well as standard methods for quality controls, normalization, visualization, and further analysis are also provided.
Version: 1.56.0
Date: 2025-07-22
Depends: R (>= 2.15.0), methods, IRanges, GenomicRanges
Imports: Biostrings, graphics, grDevices, rtracklayer, RColorBrewer,
        Matrix, parallel, Seqinfo
Suggests: BiocStyle, HiCDataHumanIMR90, BSgenome.Hsapiens.UCSC.hg18
Author: Nicolas Servant
Maintainer: Nicolas Servant <nicolas.servant@curie.fr>
License: Artistic-2.0
Collate: AllGenerics.R HTCexp-class.R HTClist-class.R qualityControl.R
        mapC.R mapC_tracks.R binningC.R export.R import.R pca.R
        normalize_5C.R normalize_hiC.R tads.R deprecated.R
LazyLoad: yes
biocViews: Sequencing, HighThroughputSequencing, HiC
Packaged: 2026-04-28 22:39:16 UTC; biocbuild
InstallableEverywhere: yes
NeedsCompilation: no
git_url: https://git.bioconductor.org/packages/HiTC
git_branch: RELEASE_3_23
git_last_commit: 4d550f9
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28

#############################
##
## IMPORT
##
#############################

##For S4 usage
import("methods")
#import("Matrix")
import("GenomicRanges")
import("IRanges")

##Package used in HiTC
importFrom("RColorBrewer", "brewer.pal")
importFrom("Matrix", "Matrix", "sparseMatrix", "t")
importFrom("Seqinfo", "seqlevels", "sortSeqlevels", "seqlengths")
importFrom("parallel", "mclapply")
importFrom("grDevices", "col2rgb", "colorRampPalette", "dev.off", "pdf", "pdf.options", "png")
importFrom("graphics", "plot", "image", "abline", "axis", "text", "barplot", "hist", "layout", "legend", "lines", "par", "points", "rect")
importFrom("stats", "prcomp", "approx", "density", "glm", "lm", "lowess", "residuals")
importFrom("utils", "combn", "object.size", "packageVersion", "read.table", "write.table")

##importFrom("MASS", "glm.nb")
importMethodsFrom("Biostrings", "detail")
importMethodsFrom("rtracklayer", "import", "export")
importMethodsFrom("Matrix", "colMeans", "colSums", "crossprod", "diag", "forceSymmetric", "isTriangular", "rowSums", "nnzero")

#############################
##
## EXPORT
##
#############################

##Export functions and generics
export(
    "HTCexp","HTClist",
    "binningC",
    "exportC", "export.my5C", "importC", "import.my5C",
    "removeIntervals", "setIntervalScale",   
    "CQC", "intervalsDist",
    "extractRegion",
    "pca.hic","getPearsonMap",
    "getExpectedCounts", "normLGF","normICE", "setGenomicFeatures",
    "getAnnotatedRestrictionSites", "getRestrictionFragmentsPerChromosome",
    "discretize",
    "directionalityIndex"
)

##Export classes
exportClasses("HTCexp", "HTClist")

##Export Methods
exportMethods(
    "intdata", "intdata<-",
    "id",
    "y_intervals", "y_intervals<-", "x_intervals", "x_intervals<-", "xy_intervals",
    "getCombinedIntervals", "getCombinedContacts",
    "forcePairwise", "forceTriangular",
    "isComplete", "isPairwise",
    "isBinned", "isIntraChrom",
    "divide", "substract",
    "mapC",
    "summary",
    "as.list", "c", "detail", "show",
    "plot",
    "seqlevels",
    "range", "ranges", "reduce",
    "substract", "divide",
    "forcePairwise", "forceSymmetric", "forceTriangular",
    "isTriangular", "isSymmetric",    
    "normPerReads", "normPerTrans", "normPerExpected"
    )

###################################
## pca.hic
##
## PCA analysis on interactio map as proposed in Lieberman et al.
##
## x = HTCexp object
## norm = if true, apply the normPerExpected normalization
## npc = number of component to report
## asGRangesList = if TRUE a GRangesList object is returned where the score represents the eigenvector
##################################

pca.hic <- function(x, normPerExpected=TRUE, npc=2, asGRangesList=TRUE, gene.gr=NULL, ...){
    stopifnot(inherits(x,"HTCexp"))
    stopifnot(isBinned(x))
    
    xcor <- getPearsonMap(x, normPerExpected, ...)
    xdata.cor <- intdata(xcor)
    xgi <- x_intervals(x)
    
    ## remove NA if still there
    idx <- which(apply(xdata.cor, 1, function(x){length(which(is.na(x)))}) != dim(xdata.cor)[1])
    xdata.cor <- xdata.cor[idx,idx]

    ## Perform PCA
    if (length(xdata.cor) == 0){
      warning("Empty correlation matrix for ", seqlevels(x))
      pca.res <- NULL
    }
    else{
      pca <- prcomp(xdata.cor, scale=TRUE)

      ## Gene density
      if (!is.null(gene.gr)){
          stopifnot(inherits(gene.gr,"GenomicRanges"))
          gene.density <- countOverlaps(xgi, gene.gr)
      }
     
      ## Get results
      if (!asGRangesList){
        pca.res <- list()
        pscore <- rep(NA, length(xgi))
        pca.res$varianceProp <- summary(pca)$importance[2,1:npc]
        for (i in 1:npc){
            pscore[idx] <- round(pca$rotation[,i],3)
                       
            if (!is.null(gene.gr)){
                gd.pos <- sum(gene.density[which(pscore>=0)])
                gd.neg <- sum(gene.density[which(pscore<0)])
                message("Gene density per bin - >0 = ", gd.pos)
                message("Gene density per bin - <0 = ", gd.neg)
                
                ## Add Chromosome compartment information
                if (gd.pos > gd.neg)
                    cc <- ifelse(pscore>0, "A", "B")
                else{
                    cc <- ifelse(pscore<0, "A", "B")
                pscore <- -pscore
                }
                pca.res[[eval(paste("PC",i, sep=""))]] <- rbind(pscore, cc, gene.density)
            }else{
                pca.res[[eval(paste("PC",i, sep=""))]] <- pscore
            }
        }
    }else{
        pca.res <- GRangesList()      
        for (i in 1:npc){
          pscore <- rep(NA, length(xgi))
          pscore[idx] <- round(pca$rotation[,i],3)
          
          if (!is.null(gene.gr)){
              gd.pos <- sum(gene.density[which(pscore>=0)])
              gd.neg <- sum(gene.density[which(pscore<0)])
              message("Gene density per bin - >0 = ", gd.pos)
              message("Gene density per bin - <0 = ", gd.neg)
            
            ## Add Chromosome compartment information
              if (gd.pos > gd.neg)
                  cc <- ifelse(pscore>0, "A", "B")
              else{
                  pscore <- -pscore
                  cc <- ifelse(pscore>0, "A", "B")
              }
              pca.res[[eval(paste("PC",i, sep=""))]] <- GRanges(seqnames(xgi), ranges=ranges(xgi), strand=strand(xgi), score=pscore, genedens=gene.density, ccompartments=cc)
          }else{
              pca.res[[eval(paste("PC",i, sep=""))]] <- GRanges(seqnames(xgi), ranges=ranges(xgi), strand=strand(xgi), score=pscore)
          }         
      }
    }
  }
    return(pca.res)
}

###################################
## sparseCor
##
## Correlation methods for sparse Matrix
##
## x = Matrix object
##################################

sparseCor <- function(x){
    n <- nrow(x)
    cMeans <- colMeans(x, na.rm=TRUE)
    covmat <- (as.matrix(crossprod(x)) - n*tcrossprod(cMeans))/(n-1)
    sdvec <- sqrt(diag(covmat)) 
    cormat <- covmat/tcrossprod(sdvec)
       
    colnames(cormat) <- colnames(x)
    rownames(cormat) <- rownames(x)
    as(cormat, "Matrix")
}


getPearsonMap <- function(x, normPerExpected=TRUE, center=TRUE, ...){
  stopifnot(inherits(x,"HTCexp"))
  stopifnot(isBinned(x))

  ## force symmetric matrix
  x <- forceSymmetric(x)
  x.save <- x
 
  ## observed/expected map
  if (normPerExpected){
    x <- normPerExpected(x, ...)
  }
  
  ## Remove bins with sd=0 to avoid correlation failure
  idx.sd <- union(names(which(apply(intdata(x), c(1), sd, na.rm=TRUE)==0)),
               names(which(apply(intdata(x), c(2), sd, na.rm=TRUE)==0)))
  x <- removeIntervals(x,idx.sd)
  
  ## Remove bins with NAs
  idx.na <- union(names(which(apply(intdata(x), 1, function(x) all(is.na(x))))),
               names(which(apply(intdata(x), 2, function(x) all(is.na(x))))))
  x <- removeIntervals(x,idx.na)

  ## Calculate correlation
  ## use="pairwise.complete.obs" means that the correlation between two vectors is calculated using only the paired non-NAs values.
  ## It means that two correlations can be calculated using a different set of points ...
  ## In theory this never appends because bins with NAs values should be removed earlier

  ## scale on rows
  ## The Pearson's is calculated after subtracting the mean of the row from the O/E matrix.  This is because correlation of small values should be as valuable as correlation of big values.  We subtract the row mean from every entry in the row, in effect recentering the distribution.  O/E ranges from something like 1/5 to 5, and values below 1 that are correlated/anti-correlated with values above 1 need to count that way.
  if (center){
    xdata <- t(scale(t(intdata(x)), center=TRUE, scale=FALSE))
  }else{
    xdata <- as.matrix(intdata(x))
  }

  ## check diagonal, if diag is NA put 0
  if (is.na(max(diag(xdata), na.rm=FALSE))){
      diag(xdata)<-0
  }

  ## calculate correlation of columns
  xdata.cor <- sparseCor(xdata)

  ## reput NA
  ##idx <- setdiff(colnames(intdata()), c(idx.sd, idx.na))
  N <- dim(intdata(x.save))[1]
  cids <- colnames(intdata(x.save))
  rids <- rownames(intdata(x.save))
 
  xdata.final <- matrix(NA, ncol=N, nrow=N)
  colnames(xdata.final) <- cids
  rownames(xdata.final) <- rids
  xdata.final[rownames(xdata.cor),colnames(xdata.cor)] <- as.vector(xdata.cor)
  
  x.save@intdata <- as(xdata.final, "Matrix")
  x.save
}

###################################
## exportC
##
## Export HiTC object to standard format
##
## x = an object of class HTCexp
## con = basename of files to create
##
###################################

writeC <- function(data2export, xgi, ygi, file, genome="mm9", header=FALSE, use.names=TRUE, rm.na=TRUE){
    message("Export genomic ranges as BED files ...")

    ## Use names
    if (!use.names){
        rownames(data2export) <- 1:nrow(data2export)
        colnames(data2export) <- 1:ncol(data2export)
        elementMetadata(xgi)$name <- 1:length(xgi)
        if (!is.null(ygi)){
            elementMetadata(ygi)$name <- 1:length(ygi)
        }
    }
    
    ## Remove NA
    if (rm.na){
        data2export[Matrix::which(is.na(data2export))] <- 0
    }
    
    ## Write header and BED files
    bed.xgi <- paste(file,"_xgi.bed", sep="")
    if (header){
        write(paste("##HiTC - v", packageVersion("HiTC"), sep=""),file=bed.xgi)
        write(paste("##",date(), sep=""),file=bed.xgi, append=TRUE)
        rtracklayer::export(xgi, con=paste(file,"_xgi.bed", sep=""), format="bed", append=TRUE)
    }else{
        rtracklayer::export(xgi, con=paste(file,"_xgi.bed", sep=""), format="bed", append=FALSE)
    }  

    if(!is.null(ygi)){
        bed.ygi <- paste(file,"_ygi.bed", sep="")  
        if (header){
            write(paste("##HiTC - v", packageVersion("HiTC"), sep=""),file=bed.ygi)
            write(paste("##",date(), sep=""),file=bed.ygi, append=TRUE)
            rtracklayer::export(ygi, con=paste(file,"_ygi.bed", sep=""), format="bed", append=TRUE)
        }else{
            rtracklayer::export(ygi, con=paste(file,"_ygi.bed", sep=""), format="bed", append=FALSE)
        }
    }
    
    ## Export map
    count.out <- paste(file,".mat", sep="")
    message("Export interaction map in '",count.out,"' ...")
    data2export <- as(data2export, "TsparseMatrix")
    
    if (header){
        write(paste("##HiTC - v", packageVersion("HiTC"), sep=""),file=count.out)
        write(paste("##",date(), sep=""),file=count.out, append=TRUE)
        write(paste(rownames(data2export)[data2export@i+1], colnames(data2export)[data2export@j+1], data2export@x, sep="\t"), file=count.out, append=TRUE)
    }else{
        write(paste(rownames(data2export)[data2export@i+1], colnames(data2export)[data2export@j+1], data2export@x, sep="\t"), file=count.out, append=FALSE)
    }
    invisible(NULL)
}
    


exportC <- function(x, file, per.chromosome=FALSE, use.names=FALSE, header=FALSE){
    if (inherits(x, "HTCexp")){
        ##stopifnot(isSymmetric(x))
        data2export <- intdata(x)
        xgi <- x_intervals(x)
        if(!isSymmetric(x))
            ygi <- y_intervals(x)
        else
            ygi <- NULL
        writeC(data2export, xgi, ygi, file, header=header, use.names=use.names)
    }else if (inherits(x, "HTClist")){
        if (per.chromosome){
            lapply(x, function(xx){
                data2export <- intdata(xx)
                xgi <- x_intervals(xx)
                if(!isSymmetric(xx))
                    ygi <- y_intervals(xx)
                else
                    ygi <- NULL
                chrname <- paste0(seqlevels(xx), collapse="")
                writeC(data2export, xgi, ygi, file=paste0(file, "_", chrname), header=header, use.names=use.names)
            })
        }else{
            if (isComplete(x)){
                data2export <- getCombinedContacts(x)
                combi <- getCombinedIntervals(x)
                writeC(data2export, combi$xgi, combi$ygi, file, use.names=use.names, header=header)
            }else{
                stop("isComplete(x) is not TRUE. Cannot export incomplete HTClist object as one file. Please use per.chromosome=TRUE.")
            }
        }
    }
    invisible(NULL)
}##exportC


###################################
## export.my5C
##
## Export HiTC object to my5C
##
## x = an object of class HTCexp
## file = name of file to create
##
###################################

write.my5C <- function(data2export, xgi, ygi, file, genome="mm9", header=TRUE){

    file <- paste0(file, ".mat")
    if (header){
        write(paste0("##HiTC - v", packageVersion("HiTC")),file=file)
        write(paste0("##",date()),file=file, append=TRUE)
        write("##my5C export matrix format",file=file, append=TRUE)
    }

    xnames <- paste(id(xgi),"|",genome,"|",seqnames(xgi),":",start(xgi),"-",end(xgi), sep="")
    ynames <- paste(id(ygi),"|",genome,"|",seqnames(ygi),":",start(ygi),"-",end(ygi), sep="")
    colnames(data2export) <- xnames
    rownames(data2export) <- ynames

    message("Exporting data in '",file,"' ...")
    suppressWarnings(write.table(as.data.frame(as.matrix(data2export)), file=file, quote=FALSE, sep="\t", append=TRUE))
}##write.my5C
    
export.my5C <- function(x, file, genome="mm9", per.chromosome=FALSE){
    if (inherits(x, "HTCexp")){
        ##stopifnot(isSymmetric(x))
        data2export <- intdata(x)
        xgi <- x_intervals(x)
        ygi <- y_intervals(x)
        write.my5C(data2export, xgi, ygi, genome=genome, file=file)
      
    }else if (inherits(x, "HTClist")){
        if (per.chromosome){
            lapply(x, function(xx){
                data2export <- intdata(xx)
                xgi <- x_intervals(xx)
                ygi <- y_intervals(xx)
                chrname <- paste0(seqlevels(xx), collapse="")
                write.my5C(data2export, xgi, ygi, file=paste0(file, "_", chrname))
            })
        }else{
            if (isComplete(x)){
                data2export <- getCombinedContacts(x)
                combi <- getCombinedIntervals(x)
                ygi <- combi$ygi
                if (!is.null(combi$xgi))
                    xgi <- combi$xgi
                else
                    xgi <- ygi
                write.my5C(data2export, xgi, ygi, file=paste0(file, "_", genome))
            }else{
                stop("isComplete(x) is not TRUE. Cannot export incomplete HTClist object as one file. Please use per.chromosome=TRUE.")
            }
        }
    }
    invisible(NULL)
}##export.my5C

###################################
## saveContactMaps
##
## Export HiTC object in images at the good resolution
##
## x = an object of class HTCexp
## con = output file name or prefix
##
###################################

saveContactMaps <- function(x, con, device="pdf", per.chrom=FALSE, ...)
{
  stopifnot(!missing(con))

  if (length(grep(".pdf$", con)) != 0)
    device <- "pdf"
  else if (length(grep(".png$", con)) != 0)
    device <- "png"

  if (device == "pdf")
    dFun <- pdf
  else if (device == "png")
    dFun <- png
  else
    stop("Unsupported device")

  minres <- 680
  if(inherits(x,"HTCexp")){
    d <- dim(intdata(x))
    if (d[1] < minres || d[2] < minres){d <- c(minres, minres)}
    dFun(con,  width=d[1], height=d[2])
    mapC(x, ...)
    dev.off()
  }else if(inherits(x,"HTClist")){
    if (per.chrom==TRUE){
      if (device == "pdf"){
        dFun(con)
        lapply(x, function(xx){
          d <- dim(intdata(xx))
          if (d[1] < minres || d[2] < minres){d <- c(minres, minres)}
          pdf.options(width=d[1], height=d[2])
          mapC(HTClist(xx), ...)
        })
        dev.off()
      }else if (device=="png"){
        lapply(x, function(xx){
          d <- dim(intdata(xx))
          if (d[1] < minres || d[2] < minres){d=c(minres, minres)}
          dFun(paste0(sub(".png$","",con), "_", paste0(seqlevels(xx), collapse=""), ".png"),  width=d[1], height=d[2])
          mapC(HTClist(xx), ...)
          dev.off()
        })
      }
    }else{
      dFun(con,  width=1200, height=1200)
      mapC(x, ...)
      dev.off()
    }
  }
}

###################################
## importC
##
## Import HTCexp object to standard format
## rows/col/counts
##
## file = name of intput file to read
###################################

importC <- function(con, xgi.bed, ygi.bed=NULL, allPairwise=FALSE, rm.trans=FALSE, lazyload=FALSE){
    
    stopifnot(!missing(con))

    if(is.null(xgi.bed) && is.null(ygi.bed))
        stop("BED files of x/y intervals are required")
    
    message("Loading Genomic intervals ...")
    xgi <- rtracklayer::import(xgi.bed, format="bed")
    xgi <- sortSeqlevels(xgi)
    names(xgi) <- id(xgi)
    if (!is.null(ygi.bed)){
        ygi <- rtracklayer::import(ygi.bed, format="bed")
        ygi <- sortSeqlevels(ygi)
        names(ygi) <- id(ygi)
    }else{
        ygi <- xgi
    }
    
    message("Reading file ...")
    cdata <- read.table(con,comment.char = "#", colClasses=c("character","character","numeric"), check.names=FALSE)
    stopifnot(ncol(cdata)==3)

    id1 <- cdata[,1]
    id2 <- cdata[,2]

    pos1 <- match(id1, id(ygi))
    pos2 <- match(id2, id(xgi))      
    
    ## -1 is performed in the sparseMatrix function
    bigMat <- Matrix::sparseMatrix(i=pos1, j=pos2, x=cdata[,3], dims=c(length(ygi), length(xgi)), dimnames=list(id(ygi), id(xgi)))
    rm(cdata)
    
    message("Convert 'C' file in HTCexp object(s)")
    x <- splitCombinedContacts(bigMat, xgi, ygi, allPairwise, rm.trans, lazyload)
}##importC


###################################
## import.my5C
##
## Create a HTCexp object from my5C's data files (matrix data file).
## Intervals not define in the BED files are not taken into account
## If multiple chromosomes are available in the intervals files, several HTCexp object are created, one per chromosome pair.
##
## my5C.datafile: data file at the matrix format
##
##################################

import.my5C <- function(file, allPairwise=FALSE, rm.trans=FALSE, lazyload=FALSE){
    
    ## Read data
    stopifnot(!missing(file))
    message("Reading file ...")
    my5Cdata <- read.table(file,comment.char = "#", check.names=FALSE, header=TRUE, row.names=1)

    message("Convert my5C matrix file in HTCexp object(s)")
    my5CdataM <- as(as.matrix(my5Cdata),"Matrix")
    rownames(my5CdataM) <- rownames(my5Cdata)
    colnames(my5CdataM) <- colnames(my5Cdata)
    
    ## Create xgi and ygi object
    gr <- dimnames2gr(my5Cdata, pattern="\\||\\:|\\-", feat.names=c("name","org","chr","start", "end"))
    ygi <- gr[[1]]
    xgi <- gr[[2]]
    
    ## Create HTClist object from my5Cdata
    rownames(my5CdataM) <- id(ygi)
    colnames(my5CdataM) <- id(xgi)
    
    ## For multiple maps in one file
    if (length(seqlevels(xgi)) > 1 || length(seqlevels(ygi)) > 1){
      obj <- splitCombinedContacts(my5CdataM, xgi, ygi, allPairwise, rm.trans, lazyload)
    }else{
      obj <- HTClist(HTCexp(my5CdataM, xgi, ygi, lazyload = lazyload))
    }
    
    return(HTClist(unlist(obj[which(!unlist(lapply(obj, is.null)))])))
}##import.my5C



###################################
##
## INTERNAL FUNCTION
## Split my5C row/colnames matrix to create intervals objects
## Generalized function of my5C2gr - now deprecated
## 
## x: Matrix data
## pattern: regular expression to split the colnames/rownames of x
## feat.names: features names associated with the regular expression
##
##################################

dimnames2gr <- function(x, pattern="\\||\\:|\\-", feat.names=c("name","chr","start", "end")){
    rdata <- strsplit(rownames(x), split=pattern)
    dr <- do.call(rbind.data.frame, rdata)
    stopifnot(dim(dr)[2]==length(feat.names))

    colnames(dr)<-feat.names
    rgr <- GRanges(seqnames=dr$chr, ranges = IRanges(start=as.numeric(as.character(dr$start)), end=as.numeric(as.character(dr$end)), names=as.character(dr$name)))

    if (length(setdiff(colnames(x),rownames(x)))>0){
        cdata <- strsplit(colnames(x), pattern)
        cr <- do.call(rbind.data.frame, cdata)
        colnames(cr)<-feat.names
        cgr <- GRanges(seqnames=cr$chr, ranges = IRanges(start=as.numeric(as.character(cr$start)), end=as.numeric(as.character(cr$end)), names=as.character(cr$name)))
    }else{
        cgr <- rgr
    }
    list(rgr, cgr)
}##dimnames2gr

###################################
## splitCombinedContacts
## INTERNAL FUNCTION
## Split a genome-wide Matrix into HTClist
## Selection is done by ids from xgi and ygi objects
## 
## x: Matrix data
## xgi: GenomicRanges of x_intervals
## ygi: GenomicRanges of y_intervals
## allPairwise: see pair.chrom
## lazyload: see HTCexp
##
##################################

splitCombinedContacts <- function(x, xgi, ygi, allPairwise=TRUE, rm.trans=FALSE, lazyload=FALSE){
  chromPair <- pair.chrom(sortSeqlevels(c(seqlevels(xgi), seqlevels(ygi))), use.order = allPairwise, rm.trans=rm.trans)
  
    obj <- mclapply(chromPair, function(chr) {
        ygi.subset <- ygi[which(seqnames(ygi) == chr[1]),] 
        seqlevels(ygi.subset) <- as.character(unique(seqnames(ygi.subset)))
        xgi.subset <- xgi[which(seqnames(xgi) == chr[2]),]
        seqlevels(xgi.subset) <- as.character(unique(seqnames(xgi.subset)))
     
        if (length(xgi.subset) > 0 && length(ygi.subset) > 0) {
            message("Creating ", chr[2], "-", chr[1], " Contact Map ...")
            if (length(ygi.subset)==1 || length(xgi.subset)==1){
                intdata <- Matrix(x[id(ygi.subset), id(xgi.subset)], nrow=length(ygi.subset), ncol=length(xgi.subset))
            }else{
                intdata <- x[id(ygi.subset), id(xgi.subset)]
            }
            colnames(intdata) <- id(xgi.subset)
            rownames(intdata) <- id(ygi.subset)
            
            ##Put as NA rows/columns with only 0s
            ## back to matrix to speed up implementation ...
            #cl <- class(intdata)
            #intdata <- as.matrix(intdata)
            #intdata[which(rowSums(intdata, na.rm=TRUE)==0),] <- NA
            #intdata[,which(colSums(intdata, na.rm=TRUE)==0)] <- NA
            #intdata <- as(intdata, cl)

            HTCexp(intdata, xgi.subset, ygi.subset, lazyload = lazyload)
        }
    })
    ##obj
    HTClist(unlist(obj))
}##splitCombinedContacts
    
###################################
## pair.chrom
## INTERNAL FUNCTION
## Compute chromsome pairwise combinaison
##
## chrom: character i.e vector with chromosome names
## use.order: if TRUE, all the pairwise combinaison are returned (i.e. chr1chr2 AND chr2chr1)
##
##################################

pair.chrom <- function(chrom, use.order=TRUE, rm.cis=FALSE, rm.trans=FALSE){
    v <- unique(chrom)
    if (use.order)
      z <- unlist(sapply(1:length(v),function(i){paste(v[i], v)}))
    else
      z <- unlist(sapply(1:length(v),function(i){paste(v[i], v[i:length(v)])}))
    lz <- strsplit(z," ")
    names(lz) <- gsub(" ","",z)

    if (rm.cis){
      lz <- lz[which(sapply(lapply(lz, duplicated), sum)==0)]
    }
    if (rm.trans){
      lz <- lz[which(sapply(lapply(lz, duplicated), sum)!=0)]    }
    lz
}##pair.chrom

###########################################
##
## Directionality Index
##
############################################

directionalityIndex <- function(x, winup=2e6, windown=2e6){
    stopifnot(isBinned(x))
    xdata <- as.matrix(intdata(x))
    rx <- ranges(x_intervals(x))
    upranges <- IRanges(start(rx)-winup+1, start(rx))
    A <- rep(NA, length(upranges))
    
    dwranges <- IRanges(end(rx), end(rx)+windown-1)
    B <- rep(NA, length(upranges))

    overup <- findOverlaps(rx, upranges,  type="within")
    overdown <- findOverlaps(rx, dwranges,  type="within")

    ## For all bins
    for (i in 1:length(rx)){
        ## What are the interactions in the up/downstream windows
        idx <- which(subjectHits(overup)==i)
        if (length(idx)>0){
            idx.rx <- queryHits(overup)[idx]
            A[i] <- sum(xdata[idx.rx, i], na.rm=TRUE)
        }
        idx <- which(subjectHits(overdown)==i)
        if (length(idx)>0){
            idx.rx <- queryHits(overdown)[idx]
            B[i] <- sum(xdata[idx.rx, i], na.rm=TRUE)
        }
        #print(paste0(A[i], "-", B[i], "-", A[i]+B[i]/2))
        #if(i>=100){stop()}
    }

    E <- (A+B)/2
    DI <- ((B-A)/abs(B-A))*(((A-E)^2)/E + ((B-E)^2)/E)
    DI[which(is.na(DI))] <- 0
    DI
}

################
## New S4 class to represent list of HTCexp objects
################

## class definition
setClass("HTClist",
         prototype = prototype(elementType = "HTCexp"),
         contains = "list")


setValidity("HTClist",
            function(object){
                fails <- character(0)
                ## Check Chromosome Pairs Data
                 chrom <- sapply(object, function(x){
                     paste(seqlevels(x_intervals(x)), seqlevels(y_intervals(x)), sep="")
                 })
                 if (any(duplicated(chrom))){
                     fails <- c(fails, "Multiple 'HTCexp' found from the same chromosome pairs")
                 }

                ## Check Chromosome bounderies
                ##if (length(unique(unlist(ranges(object)))) > length(seqlevels(object))){
                ##    fails <- c(fails, "Same chromosome found with different ranges")
                ##}

                if (length(fails) > 0) return(fails)
                return(TRUE)
            })

## constructor
HTClist <- function(...)
{
  listData <- list(...)
  stopifnot(length(listData) > 0L)

  if (length(listData) == 1L && is.list(listData[[1L]]))
      listData <- listData[[1L]]
  if (!all(sapply(listData, is, "HTCexp")))
      stop("all elements in '...' must be HTCexp objects")
  listData <- unname(listData)

  names(listData) <- sapply(listData, function(x){
    paste(seqlevels(y_intervals(x)), seqlevels(x_intervals(x)), sep="")
  })

  new("HTClist", listData)
}


################
##
## Methods
##
################

setMethod("[", "HTClist",
    function(x, i, ...)
    {
        if (is.character(i)){
            i <- match( i, names(x))
          }
        HTClist(unlist(x)[i])
    })


setMethod("as.list", "HTClist",
    function(x)
    {
        as.list(unlist(x))
    })


setMethod("c", "HTClist", function(x, ...){
    if (missing(x))
        args <- unname(list(...))
    else{
        l <- list(...)
        l <- sapply(l, function(x){
            if (inherits(x,"HTClist"))
                x <- as.list(x)
            x
        })
        args <- unname(c(as.list(x), l))
    }
    HTClist(args)
})


setMethod("detail",signature(x="HTClist"),
          function(x){
              stopifnot(validObject(x))
              lapply(x, show)
              invisible(NULL)
          })


setMethod(f="forcePairwise", signature(x="HTClist"),
          function(x){
            ##stopifnot(isComplete(x))
            ## intra sym
            #if (length(x[isIntraChrom(x)])>0){
            #  x[isIntraChrom(x)] <- HTClist(lapply(x[isIntraChrom(x)], forcePairwise))
            #}
            ## inter maps
            if (!isPairwise(x)){
              chrs <- seqlevels(x)
              pchr <- pair.chrom(chrs, rm.cis=TRUE)
              isin <- rep(0, length(pchr))
              names(isin) <- names(pchr)
              isin[names(x)] <- 1

              ptoadd <- pchr[names(which(isin==0))]
              nmaps <- mclapply(ptoadd, function(obj){
                symobj <- x[[paste0(obj[2], obj[1])]]
                HTCexp(intdata=t(intdata(symobj)), xgi=y_intervals(symobj), ygi=x_intervals(symobj))
              })

              c(x, nmaps)
            }else{
              x
            }
          })

setMethod(f="forceSymmetric", signature(x="HTClist", uplo="missing"),
          function(x, uplo){forceSymmetric(x, uplo="U")})

setMethod(f="forceSymmetric", signature(x="HTClist", uplo="character"),
          function(x, uplo){
              stopifnot(uplo=="U" || uplo=="L")
              stopifnot(isComplete(x))
              ## cis
              x[isIntraChrom(x)] <- lapply(x[isIntraChrom(x)], forceSymmetric)
              ## trans
              x <- sortSeqlevels(x)
              chrs <- seqlevels(x)
              pch <- pair.chrom(chrs, use.order=FALSE)
              if (uplo=="L"){
                  pch <- lapply(pch, "[", 2:1)
                  names(pch) <- sapply(pch, paste0, collapse="")
              }
              x[names(pch)]
          })


setMethod(f="isPairwise", signature(x="HTClist"),
          function(x){
              chrs <- seqlevels(x)
              all.pairs <- names(pair.chrom(chrs, rm.cis=TRUE))
              ret <- FALSE
              if (length(setdiff(all.pairs, names(x)))==0)
                  ret <- TRUE
              ret
          })


setMethod(f="getCombinedIntervals", signature(x="HTClist"),
          function(x, merge=FALSE){

            misintra <- length(setdiff(paste0(seqlevels(x), seqlevels(x)), names(x)))
            ## If all intrachromosomal maps are available
            if (misintra == 0){
              x <- x[isIntraChrom(x)]
              ygr <- lapply(x, y_intervals)
              ygi <- suppressWarnings(do.call(c,unname(ygr)))

              if(length(which(!sapply(x, isSymmetric)))>0){
                xgr <- lapply(x, x_intervals)
                xgi <- suppressWarnings(do.call(c,unname(xgr)))
              }else{
                xgi <- ygi
              }
            }else{
              ygr <- lapply(x, y_intervals)
              ygi <- suppressWarnings(do.call(c,unname(ygr)))
              ygi <- unique(ygi)

              xgr <- lapply(x, x_intervals)
              xgi <- suppressWarnings(do.call(c,unname(xgr)))
              xgi <- unique(xgi)
            }
            if (merge){
              gr <- suppressWarnings(unique(c(xgi, ygi)))
              gr <- sortSeqlevels(gr)
              gr <- sort(gr)
            }else{
              gr <- list(ygi=ygi, xgi=xgi)
            }
            gr
          })


setMethod(f="getCombinedContacts", signature(x="HTClist"),
          function(x){

            message("Start combining HTCexp objects ...")
            pchr <- t(as.data.frame(pair.chrom(seqlevels(x))))
            gr <- getCombinedIntervals(x, merge=TRUE)
            ## Need to use both x and y intervals for symmetric data
            dimchr <- sapply(split(gr, seqnames(gr)), length)

            col.merged <- lapply(seqlevels(x), function(chr){
              allpairs <- pchr[which(pchr[,1]==chr),]
              ##do.call(cbind,lapply(x[allpairs], function(xx){as(intdata(xx), "sparseMatrix")}})
              do.call(cbind, apply(allpairs, 1, function(chrs){
                mapname <- paste0(chrs, collapse="")
                if (is.element(mapname, names(x))){
                    as(intdata(x[[mapname]]), "sparseMatrix")
                }else{
                  as(matrix(0, ncol=dimchr[chrs[2]], nrow=dimchr[chrs[1]]), "sparseMatrix")
                }
              }))
            })

            bigMat <- do.call(rbind, col.merged)
            colnames(bigMat) <- rownames(bigMat) <- names(gr)
            message("Object size: ",object.size(bigMat))
            bigMat
          })


setMethod(f="isComplete", signature(x="HTClist"),
          function(x){
              chrs <- seqlevels(x)
              lchrs <- length(chrs)
              ret <- TRUE

              if (lchrs != length(which(isIntraChrom(x)))){
                ret <- FALSE
              }else{
                mat.pairs <- matrix(0, ncol=lchrs, nrow=lchrs, dimnames=list(chrs, chrs))
                for (m in x){
                  mat.pairs[seqlevels(m@ygi), seqlevels(m@xgi)] <- 1
                }
                maps_per_chr <- apply(mat.pairs, 1, sum) + apply(mat.pairs, 2, sum)
                if (any(maps_per_chr<(length(seqlevels(x))+1)))
                  ret <- FALSE
              }
              ret
          })


setMethod(f="isBinned", signature(x="HTClist"),
          function(x){
              sapply(x, isBinned)
          })


setMethod(f="isIntraChrom", signature(x="HTClist"),
          function(x){
              sapply(x, isIntraChrom)
          })


setMethod(f="ranges", signature(x="HTClist"),
          function(x){
              GRangesList(sapply(x, range))
          })


setMethod(f="range", signature(x="HTClist"),
          function(x){
              reduce(unlist(ranges(x)))
        })


setMethod(f="reduce", signature(x="HTClist"),
          function(x, chr, cis=TRUE, trans=TRUE, extend=FALSE){
            pc <- as.matrix(data.frame(pair.chrom(seqlevels(x), rm.cis=!cis)))
            if (extend){
                sel <- colnames(pc)[union(which(pc[1,] %in% chr), which(pc[2,] %in% chr))]
                ##sel <- colnames(pc)[which(pc[1,] %in% chr | pc[2,] %in% chr)]
            }else{
                sel <- colnames(pc)[intersect(which(pc[1,] %in% chr), which(pc[2,] %in% chr))]
                ##sel <- colnames(pc)[which(pc[1,] %in% chr & pc[2,] %in% chr)]
            }
            xr <- x[intersect(names(x), sel)]
            if (!trans){
              xr <- xr[isIntraChrom(xr)]
            }
            xr
          })


setMethod(f="seqlevels", signature(x="HTClist"),
          function(x){
              unique(unlist(as.vector(sapply(x, seqlevels))))
          })


setMethod("show",signature="HTClist",
          function(object){
              stopifnot(validObject(object))
              cat("HTClist object of length",length(object),"\n")
              if (length(object)>0){
                im <- isIntraChrom(object)
                cat(length(which(im)),"intra /", length(which(!im)), "inter-chromosomal maps\n")
              }
              invisible(NULL)
          })


setMethod("sortSeqlevels", signature="HTClist",
          function(x){
              stopifnot(validObject(x))
              chrs <- seqlevels(x)
              chrs.sort <- sortSeqlevels(chrs)
              out <- x[intersect(names(pair.chrom(chrs.sort)), names(x))]
              out
          })


setMethod("summary", signature=c(object="HTClist"),
           function(object){
               sy <- as.data.frame(t(data.frame(lapply(object, summary),stringsAsFactors=FALSE)), stringsAsFactors=FALSE)
               rownames(sy) <- paste0(sy$seq1, sy$seq2)
               sy
           })

#####################
## Deprecated
#####################
#setMethod("forcePairwise", signature=c("HTCexp"), definition=function(x){
#    .Deprecated("forcePairwise")
#})


#####################
## Defunct
#####################

##seq_name
setMethod("seq_name", signature=c("HTCexp"), definition=function(x, ...){
    .Defunct("seqlevels")
})

## Normalized per zscore
setMethod("normPerZscore", signature=c("HTCexp"), definition=function(x, ...){
    .Defunct("normPerExpected")
})

## Export Method
setMethod("export", signature("HTCexp", "character"), function(object, con){
    .Defunct("exportC")
})

##################################
## binningC
##
## Windowing of 'C' data
##
## x = an object of class HTCexp
## binsize = the window size
## bin.adjust = logical, the window size is adjusted taking into account the genome size
## upa = unique primer assignement. If true, a primer is assign to only one window
## method = the method used to summarize the window information (mean, median, sum)
## use.zero = use zero value for the summarization
## step = overlap of window. step = 1 for no overlap window, step = 2 50% overlap, etc
###################################

binningC <- function(x, binsize=100000, bin.adjust=TRUE, upa=TRUE, method=c("sum", "median", "mean"), use.zero=TRUE, step=1, optimize.by=c("speed","memory")){

    stopifnot(inherits(x,"HTCexp"))
    met <- match.arg(method)
    optim <- match.arg(optimize.by)        
  
    ygi <- y_intervals(x)
    xgi <- x_intervals(x)

    ## Optimization
    mat.data <- intdata(x)
    if (optim=="speed")
        mat.data <- as.matrix(mat.data)

    rxi <- range(xgi, ignore.strand=TRUE)
    ryi <- range(ygi, ignore.strand=TRUE)

    ## For cis data - set the same ranges
    if (isIntraChrom(x)){
        xmin <- ymin <- start(range(c(xgi,ygi), ignore.strand=TRUE))
        xmax <- ymax <- end(range(c(xgi,ygi), ignore.strand=TRUE))
    }else{
        xmin <- start(rxi)
        xmax <- end(rxi)
        ymin <- start(ryi)
        ymax <- end(ryi)
    }

    if (max(xmax, ymax)<binsize){
        binsize=xmax
        warning("Data smaller than binsize. Cannot be binned at such resolution.")
    }
           
    if (bin.adjust){
        x.nb.bin <- floor((xmax - xmin)/binsize)
        y.nb.bin <- floor((ymax - ymin)/binsize)
        x.size.bin <- ceiling(binsize+((xmax-xmin+1)-(x.nb.bin*binsize))/x.nb.bin)
        y.size.bin <- ceiling(binsize+((ymax-ymin+1)-(y.nb.bin*binsize))/y.nb.bin)
    } else{ 
        x.size.bin=binsize
        y.size.bin=binsize
    }

    x.pas <- seq(from=xmin, to=xmax, by=floor(x.size.bin/step))
    x.pas <- x.pas[1:(length(x.pas)-(step-1))]
    y.pas <- seq(from=ymin, to=ymax, by=floor(y.size.bin/step))
    y.pas <- y.pas[1:(length(y.pas)-(step-1))]

    message("Bin size 'xgi' =",floor(x.size.bin/step)*step," [",step,"x",floor(x.size.bin/step),"]", sep="")
    message("Bin size 'ygi' =",floor(y.size.bin/step)*step," [",step,"x",floor(y.size.bin/step),"]", sep="")

    x.nb.bin <- length(x.pas)
    y.nb.bin <- length(y.pas)
    
    ## Unique Primer Assignment
    if (upa){
        start(xgi) <- end(xgi) <- start(xgi)+(width(xgi)/2)
        start(ygi) <- end(ygi) <- start(ygi)+(width(ygi)/2)
    }

    x.se.bin <- matrix(NA_integer_, ncol=2, nrow=x.nb.bin, byrow=TRUE)
    x.se.bin[,1] <- x.pas
    x.se.bin[,2] <- ifelse(x.pas+x.size.bin>xmax,xmax,x.pas+x.size.bin)
    x.se.bin[,2] <- x.se.bin[,2] - 1
    
    y.se.bin <- matrix(NA_integer_, ncol=2, nrow=y.nb.bin, byrow=TRUE)
    y.se.bin[,1] <- y.pas
    y.se.bin[,2] <- ifelse(y.pas+y.size.bin>ymax,ymax,y.pas+y.size.bin)
    y.se.bin[,2] <- y.se.bin[,2] - 1

    x.bin.set <- GRanges(seqnames=seqlevels(xgi), ranges = IRanges(start=x.se.bin[,1], end=x.se.bin[,2], names=paste(seqlevels(xgi),":",x.se.bin[,1],"-",x.se.bin[,2], sep="")))
    y.bin.set <- GRanges(seqnames=seqlevels(ygi), ranges = IRanges(start=y.se.bin[,1], end=y.se.bin[,2], names=paste(seqlevels(ygi),":",y.se.bin[,1],"-",y.se.bin[,2], sep="")))

    ## Binning for 5C data - Overlap with both xgi and ygi
    if (!isBinned(x)){
        xx.bin.over <- as.list(findOverlaps(x.bin.set, xgi))
        xy.bin.over <- as.list(findOverlaps(x.bin.set, ygi))
        yy.bin.over <- as.list(findOverlaps(y.bin.set, ygi))
        yx.bin.over <- as.list(findOverlaps(y.bin.set, xgi))
        
        ## vector of pairs to combine
        if (isIntraChrom(x)){
            p <- matrix(rep(1:y.nb.bin,2), ncol=2)
            p <- rbind(p, t(combn(1:y.nb.bin, 2)))
        }else{
            p <- matrix(c(rep(1:y.nb.bin,x.nb.bin), rep(1:x.nb.bin, each=y.nb.bin)), ncol=2)
        }
                
        out<- apply(p, 1, function(idx){
            fA <-yy.bin.over[[idx[1]]]
            rA <-yx.bin.over[[idx[1]]]
            fB <-xy.bin.over[[idx[2]]]
            rB <-xx.bin.over[[idx[2]]]
               
             if ((length(fA>0) || length(rA)>0) && (length(fB>0) || length(rB)>0)){
                 if (met=="sum"){
                     if (idx[1]==idx[2])
                         (sum(mat.data[fA,rB], na.rm=TRUE)+sum(mat.data[fB,rA], na.rm=TRUE))/2
                     else
                        sum(mat.data[fA,rB], na.rm=TRUE)+sum(mat.data[fB,rA], na.rm=TRUE)
                 }
                else if(met=="mean"){
                    sdata <- c(as.vector(mat.data[fA,rB]), as.vector(mat.data[fB,rA]))
                    if (!use.zero && length(sdata[sdata!=0]>0))
                        mean(sdata[sdata!=0], na.rm=TRUE)
                    else
                        mean(sdata, na.rm=TRUE)
                }
                else if (met=="median"){
                    sdata <- c(as.vector(mat.data[fA,rB]), as.vector(mat.data[fB,rA]))
                    if (!use.zero && length(sdata[sdata!=0]>0))
                        median(sdata[sdata!=0], na.rm=TRUE)
                    else
                        median(sdata, na.rm=TRUE)
                }
            }else{
                return(0)
            }
        })
        out <- unlist(out)
        ## Create new HTCexp object
        mat.bin <- Matrix::sparseMatrix(i=p[which(out!=0),1], j=p[which(out!=0),2], x=out[which(out!=0)], dims=c(y.nb.bin, x.nb.bin))
        
        if (isIntraChrom(x))
            mat.bin <- forceSymmetric(mat.bin)
        
    }else{
        ## Binning for Hi-C data
        if (isSymmetric(x)){
            xx.bin.over <- as.list(findOverlaps(x.bin.set, xgi))
            xy.bin.over <- xx.bin.over
            
            ## vector of pairs to combine
            if (y.nb.bin>1){
                p <- matrix(rep(1:y.nb.bin,2), ncol=2)
                p <- rbind(p, t(combn(1:y.nb.bin, 2)))
            }else{
                p <- matrix(c(1,1), ncol=2, byrow=2)
            }
        }else{
            xx.bin.over <- as.list(findOverlaps(x.bin.set, xgi))
            xy.bin.over <- as.list(findOverlaps(y.bin.set, ygi))
            
            ## vector of pairs to combine
            p <- matrix(c(rep(1:y.nb.bin,x.nb.bin), rep(1:x.nb.bin, each=y.nb.bin)), ncol=2)
        }
        ## sum of pairs of overlap
        out<- apply(p, 1, function(idx){
            i <-xy.bin.over[[idx[1]]]
            j <-xx.bin.over[[idx[2]]]

            if (met=="sum"){
                if (idx[1]==idx[2])
                    sum(mat.data[i,j], na.rm=TRUE) ##/2
                else
                    sum(mat.data[i,j], na.rm=TRUE)
            }
            else if(met=="mean"){
                sdata <- as.vector(mat.data[i,j])
                if (!use.zero && length(sdata[sdata!=0]>0))
                    mean(sdata[sdata!=0], na.rm=TRUE)
                else
                    mean(sdata, na.rm=TRUE)
            }
            else if (met=="median"){
                sdata <- as.vector(mat.data[i,j])
                if (!use.zero && length(sdata[sdata!=0]>0))
                    median(sdata[sdata!=0], na.rm=TRUE)
                else
                    median(sdata, na.rm=TRUE)
            }
        })
        ## Create new HTCexp object
        mat.bin <- Matrix::sparseMatrix(i=p[which(out!=0),1], j=p[which(out!=0),2], x=out[which(out!=0)], dims=c(y.nb.bin, x.nb.bin))

        if (isSymmetric(x))
            mat.bin <- forceSymmetric(mat.bin)
    }
    colnames(mat.bin) <- paste(seqlevels(xgi),":",x.se.bin[,1],"-",x.se.bin[,2], sep="")
    rownames(mat.bin) <- paste(seqlevels(ygi),":",y.se.bin[,1],"-",y.se.bin[,2], sep="")
  
    return(HTCexp(mat.bin, x.bin.set, y.bin.set))  
}

###################################
## setIntervalScale
##
## Force xgi and ygi intervals of a HTCexp object
##
## x = HTCexp object
## xgi = xgi Genome_interval object to use to define the HTCexp object
## ygi = ygi Genome_interval object to use to define the HTCexp object
## upa = unique primer assignement. If true, a primer is assign to only one window
## method = the method used to summarize the new interval information (mean, median, sum)
## use.zero = use zero value for the summarization
##
##################################

setIntervalScale <- function(x, xgi, ygi, upa=TRUE, method=c("sum", "median", "mean"), use.zero=TRUE, optimize.by=c("speed","memory")){
    
    stopifnot(inherits(x,"HTCexp"))
    met <- match.arg(method)
    optim <- match.arg(optimize.by)

    x.ygi <- y_intervals(x)
    x.xgi <- x_intervals(x)

    mat.data <- intdata(x)
    if (optim=="speed")
       mat.data <- as.matrix(mat.data)
    
    ## Unique Primer Assignment
    if (upa){
        start(ranges(x.xgi)) <- end(ranges(x.xgi)) <- start(ranges(x.xgi)) + floor(width(ranges(x.xgi)))/2
        start(ranges(x.ygi)) <- end(ranges(x.ygi)) <- start(ranges(x.ygi)) + floor(width(ranges(x.ygi)))/2
    }

    x.nb.bin <- length(xgi)
    y.nb.bin <- length(ygi)
    
    bin.over.y <- as.list(findOverlaps(ygi, x.ygi))
    bin.over.x <- as.list(findOverlaps(xgi, x.xgi))
  
    if (optim=="speed")
        mat.bin <- matrix(0, ncol=x.nb.bin, nrow=y.nb.bin)
    else
        mat.bin <- Matrix(0, ncol=x.nb.bin, nrow=y.nb.bin)

    colnames(mat.bin) <- id(xgi)
    rownames(mat.bin) <- id(ygi)
 
    for (i in 1:(x.nb.bin)){
        rA <-bin.over.x[[i]]
        for (j in 1:(y.nb.bin)){
            fB <-bin.over.y[[j]]
            if (length(rA)>0 && length(fB)>0){
                if (met=="sum"){
                    mat.bin[j,i] <- sum(mat.data[fB,rA], na.rm=TRUE)
                } else if(met=="mean"){
                    sdata <- c(mat.data[fB,rA])
                    if (!use.zero && length(sdata[sdata!=0]>0)){
                        mat.bin[j,i] <- mean(sdata[sdata!=0], na.rm=TRUE)
                    }
                    else {
                        mat.bin[j,i] <- mean(sdata, na.rm=TRUE)
                    }
                }
                else if (met=="median"){
                    sdata <- c(mat.data[fB,rA])
                    if (!use.zero && length(sdata[sdata!=0]>0)){
                        mat.bin[j,i] <- median(sdata[sdata!=0], na.rm=TRUE)
                    }
                    else {
                        mat.bin[j,i] <- median(sdata, na.rm=TRUE)
                    }
                }
            }
        }
    }

    if (optim=="speed")
        mat.bin <- as(mat.bin,"Matrix")
    
    return(HTCexp(mat.bin, xgi, ygi))
}

## Nicolas Servant
## HiTC BioConductor package
##**********************************************************************************************************##
##
## HIC Normalization procedure from Lieberman-Aiden et al. 2009
##
##**********************************************************************************************************##

## Normalized per expected number of count
setMethod("normPerExpected", signature=c("HTCexp"), definition=function(x, ...){
    
    expCounts <- getExpectedCounts(forceSymmetric(x), asList=TRUE, ...)
    if (! is.null(expCounts$stdev.estimate)){
        x@intdata <- (x@intdata-expCounts$exp.interaction)/expCounts$stdev.estimate
    }else{
        x@intdata <- x@intdata/(expCounts$exp.interaction) 
    }
    ## Remove NaN or Inf values for further analyses
    #x@intdata[which(is.na(x@intdata) | is.infinite(x@intdata))]<-NA
    x@intdata[Matrix::which(is.infinite(x@intdata))]<-0
    x
})

## Normalized per expected number of counts across all cis maps
setMethod("normPerExpected", signature=c("HTClist"), definition=function(x, ...){

  xintra <- x[isIntraChrom(x)]

  ## estimated expected counts for all cis maps
  exp <- lapply(xintra, function(xx){
      r <- getExpectedCounts(forceSymmetric(xx), method="mean", asList=TRUE, ...)
      r$exp.interaction
  })

  ## combined all cis expected counts
  N <- max(sapply(exp, dim))
  counts <- matrix(0, ncol=N, nrow=N)
  ss <- matrix(0, ncol=N, nrow=N)
  for (i in 1:length(exp)){
    n <- dim(exp[[i]])[1]
    counts[1:n, 1:n] <- counts[1:n, 1:n]+1
    ss[1:n, 1:n] <- ss[1:n, 1:n] + as.matrix(exp[[i]])
  }
 
  ## Mean over all expected matrices
  ss <- ss / counts
  
  xintranorm <- lapply(xintra, function(xx){
    n <- dim(xx@intdata)[1]
    xx@intdata <- xx@intdata/ss[1:n,1:n]
    xx@intdata[which(is.na(xx@intdata) | is.infinite(xx@intdata))]<-0
    xx
  })
  x[isIntraChrom(x)] <- xintranorm
  x
})

###################################
## getExpectedCountsMean
##
## This way of calculate expected counts was used in Naumova et al.
## The idea is just to look at all diagonals and to calculate their mean
##
## x = a HTCexp object
##
##
## NOTES
## Migth be interesting to add an isotonic regression on the mean to force the expected value to decrease with the distance
###################################


getExpectedCounts <- function(x, method=c("mean","loess"), asList=FALSE, ...){
  met <- match.arg(method)

  if (dim(intdata(x))[1]>500 & met=="loess"){
    warning("Contact map looks big. Use mean method instead or be sure that the loess fit gives good results.")
  }
  
  if (met=="mean"){
    ret <- getExpectedCountsMean(x, ...)
  }else if (met=="loess"){
    ret <- getExpectedCountsLoess(x, ...)
  }else{
    stop("Unknown method")
  }

  if (asList){
    return(ret)
  }else{
    intdata(x) <- ret$exp.interaction
    return(x)
  }
}


logbins<- function(from, to, step=1.05, N=NULL) {
  if (is.null(N)){
    unique(round(c(from, exp(seq(log(from), log(to), by=log(step))), to)))
  }else{
    unique(round(c(from, exp(seq(log(from), log(to), length.out=N)), to)))
  }
}

getExpectedCountsMean <- function(x, logbin=TRUE, step=1.05, filter.low=0.05){

  xdata <- intdata(x)
  N <- dim(xdata)[1]
  if (logbin){
    bins <- logbins(from=1,to=N, step=step)
    bins <- as.vector(Rle(values=bins, lengths=c(diff(bins),1)))
    stopifnot(length(bins)==N)
  }else{
    bins <- 1:N
  }

  message("Estimate expected using mean contact frequency per genomic distance ...")
  
  xdata <- as.matrix(xdata)
  if (filter.low > 0){
      rc <- colSums(xdata, na.rm=TRUE)
      ##rc <- which(rc==0)
      rc <- which(rc < ceiling(quantile(rc[which(rc>0)], probs=filter.low)))
      rr <- rowSums(xdata, na.rm=TRUE)
      ##rr <- which(rr==0)
      rr <- which(rr <  ceiling(quantile(rr[which(rr>0)], probs=filter.low)))
      
      ## rm line with only zeros
      xdata[rr,] <- NA
      xdata[,rc] <- NA
  }
  ## create an indicator for all diagonals in the matrix
  rows <- matrix(rep.int(bins, N), nrow=N)
  ##d <- rows - t(rows)

  d <- matrix(bins[1+abs(col(rows) - row(rows))],nrow=N) - 1
  d[lower.tri(d)] <- -d[lower.tri(d)]
 
  if (isSymmetric(xdata)){
    ## remove half of the matrix
    d[lower.tri(d)] <- NA
  }
  ## use split to group on these values
  mi <- split(xdata, d)
  milen <- lapply(mi, length)
  mimean <- lapply(mi, mean, na.rm=TRUE)
  miexp <- lapply(1:length(milen), function(i){rep(mimean[[i]], milen[[i]])})
  names(miexp) <- names(mi)
  expmat <- as(matrix(unsplit(miexp, d), nrow=nrow(xdata), ncol=ncol(xdata)), "Matrix")
  if (isSymmetric(xdata)){
    expmat <- forceSymmetric(expmat, uplo="U")
  }
  
  colnames(expmat) <- colnames(xdata)
  rownames(expmat) <- rownames(xdata)

  ## Put NA at rc and cc
  if (filter.low > 0){
      expmat[rr,] <- NA
      expmat[,rc] <- NA
  }
  return(list(exp.interaction=expmat, stdev.estimate=NULL))
}


###################################
## getExpectedCounts
##
## Estimate the expected interaction counts from a HTCexp object based on the interaction distances
##
## x = a HTCexp object
## span=fraction of the data used for smoothing at each x point. The larger the f value, the smoother the fit. 
## bin=interval size (in units corresponding to x). If lowess estimates at two x values within delta of one another, it fits any points between them by linear interpolation. The default is 1% of the range of x. If delta=0 all but identical x values are estimated independently.
## stdev = logical,the standard deviation is estimated for each interpolation point
## plot = logical, display lowess and variance smoothing
##
## bin is used to speed up computation: instead of computing the local polynomial fit at each data point it is not computed for points within delta of the last computed point, and linear interpolation is used to fill in the fitted values for the skipped points. 
## This function may be slow for large numbers of points. Increasing bin should speed things up, as will decreasing span. 
## Lowess uses robust locally linear fits. A window, dependent on f, is placed about each x value; points that are inside the window are weighted so that nearby points get the most weight. 
##
## NOTES
## All variances are calculated (even identical values) because of the parallel implementation.
## Cannot use rle object because the neighboring have to be calculated on the wall dataset. Or have to find a way to convert rle index to real index ...
## Easy to do with a 'for' loop but the parallel advantages are much bigger
###################################

getExpectedCountsLoess<- function(x, span=0.01, bin=0.005, stdev=FALSE, plot=FALSE){
    stopifnot(inherits(x,"HTCexp"))

    xdata <- as.matrix(intdata(x))
    rc <- which(colSums(xdata, na.rm=TRUE)==0)
    rr <- which(rowSums(xdata, na.rm=TRUE)==0)

    ## rm line with only zeros
    xdata[rr,] <- NA
    xdata[,rc] <- NA
      
    ydata <- as.vector(xdata)
    ydata[which(is.na(ydata))] <- 0
    xdata.dist <- as.vector(intervalsDist(x))
    o<- order(xdata.dist)
    xdata.dist <- xdata.dist[o]
    ydata <- ydata[o]
    
    delta <- bin*diff(range(xdata.dist))
    ######################
    ## Lowess Fit
    ######################
    message("Lowess fit ...")
    #lowess.fit <- .C("lowess", x = as.double(xdata.dist), as.double(ydata), 
    #                length(ydata), as.double(span), as.integer(3), as.double(delta), 
    #                y = double(length(ydata)), double(length(ydata)), double(length(ydata)), PACKAGE = "stats")$y

    lowess.fit <-lowess(x=xdata.dist, y=ydata, f=span, delta=delta)$y
    
    y1 <- sort(ydata)
    y1 <-  quantile(y1[which(y1>1)], probs=0.99)

    if (plot){
        par(font.lab=2, mar=c(4,4,1,1))

        ##plotIntraDist(ydata, xdata.dist, xlab="Genomic Distance (bp)",  ylim=c(0,y1), ylab="Counts", main="", cex=0.5, cex.lab=0.7, pch=20, cex.axis=0.7, col="gray", frame=FALSE)
        
        plot(x=xdata.dist, y=ydata,  xlab="Genomic Distance (bp)",  ylim=c(0,y1), ylab="Counts", main="", cex=0.5, cex.lab=0.7, pch=20, cex.axis=0.7, col="gray", frame=FALSE)
        points(x=xdata.dist[order(lowess.fit)], y=sort(lowess.fit), type="l", col="red")
    }
    lowess.mat <- Matrix(lowess.fit[order(o)], nrow=length(y_intervals(x)), byrow=FALSE)
    rownames(lowess.mat) <- id(y_intervals(x))
    colnames(lowess.mat) <- id(x_intervals(x))

    ######################
    ## Variance estimation
    ######################
    stdev.mat <- NULL
    if (stdev){
        message("Standard deviation calculation ...")
        ##interpolation
        ind <- getDeltaRange(delta, xdata.dist)
        lx <- length(xdata.dist)
        Q <- floor(lx*span)
        stdev.delta <- unlist(mclapply(1:length(ind), function(k){
            i <- ind[k]
            x1 <- xdata.dist[i]
            
            ## Neighbors selection 2*Q
            ll <- i-Q-1
            lr <- i+Q-1
            if (ll<0) ll=0
            if (lr>lx) lr=lx
            xdata.dist.sub <- xdata.dist[ll:lr]
            ydata.sub <- ydata[ll:lr]
            ## Select the Q closest distances
            d <- abs(x1-xdata.dist.sub)
            o2 <- order(d)[1:Q]
            x2 <- xdata.dist.sub[o2]
            y2 <- ydata.sub[o2]
            ## Distance between x and other points
            dref <- d[o2] 
            drefs <- dref/max(abs(dref-x1)) ##max(dref) - NS
            ## Tricube weigths and stdev calculation
            w <- tricube(drefs)
            sqrt <- w*(y2-lowess.fit[i])^2
            
            stdev <- sqrt(sum(sqrt)/
                          (((length(sqrt)-1) * sum(w))/length(sqrt)))
        }))

        if (plot){
            points(x=xdata.dist[ind], y=lowess.fit[ind], col="black", cex=.8, pch="+")
            legend(x="topright", lty=c(1,NA), pch=c(NA,"+"), col=c("red","black"),legend=c("Lowess fit","Interpolation points"), cex=.8, bty="n")
        }
        
        ## Approximation according to delta
        stdev.estimate <- approx(x=xdata.dist[ind], y=stdev.delta, method="linear", xout=xdata.dist)$y
        stdev.mat <- matrix(stdev.estimate[order(o)], nrow=length(y_intervals(x)), byrow=FALSE)
        rownames(stdev.mat) <- id(y_intervals(x))
        colnames(stdev.mat) <- id(x_intervals(x))
    }
    
    ## Put NA at rc and cc
    lowess.mat[rr,] <- NA
    lowess.mat[,rc] <- NA
 
    return(list(exp.interaction=lowess.mat,stdev.estimate=stdev.mat))
}
    
###################################
## getDeltaRange
## INTERNAL FUNCTION
## Calculate the interpolation points from the delta value
##
## delta = lowess delta parameter
## xdata = Intervals distances matrix
###################################

getDeltaRange <- function(delta, xdata){
    message("Delta=",delta)
    
    if (delta>0){
        ind <- 1
        for (i in 1:length(xdata)-1){
            if (xdata[i+1]>=delta){
                ind <- c(ind,i)
                delta=delta+delta
            }
        }
        if (max(ind)<length(xdata)){
            ind <- c(ind, length(xdata))
        }
        message("Calculating stdev ... ")
    }else{
        ind <- 1:length(xdata)
    }
    ind
}

###################################
## tricube
## INTERNAL FUNCTION
## tricube distance weigth
##
## x = numeric. A distance
###################################

tricube <- function(x) {
    ifelse (abs(x) < 1, (1 - (abs(x))^3)^3, 0)
}



##**********************************************************************************************************##
##
## HIC Normalization procedure from HiCNorm package, Hu et al. 2012
##
##**********************************************************************************************************##

###################################
## normLGF
## Local Genomic Features normalization
## 
##
## x = HTCexp/HTClist object
## family = regression model Poisson or Neg Binon
##
##
##################################

normLGF <- function(x,  family=c("poisson", "nb")){
    family <- match.arg(family)
    message("Starting LGF normalization on ", seqlevels(x), " ...")
    counts <- intdata(x)
    
    ## Remove rowCounts=0 & colCounts=0
    rc <- which(rowSums(counts)>0)

    ## Intrachromosomal maps
    if (isIntraChrom(x)){
        cc <- rc
        stopifnot(length(rc)>0)
        counts.rc <- counts[rc,rc]

        elt <- elementMetadata(y_intervals(x)[rc])
        len <- elt$len
        gcc <- elt$GC
        map <- elt$map
        
        if(all(is.na(len)) || all(is.na(gcc)) || all(is.na(map)))
            stop("Genomic features are missing. Effective fragments length, GC content and mappability are required.")
        
        ##get cov matrix
        len_m<-as.matrix(log(1+len%o%len))
        gcc_m<-as.matrix(log(1+gcc%o%gcc))
        
        ##error for regions with 0 mappability
        map[which(map==0)] <- 10e-4
        map_m<-as.matrix(log(map%o%map))      

    }else{

    ## Interchromosomal maps
        cc <- which(colSums(counts)>0)
        stopifnot(length(rc)>0 & length(cc)>0)
        
        counts.rc <- counts[rc,cc]
        yelt <- elementMetadata(y_intervals(x)[rc])
        xelt <- elementMetadata(x_intervals(x)[cc])
        
        ylen <- yelt$len
        xlen <- xelt$len
        ygcc <- yelt$GC
        xgcc <- xelt$GC
        ymap <- yelt$map
        xmap <- xelt$map

        if(all(is.na(ylen)) || all(is.na(ygcc)) || all(is.na(ymap)) || all(is.na(xlen)) || all(is.na(xgcc)) || all(is.na(xmap)))
            stop("Genomic features are missing. Effective fragments length, GC content and mappability are required.")
      
        ##get cov matrix
        len_m<-as.matrix(log(1+ylen%o%xlen))
        gcc_m<-as.matrix(log(1+ygcc%o%xgcc))
        
        ##error for regions with 0 mappability
        ymap[which(ymap==0)] <- 10e-4
        xmap[which(xmap==0)] <- 10e-4
        map_m<-as.matrix(log(ymap%o%xmap))      
    }
    
    ##centralize cov matrix of enz, gcc
    #len_m<-(len_m-mean(len_m, na.rm=TRUE))/apply(len_m, 2, sd, na.rm=TRUE)
    #gcc_m<-(gcc_m-mean(gcc_m, na.rm=TRUE))/apply(gcc_m, 2, sd, na.rm=TRUE)
    #Fix bug in BioC [bioc] A: normLGF yields non-symetric matrices
    len_m<-(len_m-mean(len_m, na.rm=TRUE))/sd(len_m, na.rm=TRUE)
    gcc_m<-(gcc_m-mean(gcc_m, na.rm=TRUE))/sd(gcc_m, na.rm=TRUE)

    
    ##change matrix into vector
    if (isIntraChrom(x)){
        counts_vec<-counts.rc[which(upper.tri(counts.rc,diag=FALSE))]
        len_vec<-len_m[upper.tri(len_m,diag=FALSE)]
        gcc_vec<-gcc_m[upper.tri(gcc_m,diag=FALSE)]
        map_vec<-map_m[upper.tri(map_m,diag=FALSE)]
    }else{
        counts_vec<-as.vector(counts.rc)
        len_vec<-as.vector(len_m)
        gcc_vec<-as.vector(gcc_m)
        map_vec<-as.vector(map_m)
    }
    
    print("fit ...")
    if (family=="poisson"){
        ##fit Poisson regression: u~len+gcc+offset(map)
        fit<-glm(counts_vec~ len_vec+gcc_vec+offset(map_vec),family="poisson")
        ##fit<-bigglm(counts_vec~len_vec+gcc_vec+offset(map_vec),family="poisson", data=cbind(counts_vec, len_vec, gcc_vec, map_vec))
    }else{
        fit<-glm.nb(counts_vec~len_vec+gcc_vec+offset(map_vec))
    }

    coeff<-fit$coeff

    ## The corrected values (residuals) can be seen as a observed/expected correction.
    ## So I will compare the normalized counts with one: the observed count is higher or lower than the expected count. We may not want to compare the range of the normalized count with the range of the raw count. They have different interpretations.
    counts.cor<-round(counts.rc/exp(coeff[1]+coeff[2]*len_m+coeff[3]*gcc_m+map_m), 4)
    counts[rownames(counts.rc), colnames(counts.rc)]<-counts.cor
  
    intdata(x) <- counts
    return(x)
}##normLGF

###################################
## setGenomicFeatures
## Annotate a HTCexp or HTClist object with the GC content and the mappability features
## 
##
## x = HTCexp/HTClist object
## cutSites = GRanges object ir GRangesList from getAnnotatedRestrictionSites function
## minFragMap = Discard restriction with mappability lower the this threshold (and NA)
## effFragLen = Effective fragment length
##################################

setGenomicFeatures <- function(x, cutSites, minFragMap=.5, effFragLen=1000){
    stopifnot(inherits(x,"HTCexp"))
    stopifnot(seqlevels(x) %in% seqlevels(cutSites))
    obj <- x
    xgi <- x_intervals(x)
    message("Annotation of ", seqlevels(x), " ...")
    xgi <- annotateIntervals(xgi, cutSites[[seqlevels(xgi)]], minfragmap=minFragMap, efffraglen=effFragLen)
    x_intervals(obj) <- xgi
    if (isIntraChrom(x) & isBinned(x)){
        y_intervals(obj) <- xgi
    }else{
        ygi <- y_intervals(x)
        ygi <- annotateIntervals(ygi, cutSites[[seqlevels(ygi)]], minfragmap=minFragMap, efffraglen=effFragLen)
        y_intervals(obj) <- ygi
    }
   obj
}

###################################
## annotateIntervals
## INTERNAL FUNCTION
## 
##
## gi = GRanges object from x_intervals or y_intervals methods
## annot = GRanges object from getAnnotatedRestrictionSites function
##
##################################

annotateIntervals <- function(gi, annot, minfragmap=.5, efffraglen=1000){
    ## Preprocess, keep fragments ends with mappability score larger than .5.
    ## Depends on the data processing (see Yaffe and Tanay, 2011). These fragments will be exclude from the analysis.
    if (!is.na(minfragmap) & !all(is.na(annot$map_U)) & !all(is.na(annot$map_D))){
        idxmap <- which(annot$map_U<minfragmap | is.na(annot$map_U))
        elementMetadata(annot)[idxmap,c("len_U", "GC_U", "map_U")]<-NA_real_
        idxmap <- which(annot$map_D<minfragmap | is.na(annot$map_D))
        elementMetadata(annot)[idxmap,c("len_D", "GC_D", "map_D")]<-NA_real_
    }

    ## Get all restriction sites which overlap with the bins
    ## Split upstream and downstream bins to deal with restriction sites which overlap the start or end of a fragment
    annot_up <- annot
    end(annot_up)<-start(annot)
    elementMetadata(annot_up) <- NULL
    annot_up$len=as.numeric(annot$len_U)
    annot_up$GC=as.numeric(annot$GC_U)
    annot_up$map=as.numeric(annot$map_U)
    annot_down <- annot
    start(annot_down) <- end(annot)
    elementMetadata(annot_down) <- NULL
    annot_down$len=as.numeric(annot$len_D)
    annot_down$GC=as.numeric(annot$GC_D)
    annot_down$map=as.numeric(annot$map_D)

    outl_up<- as.list(findOverlaps(gi, annot_up))
    outl_dw<- as.list(findOverlaps(gi, annot_down))
  
    annotscores <- lapply(1:length(outl_up), function(i){
        id_up <- outl_up[[i]]
        id_dw <- outl_dw[[i]]
        ##temp <-  c(annot_up[id_up], annot_down[id_dw])
        temp_up <- annot_up[id_up]
        temp_dw <- annot_down[id_dw]

        ## len - effective length" is the fragment length truncated by 1000 bp, which is the number of bases with specific ligation.
        ## In Yaffe & Tanay's paper Figure 1b, they define specific ligation as sum of distance to cutter sites (d1+d2) <= 500 bp. Such criterion implies that d1<=500 bp and d2 <= 500 bp. So for each fragment end, only reads mapped within 500 bp to cutter sites are used for downstream analysis. 
        lenv <- unique(c(temp_up$len, temp_dw$len))
        if (!is.na(efffraglen))
            lenscore <- sum(lenv>efffraglen, na.rm=TRUE)*efffraglen + sum(lenv[lenv<efffraglen], na.rm=TRUE)
        else
            lenscore <- sum(lenv, na.rm=TRUE)
        
        ##GC
        gcscore <- mean(c(temp_up$GC, temp_dw$GC), na.rm=TRUE)

        ##map
        mapscore <- mean(c(temp_up$map, temp_dw$map), na.rm=TRUE)

        c(lenscore, gcscore, mapscore)
    })
    annotscores <- matrix(unlist(annotscores), ncol=3, byrow=TRUE)
    colnames(annotscores) <- c("len", "GC", "map")
    elementMetadata(gi)$len <- round(annotscores[,"len"],3)
    elementMetadata(gi)$GC <- round(annotscores[,"GC"],3)
    elementMetadata(gi)$map <- round(annotscores[,"map"],3)

    gi
}


###################################
## getAnnotatedRestrictionFragments
## Return the restriction fragments for a given enzyme, annotated with the GC content and the mappability
## 
##
## resSite = Cutting site of the restriction enzyme used (default HindIII)
## overhangs5 =  Cleavage 5 overhang
## chromosome = chromosomes list to focus on. If NULL, all genome chromosome are investigated
## genomePack = name of the BSgenome package to load
## w = size of the downstream/upstream window to use around the restriction site to calculate the GC content. Default is 200. See Yaffe and Tanay for more details
## mappability = GRanges object of the mappability (see the ENCODE mappability tracks)
##
## D = downstream / U = upstream the restriction site
##################################

getAnnotatedRestrictionSites <- function(resSite="AAGCTT", overhangs5=1, chromosomes=NULL, genomePack="BSgenome.Mmusculus.UCSC.mm9",  mappability=NULL, wingc=200, winmap=500){

    if(genomePack %in% loadedNamespaces()==FALSE){
        stopifnot(require(genomePack, character.only=TRUE))
    }
    genome <- eval(as.name(genomePack))

    if (is.null(chromosomes)){
        chromosomes <- seqlevels(genome)
    }
    
    genomeCutSites <- mclapply(chromosomes, function(chr){
        message("Get restriction sites for ", chr, " ...")
        cutSites <- getRestrictionSitesPerChromosome(resSite, overhangs5, genome, chr)
        message(length(cutSites), " sites")
        
        message("Calculate fragment length ...")
        ## Add chromosome start/end
        len_D <- c(end(cutSites)[-1], length(genome[[chr]])) - start(cutSites)
        len_U <- end(cutSites) - c(0, start(cutSites)[-length(cutSites)])
        cutSites$len_U <- len_U
        cutSites$len_D <- len_D

        message("Calculate GC content ...")
        ## Upstream GC content
        win <- start(cutSites)-wingc
        win[win<0] <- 1
        seq <- Biostrings::getSeq(genome, chr, start=win, end=start(cutSites)-1)
        ##cutSites$seq_U <- seq
        cutSites$GC_U<- round(Biostrings::letterFrequency(seq, as.prob=FALSE, letters="CG")/Biostrings::letterFrequency(seq, as.prob=FALSE, letters="ACGT"),3)
        
        ## Downstream GC content
        win <- start(cutSites)+wingc-1
        win[win>length(genome[[chr]])] <- length(genome[[chr]])
        seq <- Biostrings::getSeq(genome, chr, start(cutSites), win)
        cutSites$GC_D<- round(Biostrings::letterFrequency(seq, as.prob=FALSE, letters="CG")/Biostrings::letterFrequency(seq, as.prob=FALSE, letters="ACGT"),3)
        ##cutSites$seq_D <- seq

        if (!is.null(mappability)){
            message("Calculate mappability ...")
            stopifnot(inherits(mappability,"GRanges"))
            
            mappability <- mappability[seqnames(mappability)==chr]
            win <- start(cutSites)-winmap+1
            win[win<0] <- 1
            gr <- GRanges(seqnames = chr, ranges = IRanges(start=win, end=start(cutSites)))
            overl <- as.list(findOverlaps(gr, mappability))
            mscore <- mappability$score
            cutSites$map_U<- unlist(lapply(overl, function(idx){
                round(mean(mscore[idx], na.rm=TRUE),3)
            }))
        
            win <- start(cutSites)+winmap
            win[win>length(genome[[chr]])] <- length(genome[[chr]])
            gr <- GRanges(seqnames = chr, ranges = IRanges(start=start(cutSites)+1, end=win))
            overl <- as.list(findOverlaps(gr, mappability))
            mscore <- mappability$score
            cutSites$map_D<- unlist(lapply(overl, function(idx){
                round(mean(mscore[idx], na.rm=TRUE),3)
            }))
        }else{
            cutSites$map_U<-NA_real_
            cutSites$map_D<-NA_real_
        }
        message("done ...")
        cutSites
    })
    grl <- GRangesList(genomeCutSites)
    names(grl) <- chromosomes
    grl
}


###################################
## getRestrictionSitesPerChromosome
## INTERNAL FUNCTION
## 
##
## resSite = Cutting site of the restriction enzyme used
## overhangs5 =  Cleavage 5 overhang
## genome = BSgenome object of the reference genome
## chromosome = chromosome to focus on
##
##################################

getRestrictionSitesPerChromosome <- function(resSite, overhangs5, genome, chromosome){

    stopifnot(inherits(genome,"BSgenome"))
    stopifnot(length(chromosome)==1)
    
    restrictionSites<-Biostrings::matchPattern(resSite, genome[[chromosome]])
 
    ## Deal with restriction enzyme 5' overhangs
    s <- start(restrictionSites) + overhangs5
    e <- end(restrictionSites) - overhangs5

    ir <- IRanges(start=s, end=e)    
    restrictionSites <- GRanges(seqnames = chromosome, ranges = ir, strand = "*")
    return(restrictionSites)
}

###################################
## getRestrictionFragmentsPerChromosome
## 
##
## resSite = Cutting site of the restriction enzyme used
## overhangs5 =  Cleavage 5 overhang
## genome = BSgenome object of the reference genome
## chromosome = chromosome to focus on
##
##################################

getRestrictionFragmentsPerChromosome <- function(resSite="AAGCTT", overhangs5=1,
chromosomes=NULL, genomePack="BSgenome.Mmusculus.UCSC.mm9"){
    
    if(genomePack %in% loadedNamespaces()==FALSE){
        stopifnot(require(genomePack, character.only=TRUE))
    }
    genome <- eval(as.name(genomePack))
    stopifnot(inherits(genome,"BSgenome"))

    if (is.null(chromosomes)){
        chromosomes <- seqlevels(genome)
    }
  
    genomeResFrag <- mclapply(chromosomes, function(chromosome){
        message("Get restriction fragments for ", chromosome, " ...")
        restrictionSites<-getRestrictionSitesPerChromosome(resSite, overhangs5, genome, chromosome)        
        restrictionFrag <- GRanges(seqnames=chromosome, ranges=IRanges(
                                                         start=c(1,start(restrictionSites)),
                                                         end=c(start(restrictionSites)-1, seqlengths(genome)[chromosome])), strand="+")
    })
    return(genomeResFrag)
}


##**********************************************************************************************************##
##
## ICE Normalization procedure from Imakaev et al .2012
##
##**********************************************************************************************************##

###################################
## balancingSK
## INTERNAL FUNCTION
##
## Matrix balancing used in ICE normalization
## Based on the Sinkhorn-Knopp algorithm
##
## x = HTCexp object
## max_iter = maximum number of iteration to converge
## eps = threshold to converge
##
##################################

balancingSK<- function(x, max_iter=50, eps=1e-4){
    m <- dim(x)[1]

    ## Initialization    
    sum_ss <- matrix(rep(0, m), ncol=1)
    bias <- matrix(rep(1, m), ncol=1)
    old_dbias <- NULL
    ## Remove Diagonal ?
    
    for (it in 1:max_iter){
        message("it=",it," ", Sys.time())

        ## 1- calculate sum of W over all rows ++
        sum_ds <- rowSums(x, na.rm=TRUE)
        ##sum_ds <- sqrt(rowSums(x^2))

        ## 2- Calculate a vector of corrected ss reads
        ## NOT DONE
        
        ## 3- Calculate vector of bias
        dbias <- as.matrix(sum_ds, ncol=1) + sum_ss

        ## 4 - Renormalize bias by its mean valude over non-zero bins to avoid numerical instabilities
        dbias <- dbias/mean(dbias[dbias!=0])

        ## 5- Set zero values of bias to 1 to avoir 0/0 error
        dbias[dbias==0] <- 1
               
        ## 6- Divide W by bias BiBj for all (i,j) ++++
        x <- x/(dbias %*% t(dbias))

        ## 7- Multiple total vector of bias by additional biases
        ##bias <- bias * dbias

        if (!is.null(old_dbias) && sum(abs(old_dbias - dbias))<eps){
            message("Break at iteration ", it)
            break
        }
        old_dbias <- dbias 
    }
    if (it == max_iter){
        message("Did not converged. Stop at iteration ",max_iter)
    }else{
        message("Converge in ",it," iteration")
    }
    return(x)
}


###################################
## IterativeCorNormalization
## ICE normlization
## 
##
## x = HTCexp object or HTClist object
## max_iter = maximum number of iteration to converge
## eps = threshold to converge
## spars.filter = Percentage of row and column to discard based on sparsity (default=0.02)
##
##################################

normICE <- function(x, max_iter=50, eps=1e-4, sparse.filter=0.02){

    if (inherits(x, "HTCexp")){
        stopifnot(isSymmetric(x))
        idata <- intdata(x)
        gr <- y_intervals(x)
    }else if (inherits(x, "HTClist")){
        idata <- getCombinedContacts(x)
        gr <- getCombinedIntervals(x)
    }

    if (!is.na(sparse.filter)){
        message("Start filtering  ...", Sys.time())
        spars <- apply(idata, 1, function(x){length(which(x==0))}) 
        spars.t <- quantile(spars[spars!=dim(idata)[1]], probs=(1-sparse.filter))
        idx <- which(spars>as.numeric(spars.t))
        idata[idx,] <- 0
        idata[,idx] <- 0
        message("Filter out ",length(idx)," rows and columns ...")
    }
    
    message("Start Iterative Correction ...")
    xmat <- balancingSK(idata, max_iter=max_iter, eps=eps)
    
    if (inherits(x, "HTCexp")){
        intdata(x) <- xmat
    }else if (inherits(x, "HTClist")){
        ##     gr <- dimnames2gr(xmat, pattern="\\||\\:|\\-", feat.names=c("name","chr","start", "end"))
        ##     xgi <- gr[[1]]
        ##     ygi <- gr[[2]]
        ##     rownames(xmat) <- id(ygi)
        ##     colnames(xmat) <- id(xgi)
        if (is.null(gr$xgi))
            x <- splitCombinedContacts(xmat, xgi=gr$ygi, ygi=gr$ygi)
        else
            x <- splitCombinedContacts(xmat, xgi=gr$xgi, ygi=gr$ygi)
    }
    x
}

###################################
## primersDist
##
## Calculate the distances between primers/intervals for cis data only
##
## x = an object of class HTCexp.
##
##
###################################

intervalsDist<-function(x, use.zero=TRUE){

    stopifnot(inherits(x,"HTCexp"))
    if (! isIntraChrom(x)){
       stop("The distances can only be calculated for intrachromosomal interactions")
    }

    xgi <- x_intervals(x)
    ygi <- y_intervals(x)
    xdata <- intdata(x)
    
    dist.mat <- apply(cbind(as.matrix(ranges(xgi)), id(xgi)), 1, function(xi, yi){
        sxi <- as.numeric(xi[1])
        exi <- sxi+as.numeric(xi[2])-1
        nxi <- xi[3]
        
        mxi <- (exi-sxi)/2L
        sygi <- start(yi)
        eygi <- end(yi)

        ##over <- yi %over% IRanges(start=xi[1], end=xi[2]) ## !!
        over <- (exi>=sygi & exi<=eygi) | (sxi>=sygi & sxi<eygi)
        id.over <- which(over)
        id.nover <- which(!over)
                
        d <- rep(NA_integer_, length(ygi))
        if (length(id.nover)>0){
            aft <- abs(eygi[id.nover]-sxi)
            bef <- abs(sygi[id.nover]- exi)
            di <- bef
            di[which(bef>aft)] <- aft[which(bef>aft)]
            d[id.nover] <- di
            ##d[id.nover] <- apply(cbind(abs(eygi[id.nover]-sxi), abs(sygi[id.nover]- exi)), 1, min) ## !!
        }
        if (length(id.over)>0){
            myi <- (eygi[id.over] - sygi[id.over])/2
            d[id.over] <- abs(myi - mxi)
        }
        d
    }, yi=ranges(ygi))

    dist.mat <- as(as.matrix(dist.mat), "Matrix")
    rownames(dist.mat) <- id(ygi)
    colnames(dist.mat) <- id(xgi)

    ## report only non-zero values
    if (!use.zero){
        dist.mat <- dist.mat + 1 ## zero distance are different from zero values
        sl.xdata <- xdata!=0
        dist.mat[which(as.matrix(!sl.xdata))] <- 0
        dist.mat <- as(dist.mat, "dgTMatrix")
    }
    dist.mat
}##intervalsDist

## Cis/Trans ratio
plotInterIntraRatio <- function(xdata.intra, xdata.inter, ...){
    sintra <- sum(xdata.intra, na.rm=TRUE)
    sinter <- sum(xdata.inter, na.rm=TRUE)
    sall <- sintra+sinter
    
    barplot(c(sintra/sall, sinter/sall), ylab="Fraction of Reads", names.arg=c("Intra","Inter"),main="Intra/Inter Chromosomal Interaction", ...)
}

## Scatter plot of interaction distance for intrachromosomal interactions
slidingWindow <- function(mydata=consV, win=c(1, 100), start=1, end=length(consV)) {
        mydata <- mydata[start:end]
        windex <- t(sapply(0:length(mydata), function(x) win+x))
        mywind <- sapply(seq(along=mydata), function(x) mean(mydata[windex[x,1]:windex[x,2]],
        na.rm=TRUE))
        mywind <- mywind[1:(length(mywind)-win[2])]
        return(mywind)
}


plotIntraDist <- function(xdata.intra, xdata.intra.dist, trim.range=.98, winsize=NA, add=FALSE, log=FALSE, fit=FALSE, fit.lim=NA, fit.out=1, ...){
    stopifnot(length(xdata.intra)==length(xdata.intra.dist))
    r <- range(xdata.intra.dist)

    ## Remove zeros
    idx <- which(xdata.intra>0)
    xdata.intra <- xdata.intra[idx]
    xdata.intra.dist <- xdata.intra.dist[idx]
    
    ## Windowing
    ## To optimize - see tapply function
    if (!is.na(winsize)){
        win <- seq.int(from=r[1], to=r[2], by=winsize)
        xp <- yp <- rep(NA, length(win)-1)
      
        tmp <- sapply(1L:(length(win)-1L), function(i){
            idx <- which(xdata.intra.dist>=win[i] & xdata.intra.dist<win[i+1])

            sub <- xdata.intra.dist[idx]
            x <- mean(xdata.intra.dist[idx], na.rm=TRUE)
            y <- mean(xdata.intra[idx], na.rm=TRUE)
            return(c(x, y))
        })
       
        xp <- tmp[1,]
        yp <- tmp[2,]
        ptype <- "b"
    }else{
        xp <- xdata.intra.dist
        yp <- xdata.intra
        ptype <- "p"
    }
    names(xp) <- paste("n",1:length(xp), sep="")
    names(yp) <- paste("n",1:length(yp), sep="")
    
    ## Trim the interaction counts
    if (trim.range<1){
        qt <- quantile(yp, probs=c((1-trim.range),trim.range), na.rm=TRUE)
        idx <- which(yp>=qt[1] & yp<=qt[2])
        yp <- yp[idx]
        xp <- xp[idx]
    }
    
    ## Log
    if (log){
        xp <- log10(xp)
        yp <- log10(yp)
    }

    if (fit){
        ## remove extreme distances
        #qt <- quantile(xp, probs=c(0.1,0.8), na.rm=TRUE)
        #idx <- which(xp>=qt[1] & xp<=qt[2])
        #xp <- xp[idx]
        #yp <- yp[idx]

        ## Define the scaling region
        if (length(fit.lim)==2){
            yp.fit <- yp[which(xp>fit.lim[1] & xp<fit.lim[2])]
            xp.fit <- xp[which(xp>fit.lim[1] & xp<fit.lim[2])]
        }else{
            xp.fit <- xp
            yp.fit <- yp
        }
        res.fit <- lm(yp.fit~xp.fit)

        ## remove outliers on the fitted region
        ## outliers are defined as the points with the higher residual values (distance to the regression curve)
        if (fit.out<1){
            r <- residuals(res.fit)
            th<-quantile(r, probs=fit.out)
            out <- names(which(r>th))
            
            xp.fit <- xp.fit[setdiff(names(xp.fit), out)]
            yp.fit <- yp.fit[setdiff(names(xp.fit), out)]
            res.fit <- lm(yp.fit~xp.fit)

            xp <- xp[setdiff(names(xp), out)]
            yp <- yp[setdiff(names(xp), out)]
        }
    }

    ## Plotting function
    if (!add){
        plot(x=c(min(xp), max(xp)), y=c(min(yp), max(yp)),  xlab="Genomic Distance (log10)", ylab="Interaction Counts (log10)",
              frame=FALSE, type="n", ...)
    }
    
    if (fit){
        pcol <- RColorBrewer::brewer.pal(8, "Pastel2")
        if (length(fit.lim)==2)
            rect(fit.lim[1], min(yp)-1, fit.lim[2], max(yp), col=pcol[5], border=pcol[5])
        abline(res.fit, ...)
        #text(x=max(xp), y=max(yp), labels=paste("a=", round(res.fit$coefficients[2],6)), font=2, cex=.7)
    }
    points(x=xp, y=yp, type=ptype, ...)
    if (fit)
        return(res.fit)
    else
        invisible(NULL)
}

plotHistCounts <- function(x, trim.range=.98, title, ...){
    if (trim.range<1)
        x <- x[which(x<quantile(x, probs=trim.range, na.rm=TRUE))]
    histdens<-sort(hist(x,breaks=500, plot=FALSE)$density)
    ymax <- histdens[floor(length(histdens)*.995)]
    hist(x,freq=FALSE,breaks=500,col="grey", ylim=c(0,ymax),main=paste("Interaction Frequency Histogram\n",title," Interaction Counts",sep=""),ylab="Probability Density",xlab="Interaction Frequency")
    lines(density(x), col="red")
}

plotHistDist <- function(xdata.intra.dist, trim.range=.98, ...){
    x <- xdata.intra.dist
    if (trim.range<1)
        xdata.intra.dist <- xdata.intra.dist[which(x<quantile(x, probs=trim.range, na.rm=TRUE))]
    
    histdens<-sort(hist(xdata.intra.dist,breaks=500, plot=FALSE)$density)
    ymax <- histdens[floor(length(histdens)*.995)]
    hist(xdata.intra.dist,freq=FALSE,breaks=500,col="grey", ylim=c(0,ymax),main="Histogram of Distances Between \nX & Y intervals",ylab="Probability Density",xlab="CIS Interaction Distance",...)
    lines(density(x), col="red")
}

extractCounts <- function(x){
  x.intra <- x.inter <- NULL
  xdata.intra <- xdata.inter <- NULL
  
  if (inherits(x,"HTClist")){
      ## Separate Intra/Inter chromosomal interaction
      if (length(which(isIntraChrom(x)))>0){
          x.intra <- x[isIntraChrom(x)]
          xdata.intra <- unlist(lapply(x.intra,function(x){
              out <- as(as(intdata(x), "sparseMatrix"),"dgTMatrix")@x
              return(out[which(out>0)])
          }))
          xdata.intra.dist <- unlist(lapply(x.intra,function(x){
              return(intervalsDist(x, use.zero=FALSE)@x)}))
      }
      if (length(which(!isIntraChrom(x)))>0){
          x.inter <- x[!isIntraChrom(x)]
          xdata.inter <- unlist(lapply(x.inter,function(x){
              out <- intdata(x)@x
              return(out[which(out>0)])
          }))
      }
  } else if(inherits(x,"HTCexp")){
      if (isIntraChrom(x)){
          x.intra <- x
          xdata.intra <- as(as(intdata(x.intra), "sparseMatrix"),"dgTMatrix")@x
          xdata.intra <- xdata.intra[which(xdata.intra>0)]
          xdata.intra.dist <- intervalsDist(x.intra, use.zero=FALSE)@x
      }else {
          x.inter <- x
          xdata.inter <- intdata(x.inter)@x
          xdata.inter <- xdata.inter[which(xdata.inter>0)]
          xdata.intra.dist <- NULL
      }
  }else{
      stop("Wrong input type. 'HTCexp' or 'HTClist' objects expected.")
  }
  return(list(intra=xdata.intra, intra.dist=xdata.intra.dist, inter=xdata.inter))
}


###################################
## CQC
##
## 'C' Quality Control
##
## x = an object of class HTCexp/HTClist. In case of list, the x are merged as one.
## trans.ratio = if true, plot the histogram of inter/intrachromosomal interactions 
## hist.interac = if true, plot the interaction frequency. How many interaction have n reads 
## scat.interac.dist = if true, plot the scatter plot of counts vs genomic distances. Interaction Distance vs interaction frequency
## hist.dist = if true, plot an histogram of distances between y and x intervals. How many interaction have n distance
## trim.range = remove the extreme values by trimming the counts. Only used for plotting functions
## dev.new = if true, draw each plot in a new view
##
###################################

CQC<- function(x, cis.trans.ratio=TRUE, hist.interac=TRUE, scat.interac.dist=TRUE, hist.dist=TRUE, trim.range=0.98, winsize=NA, dev.new=TRUE){
  
    message("Get data ...")
    data <- extractCounts(x)
    xdata.inter <- data$inter
    xdata.intra <- data$intra
    xdata <- c(xdata.inter, xdata.intra)
    xdata.intra.dist <- data$intra.dist
    
    message("Generate quality control plots ...")
    nbplot <- length(which(c(cis.trans.ratio, scat.interac.dist, hist.dist, hist.interac)))
    if (!is.null(xdata.inter)){
        nbplot <- nbplot+1
    }
    if (!dev.new)
        par(mfrow=c(ceiling(nbplot/2),2), mar=c(4.1, 4.1, 2.5, 1.5), font.lab=2)

  ## Cis/Trans ratio
  if (cis.trans.ratio && length(xdata)>0){
      if (dev.new){
          dev.new()
          par(mar=c(4.1, 4.1, 2.5, 1.5), font.lab=2)
      }
      plotInterIntraRatio(xdata.intra, xdata.inter,
                          cex.lab=0.7, cex.axis=0.7, cex.main=0.9,
                          col=c("#FBB4AE","#B3CDE3"))
  }
   
  ## Scatter plot of interaction distance for intrachromosomal interactions
  if(scat.interac.dist && !is.null(xdata.intra)>0){
      if (dev.new){
          dev.new()
          par(mar=c(4.1, 4.1, 2.5, 1.5), font.lab=2)
      }
      plotIntraDist(xdata.intra, xdata.intra.dist, trim.range, winsize=winsize,
                    cex=0.5, cex.lab=0.7, pch=20,
                    cex.axis=0.7, cex.main=0.9, main="Scatter Plot (Frequency(Y) vs Distance(X))\nCIS Interaction Counts")
  }
 
  ## Histogram of interaction counts for intra and/or interchromosomal interaction
  if (hist.interac){
      if (dev.new){
          dev.new()
          par(mar=c(4.1, 4.1, 2.5, 1.5), font.lab=2)
      }
      plotHistCounts(xdata.intra, trim.range,
                     cex.lab=0.7, cex.axis=0.7,
                     pch=20, cex.main=0.9, title="CIS")
      if (!is.null(xdata.inter)){
          if (dev.new){
              dev.new()
              par(mar=c(4.1, 4.1, 2.5, 1.5), font.lab=2)
          }
          plotHistCounts(xdata.inter, trim.range,
                         ,cex.lab=0.7, cex.axis=0.7,
                         pch=20, cex.main=0.9, title="TRANS")
      }
  }
 
  ## Histogram of Distance for intrachromosomal interactions
  if (hist.dist && !is.null(xdata.intra)){
      if (dev.new){
          dev.new()
          par(mar=c(4.1, 4.1, 2.5, 1.5), font.lab=2)
      }
      plotHistDist(xdata.intra.dist, trim.range,
                   cex.lab=0.7, cex.axis=0.7,
                   pch=20, cex.main=0.9)
  }
}

################
## New S4 class to represent 5C or HiC data
## This class is mainly based on the GRanges class
## The interaction matrix is added to the object 
################

## class definition
setClass("HTCexp",
         representation = representation(
         "intdata"="Matrix",
         "xgi"="GRanges",
         "ygi"="GRanges")
         )


setValidity("HTCexp",
            function(object){
                fails <- character(0L)
                
                ## Format
                if(!inherits(object@intdata,"Matrix")) {
                    fails <- c(fails, "Expected Matrix for the data slot!\n")
                }
                if(!inherits(object@ygi,"GRanges") || !inherits(object@xgi,"GRanges")) {
                    fails <- c(fails, "Intervals 'xgi' and 'ygi' objects must be two GRanges objects!\n")
                }
                if (dim(object@intdata)[1] != length(object@ygi)){
                    fails <- c(fails, "The 'ygi' GRanges object must have exactly the same length as the number of rows of the data matrix")
                }
                if (dim(object@intdata)[2] != length(object@xgi)){
                    fails <- c(fails, "The 'xgi' GRanges object must have exactly the same length as the number of column of the data matrix")
                }
                if (is.null(colnames(object@intdata)) || is.null(id(object@xgi))){
                    fails <- c(fails,"No ids for 'xgi' intervals")
                }
                if (is.null(rownames(object@intdata)) || is.null(id(object@ygi))){
                    fails <- c(fails,"No ids for 'ygi' intervals")
                }
                
                ## Content
                if (length(object@xgi) == 0L || length(object@ygi) == 0L){
                    fails <- c(fails, "Inervals xgi or ygi are of size 0")
                }
                if (length(seqlevels(object@xgi)) > 1L || length(seqlevels(object@ygi)) > 1L){
                    fails <- c(fails, "Multiple chromosome found in xgi or ygi object")
                }
                if (length(setdiff(rownames(object@intdata), id(object@ygi)))>0L){
                  fails <- c(fails, "The 'ygi' intervals from the interaction matrix are different from those defined in the GRanges object")
                }
                if (length(setdiff(colnames(object@intdata), id(object@xgi)))>0L){
                    fails <- c(fails, "The 'xgi' intervals from the interaction matrix are different from those defined in the GRanges object")
                }
                
                if (length(fails) > 0L) return(fails)
                return(TRUE)
            }
)

## constructor
HTCexp <- function(intdata, xgi, ygi, lazyload=FALSE)
{
    ## check xgi/ygi length
    stopifnot(inherits(ygi,"GRanges"))
    stopifnot(inherits(xgi,"GRanges"))
    
    if (length(xgi)==0L || length(ygi)==0L)
        stop("Cannot create HTCexp object of size 0")
    
    if(length(ygi) != nrow(intdata))
        stop("Length 'ygi' is not equal to number of rows in 'intdata'")
    
    if(length(xgi) != ncol(intdata))
        stop("Length 'xgi' is not equal to number of columns in 'intdata'")

    
    ## sort xgi and ygi data
    xgi <- sort(xgi)
    ygi <- sort(ygi)
  
    ## Check matrix format
    if (!inherits(intdata, "Matrix")){
        intdata <- as(as.matrix(intdata), "Matrix")
    }

    ## Check matrix names
    colnames(intdata) <- id(xgi)
    rownames(intdata) <- id(ygi)

    ## Create HTCexp
    obj <- new("HTCexp", intdata, xgi, ygi)

    
    if (lazyload & seqlevels(xgi)==seqlevels(ygi)){
        obj <- forceSymmetric(obj)
        obj <- forceTriangular(obj)
    }
    obj
}


################
##
## Functions
##
################
removeIntervals <- function(x, ids)
{
    stopifnot(inherits(x, "HTCexp"))
    xgi <- x_intervals(x)
    ygi <- y_intervals(x)
    xgi.id <- id(xgi)
    ygi.id <- id(ygi)
    stopifnot(!is.null(xgi.id) | !is.null(ygi.id))

    xgi.id.n <- setdiff(xgi.id, ids)
    if (length(xgi.id.n) != length(xgi.id)){
        message("Discard ",(length(xgi.id)-length(xgi.id.n)), " 'x' intervals")
        x_intervals(x) <- xgi[which(id(xgi)%in%xgi.id.n),]
    }
    
    ygi.id.n <- setdiff(ygi.id, ids)
    if (length(ygi.id.n) != length(ygi.id)){
        message("Discard ",(length(ygi.id)-length(ygi.id.n)), " 'y' intervals")
        y_intervals(x) <- ygi[which(id(ygi)%in%ygi.id.n),]
    }
    x
}

extractRegion <- function(x, MARGIN=c(1,2), chr, from, to, exact=FALSE)
{
    stopifnot(inherits(x, "HTCexp"))
    stopifnot(is.element(MARGIN, c(1,2)))

    fromto <- GRanges(seqnames = chr, ranges = IRanges(start=from, end=to))
    xgi <- x_intervals(x)
    ygi <- y_intervals(x)
    data <-  x@intdata

    ## Force exact overlap by adding flanking regions
    if (exact){
        ## if no interval with from, add NA values
        ## else force overlapping coordinates
        ## x_intervals
        if (is.element(c(1), MARGIN)){
            xgi <- subsetByOverlaps(xgi, fromto, ignore.strand=TRUE)
            
            if (min(start(xgi))>from){
                ##flank(xgi[which.min(start(xgi))], width=min(start(xgi))-from, start=TRUE, ignore.strand=TRUE)
                ir <- IRanges(start=from, end=min(start(xgi))-1)
                edata <- elementMetadata(xgi)[1,]
                if (is.element("score",colnames(edata)))
                    edata$score <- 0L
                if (is.element("name",colnames(edata)))
                    edata$name <- paste(chr,":",from,"-",min(start(xgi))-1, sep="")
                if (is.element("thick",colnames(edata)))
                    edata$thick <- ir
                
                ngr <- GRanges(seqnames = chr, ranges = ir, strand = "*", edata)
                xgi <- c(ngr,xgi)
                data <- cbind(rep(NA, nrow(data)), data)
            }else if (min(start(xgi))<=from){
                maxind <- max(which(start(xgi)<=from & end(xgi)>=from))
                start(xgi[maxind,]) <- from
                data <- data[,maxind:length(xgi)]
                xgi <- xgi[maxind:length(xgi),]
            }
            if (max(end(xgi))<to){
                ir <- IRanges(start=max(end(xgi))+1, end=to)
                edata <- elementMetadata(xgi)[length(xgi),]
                if (is.element("score",colnames(edata)))
                    edata$score <- 0L
                if (is.element("name",colnames(edata)))
                    edata$name <- paste(chr,":",max(end(xgi))+1,"-",to, sep="")
                if (is.element("thick",colnames(edata)))
                    edata$thick <- ir
                
                ngr <- GRanges(seqnames = chr, ranges = ir, strand = "*", edata)
                xgi <- c(xgi, ngr)
                data <- cbind(data, rep(NA, nrow(data)))
            }else if (max(end(xgi))>=to){
                maxind <- min(which(start(xgi)<=to & end(xgi)>=to))
                end(xgi[maxind,]) <- to
                data <- data[,1:maxind]
                xgi <- xgi[1:maxind,]
            }
        }
        
        ## y_intervals
        if (is.element(c(2), MARGIN)){
            ygi <- subsetByOverlaps(ygi, fromto, ignore.strand=TRUE)
            
            if (min(start(ygi))>from){
                ir <- IRanges(start=from, end=min(start(ygi))-1)
                edata <- elementMetadata(ygi)[1,]
                if (is.element("score",colnames(edata)))
                    edata$score <- 0L
                if (is.element("name",colnames(edata)))
                    edata$name <- paste(chr,":",from,"-",min(start(ygi))-1, sep="")
                if (is.element("thick",colnames(edata)))
                    edata$thick <- ir
             
                ngr <- GRanges(seqnames = chr, ranges = ir, strand = "*", edata)
                ygi <- c(ngr,ygi)
                data <- rbind(rep(NA, ncol(data)), data)
            }else if (min(start(ygi))<=from){
                maxind <- max(which(start(ygi)<=from & end(ygi)>=from))
                start(ygi[maxind,]) <- from
             data <- data[maxind:length(ygi),]
                ygi <- ygi[maxind:length(ygi),]
            }        
            if (max(end(ygi))<to){
                ir <- IRanges(start=max(end(ygi))+1, end=to)
                edata <- elementMetadata(ygi)[length(ygi),]
                if (is.element("score",colnames(edata)))
                    edata$score <- 0L
                if (is.element("name",colnames(edata)))
                    edata$name <- paste(chr,":",max(end(ygi))+1,"-",to, sep="")
                if (is.element("thick",colnames(edata)))
                    edata$thick <- ir
                
                ngr <- GRanges(seqnames = chr, ranges = ir, strand = "*", edata)
                ygi <- c(ygi, ngr)
                data <- rbind(data, rep(NA, ncol(data)))
            }else if (max(end(ygi))>=to){
                maxind <- min(which(start(ygi)<=to & end(ygi)>=to))
                end(ygi[maxind,]) <- to
                data <- data[1:maxind,]
                ygi <- ygi[1:maxind,]
            }
        }
        
        data <-  x@intdata[id(ygi), id(xgi)]
        colnames(data) <- id(xgi)
        rownames(data) <- id(ygi)
        HTCexp(data, xgi, ygi)
     }else{
         ## Full overlap
         if (is.element(c(1), MARGIN))
             xgi <- subsetByOverlaps(xgi, fromto, type="within", ignore.strand=TRUE)
         
         if (is.element(c(2), MARGIN))
             ygi <- subsetByOverlaps(ygi, fromto, type="within", ignore.strand=TRUE)

         data <-  x@intdata[id(ygi), id(xgi)]
         HTCexp(data, xgi, ygi)
     }
}##extractRegion


################
##
## Methods
##
################

setMethod("c", "HTCexp", function(x, ...){
    if (missing(x))
        args <- unname(list(...))
    else
        args <- unname(list(x, ...))
    HTClist(args)
})


setMethod("detail",signature(x="HTCexp"),
          function(x){
              stopifnot(validObject(x))
              cat("HTC object\n")
              
              r <- range(x)
              if (length(r)==1){
                  cat("Focus on genomic region [", seqlevels(r), ":",
                      start(r),"-",end(r),"]\n", sep="")
              }else{
                  cat("Focus on genomic regions [",
                      seqlevels(r)[1], ":", start(r)[1], "-", end(r)[1],"] vs [",
                      seqlevels(r)[2], ":", start(r)[2], "-", end(r)[2],"]\n", sep="")
              }
              
              if(isIntraChrom(x)){
                  cat("CIS Interaction Map\n")
              }else{
                  cat("TRANS Interaction Map\n")
              }
              cat("Matrix of Interaction data: [", dim(x@intdata)[1],"-",dim(x@intdata)[2], "]\n", sep="")
              if (isBinned(x)){
                  cat("Binned data - window size =", width(x_intervals(x))[1],"\n")
                  cat(length(x_intervals(x)),"genome intervals\n")
              }
              else{
                  cat(length(x_intervals(x))," genomic ranges from 'xgi' object\n")
                  cat(length(y_intervals(x))," genomic ranges from 'ygi' object\n")
              }
	      
           
              xdata <- x@intdata
              
              cat("Total Reads = ",sum(xdata, na.rm=TRUE),"\n")
              cat("Number of Interactions = ",nnzero(xdata),"\n")
              cat("Median Frequency = ",median(xdata@x[xdata@x>0L]),"\n")
              #cat("Sparsity = ",round((length(xdata)-nnzero(xdata))/length(xdata),3),"\n")
              cat("Sparsity = ",round(length(xdata@x)/length(xdata),3),"\n")

	      invisible(NULL)
          }
) 

## Divide method
setMethod("divide", signature=c("HTCexp","HTCexp"),
          function(x,y){

              xgi.ov <- findOverlaps(x@xgi, y@xgi, type="equal")
              ygi.ov <- findOverlaps(x@ygi, y@ygi, type="equal")

              ##xgi <- subsetByOverlaps(x@xgi, y@xgi, type="equal")
              ##ygi <- subsetByOverlaps(x@ygi, y@ygi, type="equal")
              
              if (length(queryHits(xgi.ov)) == 0L || length(queryHits(ygi.ov)) == 0L){
                  stop("No equal intervals found in x and y objects")
              }

              x.xgi <- x@xgi[queryHits(xgi.ov)]
              x.ygi <- x@ygi[queryHits(ygi.ov)]
              y.xgi <- y@xgi[subjectHits(xgi.ov)]
              y.ygi <- y@ygi[subjectHits(ygi.ov)]

              a <- x@intdata[id(x.ygi), id(x.xgi)]
              b <- y@intdata[id(y.ygi), id(y.xgi)]
              ##a <- x@intdata[id(ygi), id(xgi)]
              ##b <- y@intdata[id(ygi), id(xgi)]
              a[Matrix::which(a==0L | b==0L)]<-NA
              b[Matrix::which(a==0L | b==0L)]<-NA

              ## Use x as reference for intervals description
              colnames(b) <- colnames(a)
              rownames(b) <- rownames(a)
              data <- a/b
              
              HTCexp(data, x.xgi , x.ygi)
          }
)

## forceSymmetric
setMethod(f="forceSymmetric", signature(x="HTCexp", uplo="missing"),
          function(x, uplo){forceSymmetric(x, uplo="U")})

setMethod(f="forceSymmetric", signature(x="HTCexp", uplo="character"),
          function(x, uplo){
              stopifnot(uplo=="U" || uplo=="L")# || uplo=="S")
              idata <- x@intdata
              #if (uplo == "U" || uplo == "L"){
              #  idata <- forceSymmetric(idata, uplo)
              #}else if (uplo== "S"){
              #   idata <- idata + t(idata)
              #   diag(idata) <- diag(idata)/2
              #   idata <- as(idata, "symmetricMatrix")
              #}
              #intdata(x) <- idata
              intdata(x) <- forceSymmetric(idata, uplo)
              x
          }
)

setMethod(f="forceTriangular", signature(x="HTCexp"),
          function(x){
            stopifnot(isSymmetric(x))
            idata <- intdata(x)
            idata[lower.tri(idata)] <- 0
            intdata(x)<-as(idata, "sparseMatrix")
            x
})

#setMethod(f="forcePairwise", signature(x="HTCexp"),
#          function(x){
#            idata <- intdata(x)
#            idata <- idata + t(idata)
#            diag(idata) <- diag(idata)/2
#            idata <- as(idata, "sparseMatrix")
#            intdata(x) <- idata
#            x
#          })

## Interaction Data
setMethod(f="intdata", signature(x="HTCexp"),
          function(x){
            #if (isTriangular(x) && isIntraChrom(x)){
            #  x <- forceSymmetric(x)
            #}
             x@intdata
          }
)

setReplaceMethod(f="intdata", signature(x="HTCexp",value="Matrix"),
                 function(x, value){
                     x@intdata <- value
                     stopifnot(validObject(x))
                     return(x)
                 }
)

## Symetric interaction maps
setMethod("isSymmetric", signature(object="HTCexp"),
	  function(object){
              ret <- FALSE
              idata <- object@intdata
              if (Matrix::isSymmetric(idata))
                ret <- TRUE
              else if (isTriangular(idata))
                ret <- TRUE
              else if (seqlevels(object@xgi)==seqlevels(object@ygi)){
                if (ncol(idata)==nrow(idata))
                  ret <- suppressWarnings(all(idata-t(idata)==0, na.rm=TRUE))
              }
              ret
          }
)

## is Triangular Matrix
setMethod("isTriangular", signature(object="HTCexp"),
	  function(object){
            object@intdata[which(is.na(object@intdata))] <- 0
            Matrix::isTriangular(object@intdata)
          }
)



## Initialize method
setMethod("initialize",signature=c("HTCexp"),
          function(.Object, interaction.data, xgi, ygi){
              if (!missing(interaction.data) && !missing(ygi) && !missing(xgi)){
                  .Object@intdata<-interaction.data
                  .Object@ygi <-ygi
                  .Object@xgi <-xgi
              }
              return(.Object)
          }
)

## Binned data
setMethod("isBinned", signature(x="HTCexp"),
          function(x){
              stopifnot(validObject(x))
              ret <- TRUE
              
              ## Same bins for intra chromosomal
              ##if(isIntraChrom(x) && !all.equal(x@xgi, x@ygi)){
              if (isIntraChrom(x) && sum(countMatches(x@xgi, x@ygi)) != length(x@xgi)){
                  ret <- FALSE
              }
              ## Same width (exept for last bins - checked in 95% of bins)
              x.excl <- round(length(x@xgi)*.95)
              y.excl <- round(length(x@ygi)*.95)
              if (length(duplicated(width(x@xgi)))< x.excl || length(duplicated(width(x@ygi)))< y.excl)
                  ret <- FALSE
              
              ## no gaps
              ##if (isDisjoint(x@xgi) || isDisjoint(x@ygi))
              if (length(reduce(x@xgi)) != 1 || length(reduce(x@ygi)) != 1)
                ret <- FALSE

              ret
          }
)

## Inter/Intra chromosomal interaction
setMethod("isIntraChrom", signature(x="HTCexp"),
          function(x){
              stopifnot(validObject(x))
              ret <- FALSE
              if (seqlevels(x@xgi) == seqlevels(x@ygi))
                  ret <- TRUE
              ret
          }
)

## Intervals names
setMethod(f="id", signature(x="GRanges"),
          function(x){
              ret <- NULL
              if (!is.null(names(x)))
                  ret <- names(x)
              else if (!is.null(elementMetadata(x)$name))
                  ret <- elementMetadata(x)$name
              as.character(ret)
          }
)

## Range
setMethod(f="range", signature(x="HTCexp"),
          function(x){
              suppressWarnings(range(c(x@ygi, x@xgi), ignore.strand=TRUE))
          }
)

## Seqlevels
setMethod(f="seqlevels", signature(x="HTCexp"),
          function(x){
              unique(c(seqlevels(x@ygi), seqlevels(x@xgi)))
          }
)

## Show method
setMethod("show",signature="HTCexp",
          function(object){
              stopifnot(validObject(object))
              cat("HTCexp object\n")
          
              r <- range(object)
              if (length(r)==1){
                  cat("Focus on genomic region [", seqlevels(r), ":",
                      start(r),"-",end(r),"]\n", sep="")
              }else{
                  cat("Focus on genomic regions [",
                      seqlevels(r)[1], ":", start(r)[1], "-", end(r)[1],"] vs [",
                      seqlevels(r)[2], ":", start(r)[2], "-", end(r)[2],"]\n", sep="")
              }
              cat("Matrix of Interaction data: [", dim(object@intdata)[1],"-",dim(object@intdata)[2], "]\n", sep="")
              gf <- union(names(elementMetadata(object@xgi)), names(elementMetadata(object@ygi)))
              if (length(gf)>0)
                  cat("Genomic Features : ", paste(gf, collapse="-"),"\n")
              invisible(NULL)
          }
)


## Substraction
setMethod("substract", signature(x="HTCexp",y="HTCexp"),
          function(x,y){
              xov <- findOverlaps(x@xgi, y@xgi, type="equal")
              yov <- findOverlaps(x@ygi, y@ygi, type="equal")

              a <- x@intdata[id(x@ygi)[queryHits(yov)],
                        id(x@xgi)[queryHits(xov)]]
              b <- y@intdata[id(y@ygi)[subjectHits(yov)],
                        id(y@xgi)[subjectHits(xov)]]
              
              #a <- x@intdata[id(x@ygi),id(x@xgi)]
              #b <- y@intdata[id(y@ygi),id(y@xgi)]
              #a[which(a==0L | b==0L)]<-NA
              #b[which(a==0L | b==0L)]<-NA
              data <- a-b
              xgi <- x@xgi[queryHits(xov)]
              ygi <- x@ygi[queryHits(yov)]
              
              HTCexp(data, xgi , ygi)
          }
)

## Summary
setMethod("summary", signature=c(object="HTCexp"),
          function(object){
              interactors <- seqlevels(object)
              if (isIntraChrom(object))
                  interactors <- rep(seqlevels(object),2)
              else
                  interactors <- seqlevels(object)

              if(max(object@intdata, na.rm=TRUE)==0){
                  dstat <- c(interactors, 0, 0, 0, 0, 1)                 
              }else{
                  ## Force cohersion to dgTMatrix to have @x equal of non zero values
                  xdata <- as(as(object@intdata, "sparseMatrix"),"dgTMatrix")
                  mx <- round(mean(xdata@x, na.rm=TRUE),4)
                  mdx <- round(median(xdata@x, na.rm=TRUE),4)
                  dstat <- c(interactors, sum(xdata@x, na.rm=TRUE), nnzero(xdata, na.counted=TRUE), mx, mdx, round((length(xdata)-length(xdata@x))/length(xdata),4))
              }
              names(dstat) <- c("seq1", "seq2", "nbreads","nbinteraction","averagefreq","medfreq","sparsity")  
              data.frame(summary=dstat, stringsAsFactors=FALSE)
          })



## X Intervals
setMethod(f="x_intervals", signature(x="HTCexp"),
          function(x){x@xgi})

setReplaceMethod(f="x_intervals", signature(x="HTCexp",value="GRanges"),
                 function(x, value){
                     x@xgi <- value
                     intdata <- as(as.matrix(x@intdata[,as.vector(id(x@xgi))]), "Matrix")
                     colnames(intdata) <-as.vector(id(x@xgi))
                     rownames(intdata) <- rownames(x@intdata)
                     x@intdata <- intdata
                     x
                 }
)


## Y Intervals
setMethod(f="y_intervals", signature(x="HTCexp"),
          function(x){x@ygi})

setReplaceMethod(f="y_intervals", signature(x="HTCexp",value="GRanges"),
                 function(x, value){
                     x@ygi <- value
                     intdata <- as(as.matrix(x@intdata[as.vector(id(x@ygi)),]), "Matrix")
                     rownames(intdata) <-as.vector(id(x@ygi))
                     colnames(intdata) <- colnames(x@intdata)
                      x@intdata <- intdata
                     x
                 }
)


## X and Y Intervals
setMethod(f="xy_intervals", signature(x="HTCexp"),
          function(x){
              GRangesList(xgi=x@xgi, ygi=x@ygi)
          }
)

###################################
## getBlocsIndex
## INTERNAL FUNCTION
## Overlap between a GRanges objects (HTCexp vs tracks)
## Return the index (start/end) of overlap region based on the HTCexp ranges
##
## gr = GRanges object, i.e. 'C' xgi for vertical view or ygi for horizontal view. 
## track = list of GRanges objects, i.e. tracks information 
##
##################################

getBlocsIndex <- function(gr, track){
    suppressWarnings(r <- countOverlaps(gr, track, ignore.strand=TRUE))
    ov <- cumsum(width(Rle(ifelse(r==0,0,1))))

    if (!is.null(ov)){
        if (r[length(r)]==0)
            ov <- ov[-length(ov)]
        if (r[1]!=0)
            ov <- c(0,ov)
        blocs <- matrix(ov, ncol=2, byrow=TRUE)
        blocs[,1] <- blocs[,1]+1
    }
    blocs
}

###################################
## addImageTracks
## INTERNAL FUNCTION
## Add a genome track information on interaction map
##
## x = 'HTCexp' object
## tracks = list of GRanges objects, i.e. tracks information 
## orientation = plot vertical or horizontal tracks
##
##################################

addImageTracks <- function(x, tracks, orientation=c("h","v"), names=TRUE){

  stopifnot(inherits(x,"HTCexp"))
  if (inherits(x, "GRanges")){
    x <- list(x)
  }
  stopifnot(unlist(lapply(tracks, inherits,"GRanges")))

  ## Invert tracks order
  tracks<-rev(tracks)
 
  ntrack <- length(tracks)
  suppressWarnings(colblocs <- RColorBrewer::brewer.pal(ntrack*2,"Paired"))
  colblocs.minus <- colblocs[seq(1, ntrack*2, by=2)]
  colblocs.plus <- colblocs[seq(2, ntrack*2, by=2)]
  blocnames <- names(tracks)


  if (orientation=="h"){
    ypos <- 1
    if (isBinned(x)){
      par(mar=c(0,0,0,0))
      plot(c(start(range(x_intervals(x))),end(range(x_intervals(x)))), c(-1, ntrack*3+1),type="n",axes=FALSE, frame=FALSE, xlab="", ylab="", xlim=c(start(range(x_intervals(x))), end(range(x_intervals(x)))), xaxs="i", yaxs="i")
      for (t in 1:ntrack){
        blocs <- tracks[[t]]
        ## keep only overlaping features between
        blocs <- subsetByOverlaps(blocs,range(x_intervals(x)), ignore.strand=TRUE)
        ## get strand information
        blocs.plus <- blocs[which(strand(blocs)=="+" | strand(blocs)=="*")]
        blocs.minus <- blocs[which(strand(blocs)=="-")]        
        ## set up y axis
      
        ## draw features
        if (length(blocs.plus)>0){
          yp <- rep(ypos, length(blocs.plus))
          yp[seq.int(from=1, to=length(yp), by=2)]<-ypos+.5
          rect(start(blocs.plus), yp, end(blocs.plus), yp+.3, col=colblocs.plus[t], border=colblocs.plus[t])
        }
        if (length(blocs.minus)>0){
          yp <- rep(ypos, length(blocs.minus))
          yp[seq.int(from=1, to=length(yp), by=2)]<-ypos+.5
          rect(start(blocs.minus), yp-.5, end(blocs.minus), yp-.8, col=colblocs.minus[t], border=colblocs.minus[t])
        }
        if (names)
          text(x=start(range(x_intervals(x)))+width(range(x_intervals(x)))/2, y=ypos+1.5, labels=blocnames[t], cex=.5, font=2, col=colblocs.plus[t])
        ypos <- ypos+3
      }
    }else{
      warning("The data are not binned, thus the scale is not linear.\nOnly the HTCexp intervals overlapping with the track's feature are displayed.")
      cset <- x_intervals(x)
      par(mar=c(0,0,0,0))
      plot(c(0,length(cset)), c(-1, ntrack*3+1),type="n", axes=FALSE, xlab="", ylab="", frame=FALSE, xlim=c(0,length(cset)), xaxs="i", yaxs="i")
      for (t in 1:ntrack){
        blocs <- tracks[[t]]
        ## keep only overlaping features between
        suppressWarnings(blocs <- subsetByOverlaps(blocs,range(x_intervals(x)), ignore.strand=TRUE))
        ## get strand information and index of cset overlapping with the annotation features
        blocs.plus <- getBlocsIndex(cset, blocs[which(strand(blocs)=="+" | strand(blocs)=="*")])
        blocs.minus <- getBlocsIndex(cset, blocs[which(strand(blocs)=="-")])
        ## draw features
        if (length(blocs.plus)>0)
          rect(blocs.plus[,1]-1, ypos+.1, blocs.plus[,2], ypos+.6, col=colblocs.plus[t], border=colblocs.plus[t])
        if (length(blocs.minus)>0)
                    rect(blocs.minus[,1]-1, ypos-.1, blocs.minus[,2], ypos-0.6, col=colblocs.minus[t], border=colblocs.minus[t])
        if (names)
          text(x=length(cset)/2, y=ypos+1, labels=blocnames[t], cex=.7, font=2, col=colblocs.plus[t])
        ypos <- ypos+3
      }
        }
  }else{
    ypos <- -1
    if (isBinned(x)){
      par(mar=c(0,0,0,0))
      plot(c(1, -ntrack*3-1),c(start(range(y_intervals(x))),end(range(y_intervals(x)))), type="n",axes=FALSE, xlab="", ylab="", frame=FALSE, ylim=c(start(range(y_intervals(x))),end(range(y_intervals(x)))), xaxs="i", yaxs="i")
      
      for (t in 1:ntrack){
        blocs <- tracks[[t]]
        ## keep only overlaping features between
        blocs <- subsetByOverlaps(blocs,range(y_intervals(x)), ignore.strand=TRUE)
        ## get strand information
        blocs.plus <- blocs[which(strand(blocs)=="+" | strand(blocs)=="*")]
        blocs.minus <- blocs[which(strand(blocs)=="-")]
        ## draw features    
        if (length(blocs.plus)>0)
          rect(ypos-.1, (start(range(y_intervals(x)))+end(range(y_intervals(x))))-start(blocs.plus),
               ypos-.6, (start(range(y_intervals(x)))+end(range(y_intervals(x))))-end(blocs.plus),
               col=colblocs.plus[t], border=colblocs.plus[t])
        if (length(blocs.minus)>0)
          rect(ypos+.1, (start(range(y_intervals(x)))+end(range(y_intervals(x))))-start(blocs.minus),
               ypos+.6, (start(range(y_intervals(x)))+end(range(y_intervals(x))))-end(blocs.minus),
               col=colblocs.minus[t], border=colblocs.minus[t])
        if (names)
          text(y=start(range(y_intervals(x)))+width(range(y_intervals(x)))/2, x=ypos-1, labels=blocnames[t], cex=.7, font=2, col=colblocs.plus[t], srt=90)
        ypos <- ypos-3
      }
    }else{
      warning("The data are not binned, and the scale is not linear.\nOnly the HTCexp intervals overlapping with the track's feature are displayed.")
      cset <- y_intervals(x)
      par(mar=c(0,0,0,0))
            plot(c(0, -ntrack*3-1),c(0,length(cset)),type="n", axes=FALSE, xlab="", ylab="", frame=FALSE, ylim=c(0,length(cset)), xaxs="i", yaxs="i")
      
      for (t in 1:ntrack){
        blocs <- tracks[[t]]
        ## keep only overlaping features between
        suppressWarnings(blocs <- subsetByOverlaps(blocs,range(y_intervals(x)), ignore.strand=TRUE))
        ## get strand information and index of cset overlapping with the annotation features
        blocs.plus <- getBlocsIndex(cset, blocs[which(strand(blocs)=="+" | strand(blocs)=="*")])
        blocs.minus <- getBlocsIndex(cset, blocs[which(strand(blocs)=="-")])
        ##draw features
        if (length(blocs.plus)>0)
          rect(ypos-.1, length(cset)-blocs.plus[,1]+1, ypos-.6, length(cset)-blocs.plus[,2], col=colblocs.plus[t], border=colblocs.plus[t])
        if (length(blocs.minus)>0)
          rect( ypos+.1, length(cset)-blocs.minus[,1]+1,  ypos+.6,length(cset)- blocs.minus[,2], col=colblocs.minus[t], border=colblocs.minus[t])
        if (names)
          text(y=length(cset)/2, x=ypos-1, labels=blocnames[t], cex=.7, font=2, col=colblocs.plus[t], srt=90)
                ypos <- ypos-3
      }
    }
  }
}


###################################
## discretize
##
## Transform matrix of counts data into discrete matrix
##
## x = data matrix with 5C interactions (counts)
## quant = if true, use quantile, else just split into equals 'nb.lev' levels
## nb.lev = number of level
##
###################################

discretize <- function (x, nb.lev = 4, quant=TRUE) {

    stopifnot(is.matrix(x))
    out <- x
    if (quant) {
        lev <- quantile(x, seq(0, 1, by = 1/nb.lev), na.rm=TRUE)
    }else{
        mind <- min(x, na.rm=TRUE)
        maxd <- max(x, na.rm=TRUE)
        diff <- (maxd - mind)/nb.lev
        lev=seq(0, maxd, by=diff)
    }
    
    if (length(which(duplicated(lev)))>0){
        lev <- unique(lev)
        warning(paste("Find only ",length(lev)-1," levels (nb.lev=",nb.lev,")", sep=""))
    }

    for (k in 1:length(lev)){
        out[which(x>=lev[k] & x<=lev[k+1])] <- k
    }
    out
}

##Generics

## Data access

setGeneric(name="intdata", def=function(x) standardGeneric("intdata"))
setGeneric(name="intdata<-", def=function(x,value) standardGeneric("intdata<-"))

setGeneric(name="id", def=function(x) standardGeneric("id"))
##setGeneric(name="id<-", def=function(x,value) standardGeneric("id<-"))

setGeneric(name="y_intervals", def=function(x) standardGeneric("y_intervals"))
setGeneric(name="y_intervals<-", def=function(x, value) standardGeneric("y_intervals<-"))
setGeneric(name="x_intervals", def=function(x) standardGeneric("x_intervals"))
setGeneric(name="x_intervals<-", def=function(x, value) standardGeneric("x_intervals<-"))
setGeneric(name="xy_intervals", def=function(x) standardGeneric("xy_intervals"))

## Update objects
setGeneric(name="getCombinedIntervals", def=function(x, ...) standardGeneric("getCombinedIntervals"))
setGeneric(name="getCombinedContacts", def=function(x) standardGeneric("getCombinedContacts"))
setGeneric(name="forcePairwise", def=function(x) standardGeneric("forcePairwise"))
setGeneric(name="forceTriangular", def=function(x) standardGeneric("forceTriangular"))

## is 
setGeneric(name="isComplete", def=function(x) standardGeneric("isComplete"))
setGeneric(name="isPairwise", def=function(x) standardGeneric("isPairwise"))

setGeneric(name="isBinned", def=function(x) standardGeneric("isBinned"))
setGeneric(name="isIntraChrom", def=function(x) standardGeneric("isIntraChrom"))

## Operation
#setGeneric(name="directionalityIndex", def=function(x) standardGeneric("directionalityIndex"))

setGeneric(name="divide", def=function(x,y) standardGeneric("divide"))
setGeneric(name="substract", def=function(x,y) standardGeneric("substract"))

setGeneric(name="normPerReads", def=function(x) standardGeneric("normPerReads"))
setGeneric(name="normPerExpected", def=function(x, ...) standardGeneric("normPerExpected"))
setGeneric(name="normPerTrans", def=function(x, xtrans, ytrans, ...) standardGeneric("normPerTrans"))

setGeneric(name="mapC", def=function(x, ...) standardGeneric("mapC"))
setGeneric(name="mapC", def=function(x, y, ...) standardGeneric("mapC"))

## Deprecated/Defunct
setGeneric(name="normPerZscore", def=function(x, ...) standardGeneric("normPerZscore"))
setGeneric(name="seq_name", def=function(x,...) standardGeneric("seq_name"))

## standard generics
setGeneric("summary")

## Normalized per reads number
setMethod("normPerReads", signature=c("HTCexp"), definition=function(x){
    x@intdata <- x@intdata/sum(x@intdata,na.rm=TRUE)
    x
})

## Normalized using TRANS data
setMethod("normPerTrans", signature=c("HTCexp","HTCexp","HTCexp"), definition=function(x, xtrans, ytrans, method="max"){

    ## Match the good objects
    ## TODO check whether the two objects are really the same
    ## x and trans share the same x_intervals
    if(length(intersect(id(x_intervals(x)),id(x_intervals(xtrans)))) != length(id(x_intervals(x)))){
   	stop("No match between xgi cis and trans objects")
    }   
    
    ## x and trans share the same y_intervals
    if(length(intersect(id(y_intervals(x)),id(y_intervals(ytrans)))) != length(id(y_intervals(x)))){
 	stop("No match between ygi cis and trans objects")
    }   

    ix <- intdata(x)
    iy <- intdata(ytrans)
    iz <- intdata(xtrans)

    ## Weigth matrices from trans data
    normfacRow<-apply(iy, 1, "mean")
    wY <- matrix(rep(normfacRow,ncol(ix)), ncol=ncol(ix), byrow=FALSE)
    
    normfacCol<-apply(iz, 2, "mean")
    wZ <- matrix(rep(normfacCol,nrow(ix)), nrow=nrow(ix), byrow=TRUE)

    ## Normalize cis data
    ## Method
    met.agglo <- c("mult", "sum", "max")
    method <- met.agglo[pmatch(method, met.agglo)]
    if (is.na(method)) 
        stop("Error :Unknown method ['mult','sum','max' are expected].")

    if (method == "sum"){
        w <- (wY+wZ)
    }else if (method == "mult"){
        w <- (wY*wZ)
    }else if (method == "max"){
        w <- matrix(NA, ncol=ncol(wY), nrow=nrow(wY))
        for (i in 1:nrow(wY)){w[i,]<-pmax(wY[i,], wZ[i,])}
    }
    w[which(w==0)] <- 1
    xnorm <- ix/w

    intdata(x) <- xnorm
    return(x)
})

###################################
## colorC
## INTERNAL FUNCTION
## Define a palette for the heatmap
##
## low = color for minimal value
## high = color for extreme values (up and down)
## mid = color for medium values
## k = number of color generated
##
##################################

##colorC <- function (low="white", high=c("red"), mid=NA, k = 50){
colorC <- function (cols, k = 50){

## low <- col2rgb(low)/255
    ## high <- col2rgb(high)/255
    ## if (is.na(mid)) {
    ##     r <- seq(low[1], high[1], len = k)
    ##     g <- seq(low[2], high[2], len = k)
    ##     b <- seq(low[3], high[3], len = k)
    ## }
    ## if (!is.na(mid)) {
    ##     k2 <- round(k/2)
    ##     mid <- col2rgb(mid)/255
    ##     r <- c(seq(low[1], mid[1], len = k2), seq(mid[1], high[1], len = k2))
    ##     g <- c(seq(low[2], mid[2], len = k2), seq(mid[2], high[2], len = k2))
    ##     b <- c(seq(low[3], mid[3], len = k2), seq(mid[3], high[3], len = k2))
    ## }
    ## rgb(r, g, b)
  colorRampPalette(colors=cols)(k)
}


###################################
## plottingfunc
## INTERNAL FUNCTION
## alias to the function used to plot the data
##
## x = color for minimal value
## high = color for extreme values (up and down)
## mid = color for medium values
## k = number of color generated
##
##################################

plottingfunc <- function(x, y, z, show.zero=FALSE, col, ...){
    if (!show.zero && length(col)>1){
        zi <- Matrix::which(!is.na(z) & z == 0)
        ##zi <- which(as.matrix(Matrix:::is0(z)))
        z <- as.matrix(z)
        z[zi] <- NA_integer_
        ## remove col.zero
        col <- col[-1]
    }else{
      #col <- c(col.zero, col)
      z <- as.matrix(z)
    }
    image(x, y, z, useRaster = TRUE, col=col, ...)
}


###################################
## heatmapC
## INTERNAL FUNCTION
## Draw heatmap of the C data
##
## xdata = 'C' interaction map as a matrix
## names = logical, add axis with the bin/interval's names
## value = logical, add values on the matrix
## show.na = logical, na are shown in gray
## show.zero = logical, zero are plotted 
## col.neg = color used in mypalette for neg count (low, mid, high)
## col.pos = color used in mypalette for pos count (low, mid, high)
## col.na = color used for NA values
## grid = logical, the grid is shown
## title = add a title to the plot
##
##################################

heatmapC <- function(xdata,  names=FALSE, value=FALSE, show.zero=FALSE,  show.na=TRUE, col.pos=c("white","red"), col.neg=c("white","blue"), col.na="#CCCCCC", col.zero="#FFFFFF", grid=FALSE, maxrange=NA, title=NULL, k=500){
  xdata <- t(xdata)
  xdata <- as(xdata[,ncol(xdata):1], "sparseMatrix")

  xdata.pos <- xdata.neg <- NULL 
  if (max(xdata, na.rm=TRUE)>0){
    xdata.pos <- xdata
    xdata.pos[xdata<0] <- 0
    xdata.pos <- as(xdata.pos,"sparseMatrix")
  }
  if (min(xdata, na.rm=TRUE)<0){
    xdata.neg <- xdata
    xdata.neg[xdata>=0] <- 0
    xdata.neg <- as(xdata.neg,"sparseMatrix")
  }
  
  ## k is the same for positive or negative values, to ensure that the same range of colors are used
  #k <- length(unique(as.vector(abs(xdata))))
  
  if (!is.null(xdata.neg)){   
    col.neg <- colorC(col.neg, k=k)
    if (!is.na(maxrange) && max(abs(xdata.neg), na.rm=TRUE) < maxrange){
      #col.neg <- col.neg[length(col.neg)-round(length(col.neg)*(max(abs(xdata.neg), na.rm=TRUE)/maxrange)):length(col.neg)]
      col.neg <- col.neg[1:round(length(col.neg)*(max(abs(xdata.neg), na.rm=TRUE)/maxrange))]
    }
    plottingfunc(x=1:nrow(xdata),y=1:ncol(xdata),z=xdata.neg, show.zero=show.zero, axes=FALSE,ylab="",xlab="",col=c(col.neg, col.zero))
    par(new = TRUE)
  }
  
  if (!is.null(xdata.pos)){
    col.pos <- colorC(col.pos,k=k)   
    if (!is.na(maxrange) && max(xdata, na.rm=TRUE) < maxrange){
      col.pos <- col.pos[1:round(length(col.pos)*(max(xdata, na.rm=TRUE)/maxrange))]
    }
    plottingfunc(x=1:dim(xdata)[1],y=1:dim(xdata)[2],z=xdata.pos,show.zero=show.zero,axes=FALSE,ylab="",xlab="",col=c(col.zero,col.pos))
  }
  
  if (names){
    axis(side=2, at=1:ncol(xdata), labels=colnames(xdata), lwd=0.5, las=1, cex.axis=0.7)  
    axis(side=1, at=1:nrow(xdata), labels=rownames(xdata), lwd=0.5, las=2, cex.axis=0.7)
  }
  if (value){
    for (i in nrow(xdata):1){
      text(x=i,y=1:ncol(xdata),labels=round(xdata[i,],2), cex=0.7)
    }
  }
  
  ## ###################
  ## Heatmap options
  ## ###################
  if (show.na){
    if (length(which(is.na(xdata))>0)){
      par(new = TRUE)
      na.xdata <- matrix(NA_integer_, ncol=ncol(xdata), nrow=nrow(xdata))
      na.xdata[which(is.na(xdata))] <- 1
      
      ##if (!show.zero)
      ##  na.xdata[which(xdata==0)] <- 1
      plottingfunc(x=1:nrow(na.xdata),y=1:ncol(na.xdata),z=na.xdata,show.zero=FALSE,axes=FALSE,ylab="",xlab="",col=col.na, add=TRUE)
    }
  }
  
  if (grid){
    abline(h=seq(0.5,ncol(xdata)+0.5, by=1), mar=c(0,0))
    abline(v=seq(0.5,nrow(xdata)+0.5, by=1), mar=c(0,0))
  }
  
  if (!is.null(title)){
    text(x=ncol(xdata)-nchar(title), y=nrow(xdata)-5, title, col="darkgray", font=2)
  }
}

###################################
## triViewC
## INTERNAL FUNCTION
## Draw triangle view of the C data
##
## xdata = 'C' interaction map as a matrix
## value = logical, add values on the matrix
## show.na = logical, na are shown in gray
## col.low = color used in mypalette for low count (low, mid, high)
## col.high = color used in mypalette for high count (low, mid, high)
## col.na = color used for NA values
## plot.zero = plot the zero value or not
## grid = logical, the grid is shown
## title = add a title to the plot
##
##################################

triViewC <- function(xdata, flip=FALSE, value=FALSE, show.zero=FALSE, show.na=TRUE, col.pos=c("white","red"), col.neg=c("white","blue"), col.na="gray80",  col.zero="#FFFFFF", title=NULL, k=500, w=NA){

    d <- min(dim(xdata))
    
    ## Start with a matrix for speed implementation - optimize.by option ???
    trimat <- matrix(0, ncol=d*2, nrow=d)
    xdata <- as.matrix(xdata)
    for (p in 0:(d-1)){
        s <- gdiag(xdata, w=p)
        ls <- length(s)
        ss <- c(s,s)[as.vector(sapply(1:ls,function(x){return(c(x,x+ls))}))]
        trimat[p+1, (p+1):(d*2-p)] <- ss
    }

    if (!is.na(w)){
        trimat <- trimat[1:w*2, ]
    }
    
    trimat <- as(trimat, "Matrix")
    
    if (flip){
        trimat <- trimat[nrow(trimat):1,]
    }
 
    trimat.pos <- trimat.neg <- NULL
    if (max(trimat, na.rm=TRUE)>0){
        trimat.pos <- trimat
        trimat.pos[trimat<0] <- 0
    }
    if (min(trimat, na.rm=TRUE)<0){
        trimat.neg <- trimat
        trimat.neg[trimat>0] <- 0
    }
    
    #k <- length(unique(as.vector(abs(trimat))))
    
    if (!is.null(trimat.neg)){     
        col.neg <- colorC(col.neg,k=k)
        plottingfunc(y=1:nrow(trimat),x=1:ncol(trimat),z=t(trimat.neg),show.zero=show.zero,axes=FALSE,ylab="",xlab="",col=c(col.zero, col.neg))
        par(new = TRUE)
    }

    if (!is.null(trimat.pos)){     
        col.pos <- colorC(col.pos,k=k)
        plottingfunc(y=1:nrow(trimat),x=1:ncol(trimat),z=t(trimat.pos),show.zero=show.zero,axes=FALSE,ylab="",xlab="",col=c(col.zero, col.pos))
     }
    if (show.na){
        if (length(which(is.na(trimat))>0)){
            par(new = TRUE)
            na.trimat <- matrix(NA_integer_, ncol=ncol(trimat), nrow=nrow(trimat))
            na.trimat[which(is.na(trimat))] <- 1
            if (!show.zero)
                na.trimat[which(trimat==0)] <- 1
            plottingfunc(y=1:nrow(na.trimat),x=1:ncol(na.trimat),z=t(na.trimat),show.zero=FALSE,axes=FALSE,ylab="",xlab="",col=col.na)
        }
    }

    if (value){
        coordodd <- seq(.5,ncol(trimat), by=2)
        coord <- seq(1.5,ncol(trimat), by=2)
        for (i in 1:nrow(trimat)){
            if (i%%2)
                text(coord,y=i,labels=round(trimat[i,seq(1,ncol(trimat),2)],2), cex=.7)
            else
                text(coordodd,y=i,labels=round(trimat[i,seq(1,ncol(trimat),2)],2), cex=.7)
        }
    }
    if (!is.null(title)){
        text(x=1, y=nrow(trimat), adj=c(0,1), title, col="darkgray", font=2)
    }
}

###################################
## gdiag
## INTERNAL FUNCTION
## Extract the diagonal of a matrix - Generalisation of diag function
##
## x = a matrix
## w = step from the real diagonal. w=0 return the diagonal of the matrix.
##
##################################

gdiag <- function(x, w=0){
    if ((m <- min(dim(x))) == 0L) 
        return(vector("numeric", 0L))
    if (w<m){
        if (w>0)
            y <- x[1L + (0L+w):(m - 1L)* (dim(x)[1L] + 1L) - w]
        else
            y <- x[1L + 0L:(m - 1L - abs(w))* (dim(x)[1L] + 1L) + abs(w)]
        return(y)
    }else{
        return(vector("numeric", 0L)) 
    }
}


###################################
## setEnvDisplay
## INTERNAL FUNCTION
## Define the environment display for visualization
## Based on the graphics package
##
## x = HTCexp/HTClist object
## y = HTCexp object
## tracks = List of GRanges objects. Each object represent a genome track information
##
##################################
setEnvDisplay <- function(x, y=NULL, view, tracks=NULL){

  chrom <- seqlevels(x)
  lc <- length(chrom)

  if(!is.null(tracks)){
      stopifnot(unlist(lapply(tracks, inherits,"GRanges")))
      ntrack <- length(tracks)
      sizeblocs <- .05
  }

  ## HTClist object
  if (view==1L){
      rx <- range(x)
      w <- as.numeric(width(rx))
      names(w) <- seqlevels(rx)
      w <- w[chrom]
    
    if(length(tracks) > 0){
      design <- matrix(NA, lc+1, lc+1)
      design[1,] <- c(1,seq(lc^2+2,(lc+1)^2-1,2))
      design[,1] <- c(1,seq(lc^2+3,(lc+1)^2,2))
      design[2:(lc+1), 2:(lc+1)] <- matrix(2:(lc^2+1), lc, lc, byrow=TRUE)

      heatspace <- 1-sizeblocs*ntrack
      layout(design, widths=c(sizeblocs*ntrack,round(w/sum(w)*heatspace,5)), heights=c(sizeblocs*ntrack,round(w/sum(w)*heatspace,5)))
      
      ##blank plot at position 1
      par(mar=c(0,0,0,0))
      plot(1, type="n", axes=FALSE, xlab="", ylab="")     
    }else{
      design <- matrix(1:lc^2, lc, lc, byrow=TRUE)
      layout(design, widths=round(w/sum(w),5), heights=round(w/sum(w),5))
    }
    ## HTCexp object
  }else if (view==2L){
    rx <- range(x)
    ## Annotation
    if(length(tracks) > 0L){
      if (!is.null(y)){
        ry <- range(y)
        if (width(rx)>=width(ry)){
          design <- rbind(rep(2L,3),rep(1L,3),c(4L,3L,5L),rep(6L,3))
          lmar <- (start(ry)-start(rx))/width(rx)
          rmar <- (end(rx)-end(ry))/width(rx)
          h <- (1-sizeblocs*ntrack)/2
          hy <- width(ry)/width(rx)
          layout(design, heights=c(h, sizeblocs*ntrack,h*hy,h*(1-hy)), widths=c(lmar,(1-rmar-lmar), rmar))
        }else{
          design <- rbind(rep(6L,3), c(4L,2L,5L), rep(1L,3),rep(3L,3)) 
          lmar <- (start(rx)-start(ry))/width(ry)
          rmar <- (end(ry)-end(rx))/width(ry)
          h <- (1-sizeblocs*ntrack)/2
          hx <- width(rx)/width(ry)
          layout(design, heights=c(h*(1-hx), h*hx, sizeblocs*ntrack,h), widths=c(lmar,(1-rmar-lmar), rmar))
        }
      }else{
        design <- matrix(1:2, 2, 1, byrow=TRUE)
        layout(design, heights=c(1-sizeblocs*ntrack, sizeblocs*ntrack))
      }
    }else{
      if (!is.null(y)){
        ry <- range(y)
        ## Adjust position of x and y if with are not the same
        if (width(rx)>=width(ry)){
          design <- matrix(c(1L,1L,1L,3L,2L,4L,5L,5L,5L), ncol=3, byrow=TRUE)
          lmar <- (start(ry)-start(rx))/width(rx)
          rmar <- (end(rx)-end(ry))/width(rx)
          hy <- width(ry)/width(rx)
          layout(design, widths=c(lmar,(1-rmar-lmar), rmar), heights=c(1L, hy, 1-hy))
          
        }else{
          design <- matrix(c(5L,5L,5L,3L,1L,4L,2L,2L,2L), ncol=3, byrow=TRUE)
          lmar <- (start(rx)-start(ry))/width(ry)
          rmar <- (end(ry)-end(rx))/width(ry)
          hx <- width(rx)/width(ry)
          layout(design, widths=c(lmar,(1-rmar-lmar), rmar), heights=c(1-hx, hx ,1L))
        }
      }else{
        design <- matrix(1:lc^2, lc, lc, byrow=TRUE)
        layout(design, widths=rep(1/lc, lc), heights=rep(1/lc, lc))
      }
    }
  }
}

###################################
## getMapData
## INTERNAL FUNCTION
## Operations to apply to the data before the visualization
##
## x = HTCexp object
## y = HTCexp object
## tracks = List of GRanges objects. Each object represent a genome track information
##
##################################

getData2Map <- function(x, minrange, maxrange, trim.range, log.data){
    stopifnot(inherits(x,"HTCexp"))
    xdata <- intdata(x)
    
    ## #####################
    ## Data Transformation
    ## #####################
    if (log.data){
        xdata <- log2(xdata)
        xdata[is.infinite(xdata)] <- 0
    }
 
    ## #################
    ## Play with contrast
    ## #################
    if (max(xdata, na.rm=TRUE)>0){
      if (trim.range <1 && is.na(maxrange) && is.na(minrange)){
        if (inherits(xdata, "sparseMatrix")){
          xmaxrange <- quantile(abs(xdata@x), probs=trim.range, na.rm=TRUE)
          xminrange <- quantile(abs(xdata@x), probs=1-trim.range, na.rm=TRUE)
        }else{
          xmaxrange <- quantile(abs(xdata@x[which(xdata@x!=0)]), probs=trim.range, na.rm=TRUE)
          xminrange <- quantile(abs(xdata@x[which(xdata@x!=0)]), probs=1-trim.range, na.rm=TRUE)
        }
      }
      else{
        if (is.na(maxrange))
          xmaxrange <- max(abs(xdata@x), na.rm=TRUE)
        else
          xmaxrange=maxrange
        if (is.na(minrange))
          xminrange <- min(abs(xdata@x), na.rm=TRUE)         
        else
          xminrange=minrange
      }
      
      ## The minrange/maxrange are symmetrical around zero. The same threshold are used for positive and negative values
      xdata@x[which(xdata@x<=xminrange & xdata@x>0)] <- xminrange
      xdata@x[which(xdata@x>=xmaxrange & xdata@x>0)] <- xmaxrange
      xdata@x[which(xdata@x>=-xminrange & xdata@x<0)] <- -xminrange
      xdata@x[which(xdata@x<=-xmaxrange & xdata@x<0)] <- -xmaxrange

      if (max(xdata@x, na.rm=TRUE)>0)
        message(paste("minrange=",round(min(xdata@x[which(xdata@x>0)], na.rm=TRUE),3)," - maxrange=", round(max(xdata@x[which(xdata@x>0)], na.rm=TRUE),3)))
    }
    if (max(xdata, na.rm=TRUE)==0){
      ## Fix bug in case of empty matrix
      message("Warning: no data to plot. Fixed to 1e-4.")
      xdata[1,1] <- 1e-4
    }
    xdata
}
       

maskdiag <- function (x, w = 0){
    m <- min(dim(x))
    ret <- matrix(FALSE, nrow=dim(x)[1], ncol=dim(x)[2])
    w <- abs(w)
    for (i in 0:w)
        ret[1L + (0L + i):(m - 1L) * (dim(x)[1L] + 1L) - i] <- TRUE
    w <- -w
    for (i in w:0)
        ret[1L + 0L:(m - 1L - abs(i)) * (dim(x)[1L] + 1L) + abs(i)] <- TRUE
    ret
}

###################################
## mapC methods
## 
## Visualization of HTCexp or HTClist objects
##
## x = HTCexp/HTClist object
## y = optional. HTCexp/HTClist object or matrix data
## tracks = List of GRanges objects. Each object represent a genome track information
## minrange = minimum value to draw
## maxrange = maximum value to draw
## trim.range = remove the outliers values by trimming the maxrange (quantile)
## names = logical, add axis with the bin/interval's names
## value = logical, add values on the matrix
## show.na = logical, na are shown in gray
## col.low = color used in mypalette for low count (low, mid, high)
## col.high = color used in mypalette for high count (low, mid, high)
## col.na = color used for NA values
## grid = logical, the grid is shown
## title = add a title to the plot
##
##################################

setMethod("mapC", signature="HTClist",
          function(x, tracks=NULL,
                   minrange=NA, maxrange=NA, trim.range=0.98, show.zero=FALSE, show.na=FALSE, log.data=FALSE, names=FALSE, value=FALSE, k=500,
                   col.pos=c("white","red"), col.neg=c("blue","white"), col.na="#CCCCCC",  col.zero="#FFFFFF", grid=FALSE){
              
              ## Set Graphical Environment
              setEnvDisplay(x, tracks=tracks, view=1)
              
              ## Get data to map and plots
              #isPairwise <- FALSE
              #if (isPairwise(x)){
              #  isPairwise <- TRUE
              #}             
              tmp <- sapply(names(pair.chrom(seqlevels(x))), function(i){
                  if (is.element(i,names(x))){
                      obj <- x[[i]]
                      message("Plotting ",i,"...")
                      xdata <- getData2Map(obj, minrange=minrange, maxrange=maxrange, trim.range=trim.range, log.data=log.data)
                      if (!names)
                          par(mar=c(0,0,0,0))
                      else
                          par(mar=c(mean(sapply(rownames(xdata),nchar))/2,mean(sapply(colnames(xdata),nchar))/2,0,0))
                      heatmapC(xdata, names=names, value=value, show.zero=show.zero ,show.na=show.na, col.pos=col.pos,
                               col.neg=col.neg, col.na=col.na, col.zero=col.zero, maxrange=maxrange, grid=grid, k=k)
                  }else{
                    plot(1, type="n", axes=FALSE, xlab="", ylab="")
                  }
              })
              
              ## Add annotation based on intrachromosomal maps
              if (!is.null(tracks)){
                  tmp <- sapply(paste(seqlevels(x), seqlevels(x), sep=""), function(i){
                      if (is.element(i,names(x))){
                          obj <- x[[i]]
                          addImageTracks(obj, tracks, orientation="h", names=FALSE)
                          addImageTracks(obj, tracks, orientation="v", names=FALSE)
                }else{
                    warning("Intrachromosomal map for ",i," not found. Annotation skipped.")
                    plot(1, type="n", axes=FALSE, xlab="", ylab="")
                    plot(1, type="n", axes=FALSE, xlab="", ylab="")
                }
                  })
              }
          }
)


setMethod("mapC", signature="HTCexp",
          function(x, tracks=NULL,
                   minrange=NA, maxrange=NA, trim.range=0.98, show.zero=FALSE, show.na=FALSE, log.data=FALSE, value=FALSE, k=500, 
                   col.pos=c("white", "red"), col.neg=c("blue","white"), col.na="#CCCCCC",  col.zero="#FFFFFF", grid=FALSE, title=NULL){

              if (!isIntraChrom(x))
                  stop("The triangle view is available for intrachromosomal data only")

              ## Set Graphical Environment
              setEnvDisplay(x, tracks=tracks, view=2)
              
              ## Get data to map
              xdata <- getData2Map(x, minrange=minrange, maxrange=maxrange, trim.range=trim.range, log.data=log.data)

              ## Plots tracks and C map
              par(mar=c(0,0,0,0))
              triViewC(xdata, show.zero=show.zero, show.na=show.na, col.pos=col.pos, col.neg=col.neg, col.na=col.na, value=value, col.zero=col.zero, title=title, k=k)
              if (!is.null(tracks))
                addImageTracks(x, tracks, orientation="h")
          }
)


setMethod("mapC", signature=c("HTCexp","HTCexp"),
          function(x, y, tracks=NULL,
                   minrange=NA, maxrange=NA, w=NA, trim.range=0.98,  show.zero=FALSE, show.na=FALSE, log.data=FALSE, value=FALSE, k=500,
                   col.pos=c("white","red"), col.neg=c("blue","white"), col.na="#CCCCCC",  col.zero="#FFFFFF", grid=FALSE, title=NULL){

              if (!isBinned(x) || !isBinned(y))
                  stop("x and y have to be binned to plot them on the same scale")
              if (seqlevels(x) != seqlevels(y))
                  stop("x and y have to come from the same chromosome")

              ## Remore long range contact if specified
              if (!is.na(w)){
                  intdata(x)[!maskdiag(intdata(x), w=w)] <- NA
                  intdata(y)[!maskdiag(intdata(y), w=w)] <- NA
              }
              
              ## Set Graphical Environment
              setEnvDisplay(x, y, tracks=tracks, view=2)

              ## Get data to map and plots
              xdata <- getData2Map(x, minrange=minrange, maxrange=maxrange, trim.range=trim.range, log.data=log.data)
              ydata <- getData2Map(y, minrange=minrange, maxrange=maxrange, trim.range=trim.range, log.data=log.data)

              ## Plots tracks and C map
              if (!is.null(tracks)){
                if (width(range(x))>=width(range(y))){
                  addImageTracks(x, tracks, orientation="h")
                }else{
                  addImageTracks(y, tracks, orientation="h")
                }
              }
              par(mar=c(0,0,0,0))
              triViewC(xdata, value=value, show.zero=show.zero, show.na=show.na, col.pos=col.pos, col.neg=col.neg, col.na=col.na, col.zero=col.zero, title=title[1], k=k, w=w)
              par(mar=c(0,0,0,0))
              triViewC(ydata, flip=TRUE, value=value, show.zero=show.zero, show.na=show.na, col.pos=col.pos, col.neg=col.neg, col.na=col.na, col.zero=col.zero, title=title[2], k=k, w=w)
            }
)

###################################
## plot Alias
##################################

setMethod("plot", signature="HTClist",
          function(x, ...){
              mapC(x, ...)
          }
)

setMethod("plot", signature="HTCexp",
          function(x, ...){
              mapC(x, ...)
          }
)

setMethod("plot", signature=c("HTCexp","HTCexp"),
          function(x, y, ...){
              mapC(x, y, ...)
          }
)

%\VignetteIndexEntry{Hi-C data analysis using HiTC}
%\VignetteDepends{}
%\VignetteKeywords{next-generation sequencing}
%\VignettePackage{HiTC} % name of package

%%%% HEAD SECTION: START EDITING BELOW %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\documentclass[11pt, a4paper, fleqn]{article}

<<style, eval=TRUE, echo=FALSE, results=tex>>=
BiocStyle::latex(use.unsrturl=FALSE)
@ 

\usepackage{geometry}
%\usepackage{color}
%\definecolor{darkblue}{rgb}{0.0,0.0,0.75}
\definecolor{mygray}{gray}{0.90}
%\usepackage[%
%baseurl={http://www.bioconductor.org},%
%pdftitle={HiTC - Hi-C data analysis},%
%pdfauthor={Nicolas Servant},%
%pdfsubject={HiTC Vignette},%
%pdfkeywords={Bioconductor},%
%pagebackref,bookmarks,colorlinks,linkcolor=darkblue,citecolor=darkblue,%
%filecolor=darkblue,urlcolor=darkblue,pagecolor=darkblue,%
%raiselinks,plainpages,pdftex]{hyperref}

%\usepackage[utf8]{inputenc} 
%\usepackage{verbatim} % for multi-line comments
%\usepackage{amsmath,a4,t1enc, graphicx}
\usepackage{natbib}
%\bibpunct{(}{)}{;}{a}{,}{,}

%\parindent0mm
%\parskip2ex plus0.5ex minus0.3ex

%\newcommand{\Robject}[1]{{\texttt{#1}}}
%\newcommand{\Rfunction}[1]{{\texttt{#1}}}
%\newcommand{\Rpackage}[1]{{\textit{#1}}}
%\newcommand{\Rclass}[1]{{\textit{#1}}}
\newcommand{\Rmethod}[1]{{\texttt{#1}}}
%\newcommand{\Rfunarg}[1]{{\texttt{#1}}}
%\newcommand{\phead}[1]{{\flushleft \sf \small \textbf{#1} \quad}}
% \newcommand{\myincfig}[3]{%
%   \begin{figure}[h!tb]
%     \begin{center}
%       \includegraphics[width=#2]{#1}
%       \caption{\label{#1}\textit{#3}}
%     \end{center}
%   \end{figure}
% }
%\addtolength{\textwidth}{2cm}
%\addtolength{\oddsidemargin}{-1cm}
%\addtolength{\evensidemargin}{-1cm}
%\addtolength{\textheight}{2cm}
%\addtolength{\topmargin}{-1cm}
%\addtolength{\skip\footins}{1cm}


%%%%%%% START EDITING HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\begin{document}

\SweaveOpts{prefix.string = ./HiTC, eps=false, keep.source=TRUE, eval=TRUE, echo=TRUE} % produce no 'eps' figures
%\setkeys{Gin}{width=0.5\textwidth}

\title{\textbf{Analyzing Hi-C data with the \Rpackage{HiTC} BioC package}}
\author{
\href{mailto:nicolas.servant@curie.fr}{Nicolas Servant}
}
\maketitle

%\tableofcontents
%\newpage

\section{Introduction}
The Hi-C technic was first introduced by \cite{Lieberman-Aiden2009} to simultaneoulsy detect all chromosomal interactions in a single experiment. The Hi-C aims at measuring the population-averaged frequency at which two genomic loci physically interact in three-dimensional space. Briefly, after a first crosslink and digestion with a restriction enzyme, all genomic fragments are labeled with a biotinylated nucleotide before ligation. These junctions can then be purified efficiently by streptavidin-coated magnetic beads, and finaly sequenced using an Illumina paired-end protocol.\\
After sequencing, raw reads have to be processed to generate the inter/intrachromosomal contact maps. The main steps of this processing are described in \cite{Imakaev2012}. The raw paired-end reads are first independently aligned on a reference genome. The two mates from the same DNA fragment therefore define the two interacting loci on the genome. Imakaev et al. also proposed an iterative mapping procedure to rescue the reads spanning the ligation junction (and thus containing the two interactors in a single read).\\
After the reads alignment, the Hi-C molecule generated from the DNA digestion and the ligation products are reconstructed using the position and direction of the sequenced mates. Self-circle ligations, single side reads and dangling ends are discarded, and the valid ligation products aligned to different restriction fragments and face toward the restriction site are used to reconstruct the contact maps. The interaction frequencies are therefore estimated by counting how many times two genomic bins (at a given resulution) were found as interactors. The data resolution usually depends on the sequencing depth. In their first paper, \cite{Lieberman-Aiden2009} generated data at a resolution of 1Mb (up to 100kb) and reveal the compartmentalization of the genome into regions of open and closed (active and inactive) chromatin as well as the three-dimensional structure of the genome (fractal globule). More recently, \cite{Dixon2012} generated 20 to 40kb contact maps in order to go deeper in the conformation structure and to study topological domains (\cite{Nora2012a}).\\
The \Rpackage{HiTC} is designed to import processed data as contact maps. In order to generate these maps from raw sequencing reads a couple of pipeline are available on the web. We have recently developed the \href{https://github.com/nservant/HiC-Pro}{HiC-Pro} software which is an optimzed pipeline to process Hi-C data from raw reads to normalized contact maps \cite{Servant2015}. The outputs of \href{https://github.com/nservant/HiC-Pro}{HiC-Pro} is fully compatible with the HiTC package.\\
This vignette is based on the analysis of the \cite{Dixon2012} contact maps, at a resolution of 40kb. These data are stored as a \Rclass{HTClist} object, i.e. a list of inter/intrachromosomal contact maps, one for each pair of chromosomes. The goal of this vignette is to describe how the \Rpackage{HiTC} R package can be used to explore such data.\\

If you use \Rpackage{HiTC} for analyzing your data, please cite:
\begin{itemize}
\item Servant N., Lajoie B.R., Nora E.P., Giorgetti L., Chen C., Heard E., Dekker J., Barillot E. (2012) HiTC : Exploration of High-Throughput 'C' experiments. \textit{Bioinformatics}.
\end{itemize}


\section{Working with Hi-C data}

The \Rclass{HTCexp} (High-Throughput 'C' experiment) class aims at representing a single 'C' experiment, characteriez by :
  \begin{itemize}
  \item A contact map (i.e a \Rclass{Matrix})
  \item Two \Rclass{GRanges} objects that describe each features of the contact map, respectively, the x (i.e. columns) and y (i.e. rows) labels of the matrix. In the context of 5C, these two \Rclass{GRanges} objects will describe the set of forward and reverse primers, and for the Hi-C the binned genomic intervals at a given resolution.\\
\end{itemize}

Whereas a 5C dataset is usually composed of a single intrachromosomal contact map (i.e. \Rclass{HTCexp} object), a Hi-C dataset is represented by a list of inter/intrachromosomal contact maps, characterized by the physical interactions of each pair of chromosomes. The \Rclass{HTClist} was designed as a list of \Rclass{HTCexp} objects, with a couple of dedicated methods.\\
  
Working at a resolution of 40kb (or even at a lower resolution) can result in an intensive memory usage. Assuming that every restriction fragment could ligate to any other, there are on the order of $10^{11}$ possible HindIII restriction fragment pairs in the Human genome. In addition to the fact that the generation of a Hi-C library with enough complexity or sequence depth to cover all possible restriction fragment interactions is difficult, this will result in contact maps with a very high level of sparsity. The \Rpackage{HiTC} package provides an efficient memory storage of the data, based on the \Rclass{sparseMatrix} class and the \Rpackage{Matrix} R package. In addition, binned intrachromosomal maps are expected to be symmetrical around the diagonal and can thus be stored as symmetrical matrix (\Rclass{dsCMatrix} Matrix class). In the same way, the interchromosomal maps are also stored once as the chr1-chr2 map is expected to be the transposed of the chr2-chr1 map.\\

<<head>>=
require(HiTC)
require(HiCDataHumanIMR90)
data(Dixon2012_IMR90)

show(hic_imr90_40)
class(intdata(hic_imr90_40$chr1chr1))
object.size(hic_imr90_40)
@

\section{Description of the Hi-C data}

The \Rpackage{HiTC} package provides several methods to describe a \Rclass{HTClist} object.
<<describe>>=
## Show data
show(hic_imr90_40)
## Is my data complete (i.e. composed of intra + the related inter chromosomal maps)
isComplete(hic_imr90_40)
## Note that a complete object is not necessarily pairwise 
## (is both chr1-chr2 and chr2-chr1 stored ?)
isPairwise(hic_imr90_40)
## Which chromosomes ?
seqlevels(hic_imr90_40)
## Details about a given map
detail(hic_imr90_40$chr6chr6)
## Descriptive statistics
head(summary(hic_imr90_40))
@ 

\section{Hi-C Visualization}

The Hi-C data can be visualized as contact maps, representing the contact frequencies between two chromosomes, or at the level of the genome.
By default, objects from the \Rclass{HTClist} class will be represented as an heatmap, whereas object from the \Rclass{HTCexp} class (i.e. single map) as a triangle. Depending on what you want to visualize the resolution of the map can also be changed (from high to lower resolution).\\

<<plot1, fig=TRUE, width=5, height=5>>=
## Go back to a smaller dataset (chr21, 22, X) at lower resolution
sset <- reduce(hic_imr90_40, chr=c("chr5","chr6","chr7"))
imr90_500 <- HTClist(lapply(sset, binningC, 
                          binsize=500000, bin.adjust=FALSE, method="sum", step=1))
mapC(imr90_500)
@ 


As we can see on this exemple, only half of the inter-chromosomal maps as stored and thus plotted. To display the full pairwise maps, methods such as \Rmethod{forcePairwise}, or \Rmethod{forceSymmetric} can be used to switch from a pairwise (and more memory consuming) form to a reduced form.

<<plot2, fig=TRUE, width=2, height=2>>=
mapC(forcePairwise(imr90_500), maxrange=200)
@ 


\section{Hi-C Normalization}

\subsection{Back to restriction fragments}

In addition to descriptive methods on the \Rclass{HTClist} object, the \Rpackage{HiTC} package provides functions to extract biological information related to the data processing. These functions are useful for data normalization, in order to extract the expected bias at the level of the restriction fragment.

<<resFrag>>=
## Example on chromosome X
## GRanges of restriction fragments after HindIII digestion
resFrag <- getRestrictionFragmentsPerChromosome(resSite="AAGCTT", overhangs5=1, 
                                                chromosomes="chr6", 
                                                genomePack="BSgenome.Hsapiens.UCSC.hg18")
resFrag
@ 

\subsection{Local genomic feature (LGF) normalization}
As any sequencing application, the Hi-C library preparation contains bias, which can be broadly classified as ligation bias and sequence content bias. These effects were first described by \cite{Yaffe2011} and require appropriate normalization methods.\\
\cite{Hu2012} recently proposed a linear model strategy to normalize the data. Their method (named HiCNorm) requires that the bias was infered from the restriction fragments and then used at the Hi-C resolution. 
The \Rfunction{getAnnotatedRestrictionSites} function aims at annotating the restriction fragments according to their mappability (optional), GC content and effective length features. The local genomic features can be then assign to each genomic region, and normalized using the \cite{Hu2012} method.\\
In the following example, we will focus on chromosome 6 only. The same code can be easily applied on a \Rclass{HTClist} object using the \Rfunction{lapply} function.
In the same way, we will not use here the mappability information for space issue. In practice, the mappability track can be downloaded from the ENCODE project data, and is important to normalize the Hi-C data.

<<annot>>=
## Annotation of genomic features for LGF normalization
## Example on chromosome 6

## Load mappability track
require(rtracklayer)
##map_hg18 <- import("wgEncodeCrgMapabilityAlign100mer_chr6.bw",format="BigWig")
map_hg18 <- NULL
cutSites <- getAnnotatedRestrictionSites(resSite="AAGCTT", overhangs5=1, 
                                         chromosomes="chr6", 
                                         genomePack="BSgenome.Hsapiens.UCSC.hg18", 
                                         wingc=200, mappability=map_hg18, winmap=500)

head(cutSites)
## Annotation of Hi-C object
imr90_500_chr6annot <- setGenomicFeatures(imr90_500$chr6chr6, cutSites)
y_intervals(imr90_500_chr6annot)
@ 

<<normLGF, eval=FALSE>>=
## LGF normalization
imr90_500_LGF <- normLGF(imr90_500_chr6annot)
@ 


\subsection{Iterative correction and eigenvector decomposition (ICE) normalization}

The ICE normalization is one of the most popular method to correct data from Hi-C bias. This method is based on the assumption of equal visibility of each genomic locus. The matrix of contact probabilities, $M$, for all given pairs of regions ($i$,$j$) is thus normalized such as $\sum\limits_{i,i\neq j,j\pm 1}M_{ij} = 1$ for each region $j$. Note that running the ICE normalization method can be memory consuming because it uses the full genome matrix, and then store the bias vectors. If we advice to use apply the ICE normalization on a full Hi-C dataset (using inter and intrachromosomal maps), the \Rpackage{HiTC} package also allows to run it on a single intrachromosomal map.\\

<<normICE, fig=TRUE, width=3, height=3>>=
imr90_500_ICE <-  normICE(imr90_500, max_iter=10)
mapC(HTClist(imr90_500_ICE$chr6chr6), trim.range=.95, 
     col.pos=c("white", "orange", "red", "black"))
@ 

\section{Detection of Topological Domains}

Recent studies on a high resolution human and mouse Hi-C dataset have discovered that the genome organization can be further divided into megabase-long and evolutionarily conserved topological domains (TADs), with high frequencies of intra-domain chromatin interactions but infrequent inter-domain chromatin interactions (\cite{Nora2012a}, \cite{Dixon2012}). More recently, \cite{Phillips-Cremins2013a} have demonstrated that the cell-type-specific chromatin organization seems to occur at the sub-megabase scale involving different ligation proteins and epigenomic mechanisms.\\
The following code shows how to focus on TADs, such as the ones describes in IMR90 around the Hox genes.

<<tads, fig=TRUE, width=5, height=3>>=
hox <- extractRegion(hic_imr90_40$chr6chr6, chr="chr6", from=50e6, to=58e6)
plot(hox, maxrange=50, col.pos=c("white", "orange", "red", "black"))
@ 

Different algorithms have been proposed to detect TADs. The directionality index was proposed by \cite{Dixon2012} as an input to their HMM model.
<<di, fig=TRUE, width=6, height=4>>=
di<-directionalityIndex(hox)
barplot(di, col=ifelse(di>0,"darkred","darkgreen"))
@ 

\small
\section*{Package versions}
This vignette was generated using the following package versions:
<<sessionInfo, echo=FALSE, results=tex>>=
toLatex(sessionInfo(), locale=FALSE)
@

\section*{Acknowledgements}
Many thanks to Nelle Varoquaux and Pierre Gestraud for useful discussion and help in developping this R package. Thank you to Ming Hu who developped the HiCNorm method, and help us in the integration of its method in the HiTC package. A special thanks to the \Rpackage{HiTC} users for useful discussions and idea to improve it.

\newpage
\small
%%% BIBLIOGRAPHY STARTS HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\bibliographystyle{abbrvnat}
\bibliography{HiTC}

%\begin{thebibliography}{}
%\end{thebibliography}

\end{document}

%\VignetteIndexEntry{Introduction to HiTC package}
%\VignetteDepends{}
%\VignetteKeywords{next-generation sequencing}
%\VignettePackage{HiTC} % name of package

%%%% HEAD SECTION: START EDITING BELOW %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\documentclass[11pt, a4paper, fleqn]{article}

<<style, eval=TRUE, echo=FALSE, results=tex>>=
BiocStyle::latex(use.unsrturl=FALSE)
@ 

\usepackage{geometry}
%\usepackage{color}
%\definecolor{darkblue}{rgb}{0.0,0.0,0.75}
\definecolor{mygray}{gray}{0.90}
%\usepackage[%
%baseurl={http://www.bioconductor.org},%
%pdftitle={HiTC - High Throughput analysis of 'C' experiments},%
%pdfauthor={Nicolas Servant},%
%pdfsubject={HiTC Vignette},%
%pdfkeywords={Bioconductor},%
%pagebackref,bookmarks,colorlinks,linkcolor=darkblue,citecolor=darkblue,%
%filecolor=darkblue,urlcolor=darkblue,pagecolor=darkblue,%
%raiselinks,plainpages,pdftex]{hyperref}

%\usepackage[utf8]{inputenc} 
%\usepackage{verbatim} % for multi-line comments
%\usepackage{amsmath,a4,t1enc, graphicx}
\usepackage{natbib}
%\bibpunct{(}{)}{;}{a}{,}{,}

%\parindent0mm
%\parskip2ex plus0.5ex minus0.3ex

%\newcommand{\Robject}[1]{{\texttt{#1}}}
%\newcommand{\Rfunction}[1]{{\texttt{#1}}}
%\newcommand{\Rpackage}[1]{{\textit{#1}}}
%\newcommand{\Rclass}[1]{{\textit{#1}}}
%\newcommand{\Rmethod}[1]{{\texttt{#1}}}
%\newcommand{\Rfunarg}[1]{{\texttt{#1}}}
%\newcommand{\phead}[1]{{\flushleft \sf \small \textbf{#1} \quad}}
% \newcommand{\myincfig}[3]{%
%   \begin{figure}[h!tb]
%     \begin{center}
%       \includegraphics[width=#2]{#1}
%       \caption{\label{#1}\textit{#3}}
%     \end{center}
%   \end{figure}
% }
%\addtolength{\textwidth}{2cm}
%\addtolength{\oddsidemargin}{-1cm}
%\addtolength{\evensidemargin}{-1cm}
%\addtolength{\textheight}{2cm}
%\addtolength{\topmargin}{-1cm}
%\addtolength{\skip\footins}{1cm}


%%%%%%% START EDITING HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\begin{document}

\SweaveOpts{prefix.string = ./HiTC, eps=false, keep.source=TRUE, eval=TRUE, echo=TRUE} % produce no 'eps' figures
%\setkeys{Gin}{width=0.5\textwidth}

\title{\textbf{HiTC - Exploration of \textbf{Hi}gh \textbf{T}hroughput '\textbf{C}' experiments}}
\author{
\href{mailto:nicolas.servant@curie.fr}{Nicolas Servant}
}
\maketitle

%\tableofcontents
%\newpage

% \begin{center}
%     \fcolorbox{black}{mygray}{
%      \begin{minipage}[c]{1\textwidth}
%       \textbf{HiTC - Major Release}\\\\ Since version 1.5.2 the HiTC package now depends on the \Rpackage{Matrix} package. The main reason of this change is an improvment of the memory usage of the package in order to deal with recent high resolution Hi-C data. See section~\ref{sec:speedandmem} for details.
    
%      \end{minipage}}
% \end{center}
\vspace{2cm}

\section{Introduction}
Chromosome Capture Conformation (3C) was first introduced by \cite{Dekker2002a} ten years ago. The 3C technique aims in detecting physical contact between pairs of genomic loci and is now widely used to detect intrachromosomal (cis) and interchromosomal (trans) interactions between genes and regulatory elements. The development of the 3C-based techniques has changed our vision of the nulcear oragnization  (see \cite{Wit2012} for a review).\\
With the development of high throughput analyses, and in particular second-generation sequencing, the 3C has been adapted to study in parallel physical interactions between many loci, and thus increase the scale at which interactions between genomic loci can be detected (4C - Circular 3C, \cite{Simonis2006}, \cite{Zhao2006}; 5C - 3C Carbone Copy, \cite{Dostie2006}). More recently, this technique was further extended to obtain detailed insights into the general three-dimensional arrangements of complete genomes (Hi-C, \cite{Lieberman-Aiden2009}).

While the use of high-throuput 'C' techniques is expected to increase in the coming years, it also creates some new statistical and bioinformatics challenges. In this way, publicly available bioinformatics tools, as well as clear analysis strategy are still lacking. The \href{http://my5c.umassmed.edu/welcome/welcome.php}{my5C web browser} was proposed by \cite{Lajoie2009} to visualize, transform and analyze 5C data. However, the my5C webtool is targeted to end-users and biologists to prepare their 5C experiments and to handle their data but is not dedicated
to the development of new statistical algorithms.

The \Biocpkg{HiTC} R package has been developed to offer a bioinformatic environment to explore high-troughput 'C' data. One advantage of this package is that it operates within the open source Bioconductor framework, and thus, offers new opportunities for futur development in this field. The current version of the package provides the basic visualization, transformation and normalization functions described in \cite{Lajoie2009}, but also some new functionnalities such as data import, new visualization functions, annotation and other data transformation. Our goal is also to provide a flexible basis for further development, aiming at the integration of new analysis algorithm that are being developped (\cite{Yaffe2011}, \cite{Hu2012}, \cite{Imakaev2012})

This document briefly describes how to use the \Biocpkg{HiTC} R package. The package is built on the functionality of Bioconductor packages such as \Biocpkg{IRanges} and \Biocpkg{GenomicRanges}, and provides new classes and methods to handle with high-throughput 'C' data. It is especially suited to 5C and Hi-C data handling, but can also in principle be used for 4C, though specific needs of 4C users may be best met by \Biocpkg{r3Cseq} R package.\\
Even if the 5C and Hi-C approaches are derived from the same 3C technique, strong differences in their protocol can also be noticed. While 5C enables analysis of interactions between many loci, it also required an extensive number of primers, which is not suitable for a genome-wide analysis as the Hi-C. Thus, the pre-processing of these two types of data is totally different with, for instance, two different mapping strategies.

If you use \textit{HiTC} for analyzing your data, please cite:
\begin{itemize}
\item Servant N., Lajoie B.R., Nora E.P., Giorgetti L., Chen C., Heard E., Dekker J., Barillot E. (2012) HiTC : Exploration of High-Throughput 'C' experiments. \textit{Bioinformatics}.
\end{itemize}

\section{Getting started}

The current version of the \Biocpkg{HiTC} package was developped to work on processed 5C, Hi-C or other high-throughput 3C data.\\
The \Rclass{HTCexp} (High-Throughput 'C' experiment) class aims at representing a single 'C' experiment, characteriez by :
  \begin{itemize}
  \item An interaction map (i.e a \Rclass{Matrix})
  \item Two \Rclass{GRanges} objects that describe each features of the interaction matrix, respectively, the x (i.e. columns) and y (i.e. rows) labels of the interaction matrix. Basically, in the context of 5C, these objects will be the forward and reverse primers, and for the Hi-C the binned genomic intervals.
\end{itemize}

Note that \Biocpkg{HiTC} was not designed to processed chromatine conformation capture from raw reads, but takes contact maps as input. In order to process Hi-C data from raw sequencing reads, you can use the HiC-Pro pipeline \cite{Servant2015} which is freely available at \href{https://github.com/nservant/HiC-Pro}{https://github.com/nservant/HiC-Pro}. Data processed with HiC-Pro can then be loaded into the R environment using the importC function of \Biocpkg{HiTC}.

Whereas a 5C dataset can be composed of a single cis interaction map (i.e. \Rclass{HTCexp} object), a complete Hi-C dataset is composed of a list of cis and trans interaction maps, characterized by the physical interactions of each pair of chromosomes. The \Rclass{HTClist} class represents a list of \Rclass{HTCexp} objects and provides dedicated methods and visualization functions.

<<head>>=
library(HiTC)
showClass("HTCexp")
showClass("HTClist")
@ 

\section{Working with 'C' Data}
\Rclass{HTCexp} or \Rclass{HTClist} objects can be easily created using the dedicated constructors. Additional functions to import data from files are also available.

\subsection{A simple example}

<<importSimple>>=
require(Matrix)
## Two genome intervals objects with primers informations
reverse <- GRanges(seqnames=c("chr1","chr1"), 
                   ranges = IRanges(start=c(98831149, 98837507), 
                   end=c(98834145, 98840771), 
                   names=c("REV_2","REV_4")))

forward <- GRanges(seqnames=c("chr1","chr1"), 
                   ranges = IRanges(start=c(98834146, 98840772), 
                   end=c(98837506, 98841227), 
                   names=c("FOR_3","FOR_5")))

## A matrix of interaction counts
interac <- Matrix(c(8463, 7144, 2494, 8310), ncol=2)
colnames(interac) <- c("REV_2","REV_4")
rownames(interac) <- c("FOR_3","FOR_5")

z <- HTCexp(interac, xgi=reverse, ygi=forward)
detail(z)

## Access to the slots
x_intervals(z)
y_intervals(z)
intdata(z)

## Methods
range(z)
isBinned(z)
isIntraChrom(z)
seqlevels(z)
@ 

\subsection{Import/Export Data}

The HiTC package provides the \Rfunction{importC} and \Rfunction{exportC} functions to import/export data from a list file. Only non null values have the written in this file allowing a efficient storage of Hi-C (sparse) data.
The format is defined as follow :
\begin{itemize}
\item A list file (tab-separated) with per line, the name of both interactors and the number of associated sequencing reads (i.e. \verb?I1 I2 Count1-2?).
\item The associated \href{http://genome.ucsc.edu/FAQ/FAQformat.html#format1}{BED} files describing the x and y intervals of the HTCexp object. For 5C experiment, it can be the forward and reverse primers location, whereas for Hi-C experiment, it can be a description of the genomic bins. The name of these intervals must match with the name of the interactors in the list file.
 \end{itemize}

In addition, the package is fully compatible with the \href{http://my5c.umassmed.edu/welcome/welcome.php}{my5C web browser}. The interaction counts matrices can be imported/exported from a matrix file format. The matrix format summarizes all the informations with genomic coordinates as row and column names (ex: \verb?HIC_bin1|hg18|chr14:1-999999?). The row and column names are splitted to create the HTCexp object.

The \Biocpkg{HiTC} package includes a sample of the Human Hi-C dataset (\href{http://0-www.ncbi.nlm.nih.gov.elis.tmu.edu.tw/geo/query/acc.cgi?acc=GSE18199}{GSE18199}) published by \cite{Lieberman-Aiden2009}. The interaction map of chromosome 12 to 14 is used to illustrate the capabilities of the \Biocpkg{HiTC} package to explore Hi-C data.

<<importMatrix>>=
## Load Lieberman et al. Chromosome 12 to 14 data (from GEO GSE18199)
exDir <- system.file("extdata", package="HiTC")
l <- sapply(list.files(exDir, pattern=paste("HIC_gm06690_"), full.names=TRUE),
            import.my5C)
hiC <- HTClist(l)
show(hiC)
names(hiC)

## Methods
ranges(hiC)
range(hiC)
isBinned(hiC)
isIntraChrom(hiC)
isComplete(hiC)
seqlevels(hiC)
summary(hiC)
@

\section{Quality Control}
The first step after data pre-procesing is a quality control to check weither the data are likely to reflect cis and/or trans chromosomal interactions rather than just random collisions. Quality control for the percentage of reads aligned to interchromosomal and intrachromosomal interactions is available, as well as distribution of the interaction frequency against the genomic distance between two loci, and simple statistics (see Figure~\ref{HiTC-qcc}).

<<qcc, include=FALSE, fig=TRUE, width=7, height=7>>=
par(mfrow=c(2,2))
CQC(hiC, winsize = 1e+06, dev.new=FALSE, hist.dist=FALSE)
@

\incfig{HiTC-qcc}{0.6\textwidth}{Quality Control of \Robject{hiC} data.}{From top-left to bottom-right : proportion of intra/inter chromosomal interactions, scatter-plot of interaction counts versus genomic distance between two loci, histogram of interaction counts for intra (CIS) and inter (TRANS) interactions, histogram of distances between two intrachromosomal loci.}


\newpage
\section{HTCexp : single 'C' map experiment}

\subsection{Attached 5C data}
The \Biocpkg{HiTC} package includes a 5C dataset (\href{http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE35721}{GSE35721}) published by \cite{Nora2012a}, from which we choose two different Mouse samples, male undifferentiated ES cells (E14, GSM873935) and male embryonic fibroblasts (MEF, GSM873924). This dataset is mainly used to describe the available functionalities of the package.

<<importNora, echo=TRUE>>=
## Load Nora et al 5C dataset
data(Nora_5C)
show(E14)
show(MEF)
@ 

\subsection{Visualization of Interaction Maps}
The interaction map represents the frequency at which each pair of restriction fragments have been ligated together during the 3C procedure. The goal is to visualize at once these counts for many pairs of restriction fragments across a large genomic region. Each entry in the matrix corresponds to a count information, i.e., number of times two restriction fragments have been sequenced as a pair.\\
In the \Biocpkg{HiTC} package, the \Rclass{HTCexp} object are represented as a triangle view (see Figure~\ref{HiTC-mapC.png}). This view is particulary useful for interaction maps comparison and alignment with genomic or epigenomic features on a small region. The mapC function proposes a list of options to play with data visualization, such as contrast, color, or trimming.

<<mapC, eval=FALSE, echo=TRUE>>=
mapC(E14$chrXchrX)
@

<<norm5CEval, echo=FALSE, results=verbatim>>=
png(file="HiTC-mapC.png", res=300, units="in", width=5, height=5)
mapC(E14$chrXchrX)
graphics.off()
@

\incfig{HiTC-mapC.png}{0.6\textwidth}{Visualization of \Rclass{HTCexp} object (1).}{Raw 5C interaction map of chromosome X.}

\subsection{Data Transformation}
\subsubsection{Windowing}
Each pixel of an interaction map can correspond either to a single restriction fragment, several restriction fragments or genomic intervals of any given size (and therefore various restriction fragment numbers). 5C allows assessing interaction frequencies for each pair of restriction fragments. The Hi-C protocol, on contrary, does not necessarily yields counts for every single pair of restriction fragments, especially when working with large genomes. Results are thus typically displayed for genomic bins of an arbitrary size.\\
To produce an interaction map, the genomic range of the display should be divided into appropriately size loci. This size depends on the resolution desired for the analysis. For instance, 5C data can be visualized at the primers resolution, or segmented into 100Kb or 1Mb bins that can be partially overlap or not. Such binned interaction map is symmetrical around the diagonal. For the following example, we decided to focus on a subset of the original dataset (see Figure~\ref{HiTC-bin5C}).

<<bin5C, fig=TRUE, include=FALSE, width=7, height=7>>= 
## Focus on a subset chrX:100295000:102250000
E14subset<-extractRegion(E14$chrXchrX, c(1,2),
                         chr="chrX", from=100295000, to=102250000)

## Binning of 5C interaction map
E14subset.binned <- binningC(E14subset, binsize=100000, method="median", step=3)
mapC(E14subset.binned)
@

\incfig{HiTC-bin5C}{0.5\textwidth}{Visualization of \Rclass{HTCexp} object (2).}{Binned 5C interaction map of chrX:100295000-102250000.}

\subsubsection{Data Normalization}
Due to the polymer nature of chromatin, at small genomic distances, pairs of restriction fragments that are close to each other in the linear genome will give higher signal than fragments that are further apart. Such property leads to strongest counts falling on the heatmap diagonal. When considering any given pair of restriction fragments, it is therefore informative to assess whether the observed counts are above what is expected given the genomic distance that separate them.\\
Different ways of normalization have been proposed. Here, we propose to estimate the expected interaction counts as presented in \cite{Bau2011}. The expected value is the interaction frequency between two loci that one would expect based on a sole dependency on the genomic proximity of these fragments in the linear genome. This can be estimated using a Loess regression model (see Figure~\ref{HiTC-expint.png}). Note that another model based on mean counts at each genomic distance can also be used (method=mean)

<<norm5Cexp, eval=FALSE, echo=TRUE, fig=FALSE>>=
## Look at exptected counts
E14exp <- getExpectedCounts(E14subset.binned, method="loess", stdev=TRUE, plot=TRUE)
@ 

<<norm5CexpEval, echo=FALSE, results=verbatim>>=
png(file="HiTC-expint.png", res=300, units="in", width=6, height=6)
E14exp <- getExpectedCounts(E14subset.binned, method="loess", stdev=TRUE, plot=TRUE)
graphics.off()
@

\incfig{HiTC-expint.png}{0.5\textwidth}{Estimation of expected count using a Loess smoothing.}{The crosses represent the interpolation points.}

Interaction frequencies can be then normalized for distance by dividing the observed value by the expected value (\Rfunction{normPerExpected}). The variability between the interaction counts and the genomic distance between pairs of loci can be calculated if specified. These normalization methods can be easily applied using the methods \Rfunction{normPerReads} and \Rfunction{normPerExpected}.

<<norm5Cznorm, fig=TRUE, include=FALSE, width=7, height=7>>=
E14norm.binned <- normPerExpected(E14subset.binned, method="loess", stdev=TRUE)
mapC(E14norm.binned)
@

\incfig{HiTC-norm5Cznorm}{0.5\textwidth}{Normalized 5C data.}{Interaction map of data normalized from the background level of interactions.}

\subsubsection{Annotation of Interaction Maps}
The \Biocpkg{HiTC} package contains functions for visualizing genomic regions with interaction maps (see Figure~\ref{HiTC-annot5C}).
The annotation objects have to belong to the \Rclass{GRanges} class, cand can be loaded from \href{http://genome.ucsc.edu/FAQ/FAQformat.html#format1}{BED} files using the \Biocpkg{rtracklayer} package.
For instance, the following example displays the CTCF enriched regions (\cite{Kagey2010}) and RefSeq genes over the interaction map of the E14 sample.

<<annot5C, eval=TRUE, echo=TRUE, print=FALSE, fig=TRUE, width=7, height=7, include=FALSE>>=
E14.binned <- binningC(E14$chrXchrX, binsize=100000, method="median", step=3)
require(rtracklayer)
gene <- import(file.path(exDir,"refseq_mm9_chrX_98831149_103425150.bed"), 
               format="bed")
ctcf <- import(file.path(exDir,"CTCF_chrX_98892125_102969775.bed"), 
               format="bed")
mapC(E14.binned, 
     tracks=list(RefSeqGene=gene, CTCF=ctcf), 
     maxrange=10)
@ 
\incfig{HiTC-annot5C}{0.6\textwidth}{Visualization of interaction map and genomic annotations.}{CTCF enriched regions and RefSeq genes over the interaction map of the E14 sample.}

\subsection{Comparison of HTCexp objects}
 The \Biocpkg{HiTC} package provides methods to perform simple operations on \Rclass{HTCexp}, such as dividing, substracting two objects or extracting a genomic region.\\
 It also proposes a graphical view to compare two 'C' experiments. In the following example, the MEF sample is compared to the E14 sample (see Figure~\ref{HiTC-comp5C}).
 
<<comp5C, eval=TRUE, echo=TRUE, fig=TRUE, width=7, height=7, include=FALSE>>=
MEF.binned <- binningC(MEF$chrXchrX, binsize=100000, method="median", step=3)
mapC(E14.binned, MEF.binned,
     tracks=list(RefSeqGene=gene, CTCF=ctcf), 
     maxrange=10)
@ 
\incfig{HiTC-comp5C}{0.7\textwidth}{Comparison of interaction maps.}{Comparison of two binned interaction maps, and visualization with genomic annotations.}

\newpage
\section{HTClist : Multiple 'C' experiments}
\label{section:hic}

Basically, 5C and Hi-C data can be described in the same way. Thus, most of the functions and methods described for the 5C data can be applied to the Hi-C data.

\subsection{Visualization of Interaction Maps}
The visualization of the \Rclass{HTClist} is designed such as several interaction maps from the same experiment can be displayed together.\\
Therefore these data are typically displayed using two dimensional heatmaps of all cis/trans maps.

<<mapClist, eval=TRUE, echo=TRUE, fig=TRUE, width=7, height=7, include=FALSE>>=
mapC(hiC, maxrange=100)
@ 
\incfig{HiTC-mapClist}{0.5\textwidth}{Visualization of a Hi-C dataset.}{Two dimensional heatmaps of the cis/trans maps from the Liberman-Aiden et al. dataset.}

\subsection{Hi-C analysis}

In this section, we present how, using a few command lines, we can reproduce some analyses of the \cite{Lieberman-Aiden2009} paper (see Figures~\ref{HiTC-mapChic}-\ref{HiTC-mapCorhic}) on the chromosome 14, from visualization of maps to Principal Component Analysis (PCA).

<<mapChic, eval=TRUE, echo=TRUE, fig=TRUE, width=7, height=7, include=FALSE>>=
## Extract region of interest and plot the interaction map
hiC14 <- extractRegion(hiC$chr14chr14, 
                     chr="chr14", from=1.8e+07, to=106368584)
mapC(HTClist(hiC14), maxrange=100)
@ 
\incfig{HiTC-mapChic}{0.5\textwidth}{Hi-C interaction map of chromosome 14.}{}

<<mapNormhic, eval=TRUE, echo=TRUE, fig=TRUE, width=7, height=7, include=FALSE>>=
## Data Normalization by Expected number of Counts
hiC14norm <- normPerExpected(hiC14, method="loess")
mapC(HTClist(hiC14norm), log.data=TRUE)
@
\incfig{HiTC-mapNormhic}{0.5\textwidth}{Interaction map of chromosome 14 normalized by the expected interaction counts.}{}

<<mapCorhic, eval=TRUE, echo=TRUE, fig=TRUE, width=7, height=7, include=FALSE>>=
## Correlation Map of Chromosome 14
#intdata(hiC14norm) <- as(cor(as.matrix(intdata(hiC14norm))),"Matrix")
intdata(hiC14norm) <- HiTC:::sparseCor(intdata(hiC14norm))
mapC(HTClist(hiC14norm), maxrange=1, minrange=-1, 
     col.pos=c("black", "red"), col.neg=c("blue","black"))
@
\incfig{HiTC-mapCorhic}{0.5\textwidth}{Correlation map of chromosome 14}{}

<<mapPCAhic, eval=TRUE, echo=TRUE, fig=TRUE, width=7, height=7, include=FALSE>>=
## Principal Component Analysis
pc <- pca.hic(hiC14, normPerExpected=TRUE, method="loess", npc=1)
plot(start(pc$PC1), score(pc$PC1), type="h", 
     xlab="chr14", ylab="PC1vec", frame=FALSE)

@
\incfig{HiTC-mapPCAhic}{0.5\textwidth}{PCA analysis (Lieberman-Aiden et al.).}{Results of the PCA (eigenvector), which reflect the compartmentalization inherent in the heatmap.}

\newpage
\section{A word about speed and memory usage}
\label{sec:speedandmem}
In order to improve the run time on machines with multiple processors, some of the functions in the \Biocpkg{HiTC} package have been implemented to make use of the functionality of the \Rpackage{parallel} package. If the options \Robject{mc.cores} is initialised before calling these functions, they will make use of \Rfunction{mclapply} instead of the normal \Rfunction{lapply}.\\
Since the version 1.5.2 of the package, the interaction maps are now stored as \Rpackage{Matrix} object. In case of very high resolution data, such as the 20kb interaction maps published by \cite{Dixon2012}, a sparse matrix representation is much more efficient in terms of memory usage. The memory requires by the HiTC package for high resolution Hi-C data is represented in the figure~\ref{HiTC_memory_usage.png}. However, in many cases, using \Rclass{Matrix} objects instead of \Rclass{matrix} objects is much more slower. Thus, for some functions such as \Rfunction{binningC}, the user can now set the \Robject{optimize.by} argument to "speed" or "memory". If set to "speed", the \Rclass{Matrix} object is convert into a standard \Rclass{matrix} class, thus taking much more memory during the execution of the function but being much faster.
One can notice that for now, all the vizualition functions are based on \Rclass{matrix} object.

\incfig{HiTC_memory_usage.png}{0.5\textwidth}{HiTC memory usage.}{Improvement of the memory usage of the HiTC package through the storage of sparse matrix (\Rclass{Matrix}).}

\small
\section*{Package versions}
This vignette was generated using the following package versions:
<<sessionInfo, echo=FALSE, results=tex>>=
toLatex(sessionInfo(), locale=FALSE)
@

\section*{Acknowledgements}
Many thanks to Pierre Gestraud for useful discussion and help in developping this R package. A special thanks to the \Rpackage{HiTC} users, and especially to Sameet Mehta for useful discussions and idea to improve it.

\newpage
\small
%%% BIBLIOGRAPHY STARTS HERE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\bibliographystyle{abbrvnat}
\bibliography{HiTC}

%\begin{thebibliography}{}
%\end{thebibliography}

\end{document}

\name{setGenomicFeatures}
\Rdversion{1.1}
\alias{setGenomicFeatures}

\title{Annotation of Hi-C contact map}
\description{
  Annotate a Hi-C contact map with the genomic local features (i.e. GC content, mappability, effective fragment length)
}

\usage{setGenomicFeatures(x, cutSites, minFragMap=.5, effFragLen=1000)}

\arguments{
  \item{x}{\code{HTCexp} object to annotate}
  \item{cutSites}{\code{GRangesList} or \code{GRanges} object with restriction sites
    annotation obtained using the \code{getAnnotatedRestrictionSites}
    function}
  \item{minFragMap}{Minimum Fragment Mappbility. All fragments with a lower
    mappability are not used for the annotation.}
  \item{effFragLen}{Efective Fragment Length. Size of specific fragment ligation}
}


\details{
  The function require the restriction sites annotation as provided by
  the \code{getAnnotatedRestrictionSites} function.
  The restriction sites are first filtered according to their
  mappability. This threshold has to be defined according to the data
  pre-processing. All remaining restriction sites are then intersected
  with the genomic bins of the contact map. All restriction sites
  included within a bin are averaged.
  The effective fragment length is defined as the size of specific
  ligation product. (See Yaffe and Tanay, 2011). In this paper, the
  authors define specific ligation as sum of distance to cutter sites
  (d1+d2) <= 500 bp. Such criterion implies that d1<=500 bp and d2 <=
  500 bp. So for each fragment end, only reads mapped within 500 bp to
  cutter sites are used for downstream analysis.
  All defults paramters correspond to the ones used in the HiCNorm method.
  
}
\value{
  Returns a \code{HTCexp} object with local genomic features annotations.}
\author{N. Servant}
\seealso{\code{\link{normLGF}}, \code{\link{setGenomicFeatures}}}
\examples{

\dontrun{
require(BSgenome.Hsapiens.UCSC.hg18)
require(rtracklayer)

##Lieberman data
exDir <- system.file("extdata", package="HiTC")
l <- sapply(list.files(exDir, pattern=paste("HIC_gm06690_"), full.names=TRUE),
            import.my5C)
hiC <- HTClist(l)
hiC <- hiC[isIntraChrom(hiC)]
names(hiC)

## Mappability data From http://hgdownload.cse.ucsc.edu/goldenPath/hg18/encodeDCC/wgEncodeMapability/
map_hg18<- import("wgEncodeCrgMapabilityAlign100mer.bw", format="BigWig")

## Get the genomic feature of the HiC chr12 data
cutSites <- getAnnotatedRestrictionSites(resSite="AAGCTT", overhangs5=1, chromosomes=seqlevels(hiC), genomePack="BSgenome.Hsapiens.UCSC.hg18", wingc=200, mappability=map_hg18, winmap=500)

chr12_annot <- setGenomicFeatures(hiC$chr12chr12, cutSites)
}
}

\name{normLGF}
\Rdversion{1.1}
\alias{normLGF}


\title{Local Genomic Feature (LGF) normalization}
\description{
  Parametric model to remove systematic biases in the raw contact maps
}

\usage{normLGF(x, family=c("poisson", "nb"))}

\arguments{
    \item{x}{object that inherits from class \code{HTCexp}}
    \item{family}{parametric model to fit (poisson or nb)}
}

\details{
  This function implements the HiCNorm method proposed by Hu et al.
  Briefly, the method uses a generalized linear model to correct the
  systematic biases (effective fragment length, GC content, mappability)
  in a Hi-C contact map.
}
\value{
  Returns a \code{HTCexp} object with a normalized contact map.
}
\references{
Hu M, Deng K, Selvaraj S, Qin Z, Ren B, Liu JS. HiCNorm: removing biases in Hi-C data via Poisson regression. Bioinformatics. 2012;28(23):3131-3.
}
\author{N. Servant, M. Hu, S. Selvaraj}
\seealso{\code{\link{getAnnotatedRestrictionSites}}, \code{\link{setGenomicFeatures}}}
\examples{
\dontrun{
require(HiTC)
require(BSgenome.Hsapiens.UCSC.hg18)

##Lieberman data
exDir <- system.file("extdata", package="HiTC")
l <- sapply(list.files(exDir, pattern=paste("HIC_gm06690_"), full.names=TRUE),
            import.my5C)
hiC <- HTClist(l)
hiC <- hiC[isIntraChrom(hiC)]
names(hiC)

## Mappability data From http://hgdownload.cse.ucsc.edu/goldenPath/hg18/encodeDCC/wgEncodeMapability/
map_hg18<- import("wgEncodeCrgMapabilityAlign100mer.bw", format="BigWig")

## Get the genomic feature of the chromosome 12
hiC_annot <- HTClist(lapply(hiC, setGenomicFeatures, resSite="AAGCTT", overhangs5=1, genomePack="BSgenome.Hsapiens.UCSC.hg18", wingc=200, mappability=map_hg18, winmap=500))
hiC_annot$chr12chr12

## Normalize the data
hiCnorm <- HTClist(lapply(hiC_annot, normLGF))

}
}

\name{HTClist-class}
\Rdversion{1.1}
\docType{class}
\alias{HTClist-class}
\alias{HTClist}
\alias{c,HTClist-method}
\alias{detail,HTClist-method}
\alias{as.list,HTClist-method}
\alias{forcePairwise}
\alias{forcePairwise,HTClist-method}
\alias{forceSymmetric,HTClist,character-method}
\alias{forceSymmetric,HTClist,missing-method}
\alias{getCombinedContacts}
\alias{getCombinedContacts-method}
\alias{getCombinedContacts,HTClist-method}
\alias{getCombinedIntervals}
\alias{getCombinedIntervals-method}
\alias{getCombinedIntervals,HTClist-method}
\alias{isComplete}
\alias{isComplete-methods}
\alias{isComplete,HTClist-method}
\alias{isPairwise}
\alias{isPairwise-method}
\alias{isPairwise,HTClist-method}
\alias{isBinned,HTClist-method}
\alias{isIntraChrom,HTClist-method}
\alias{normPerExpected,HTClist-method}
\alias{plot,HTClist,ANY-method}
\alias{range,HTClist-method}
\alias{ranges}
\alias{ranges-methods}
\alias{ranges,HTClist-method}
\alias{reduce}
\alias{reduce-methods}
\alias{reduce,HTClist-method}
\alias{show,HTClist-method}
\alias{seqlevels,HTClist-method}
\alias{summary,HTClist-method}
\alias{[,HTClist,ANY,ANY-method}
\alias{[,HTClist,ANY,ANY,ANY-method}


\title{Class 'HTClist'}
\description{
  A class for representing a list of high throughput Chromosome Conformation Capture data from next-generation sequencing experiments.
}

\section{Constructor}{
  \describe{
    The \code{HTClist} represents a list of \code{HTCexp} objects and can
    be created as follow :
    \item{}{
      \code{HTClist(...)} : Creates a HTClist object using HTCexp objects supplied in '...'
    }
  }
}

\section{Methods}{
  \describe{
    \item{c(x, ...)}{Combines a \code{signature("HTClist")} object 'x'
      with \code{signature("HTClist")} or \code{signature("HTCexp")} 
      objects in '...'. The results is an object of class \code{signature("HTCList")}}
    \item{detail(x)}{\code{signature("HTClist")}: a more
      detailed output of the experiment than provided by \code{show}.}
    \item{forcePairwise(x)}{return a \code{signature("HTClist")} with
      all the pairwise contact maps}
    \item{forceSymmetric(x)}{return a \code{signature("HTClist")} with
       half of the pairwise contact maps}
     \item{isComplete(x)}{Logical; true if 'x' contains all intra and
      interchromosomal maps}
    \item{isPairwise(x)}{Logical; true if 'x' contains all interchromosomal
      pairs, i.e. chr1chr2 and chr2chr1}
    \item{isBinned(x)}{applies 'isBinned' to each element in 'x'}
    \item{isIntraChrom(x)}{applies 'isIntraChrom' to each element in
      'x'}
    \item{getCombinedContacts(x)}{merge all contact maps in a single big matrix}
    \item{getCombinedIntervals(x, merge=FALSE)}{merge all x and y intervals in single
  GRangesList object, or in a single GRanges object if merge=TRUE}
    \item{normPerExpected(x)}{normalized by genomic distance all
  intra-chromosomal maps. See details}
    \item{ranges(x)}{applies 'range' to each element in 'x'}
    \item{range(x)}{return the reduce range of all elements in 'x'}
    \item{reduce(x, chr, cis=TRUE, trans=TRUE, extend=FALSE)}{reduce a HTClist object
  to the list of provided chromosomes. Intra/Interchromosomal maps are
  returned accoring to the cis and trans args. If extend = True, all
  maps involving one of the chromosomes are returned}
    \item{seqlevels(x)}{return the sequence levels of all elements in 'x'}
    \item{as.list(x)}{coercion to simple \code{list} object}
    \item{names(x)}{get the names of the elements}
    \item{show(x)}{summarized output of the experiment, with informations
      about the data dimension}
    \item{summary(x)}{return descriptive summary statistics for each interaction map}
    \item{x[i]}{Get elements \code{i} from \code{x}. Can be the
    positional index or its name.} 
  }
}
\author{Nicolas Servant}

\details{
  A \code{signature("HTClist")} is composed of a list of contact maps,
  representing the chromosomal interactions between pair of
  chromosomes.\cr
  The expected number of maps for a complete
  \code{signature("HTClist")} object should be equal to
  'lchrs+(lchrs*(lchrs-1)/2)'. In this case, the chr1-chr2 map is stored
  once, but the dataset is complete.\cr
  If a \code{signature("HTClist")} object is composed of all pairwise
  interaction maps, it means that the chr1-chr2 AND the chr2-chr1 maps
  will be stored. This way of storing the data is less memory efficient
  but can ease the use of some genome-wide algorithm.\cr
  Note that the \code{getCombinedContact} method should be used
  carefully. This method merges all single contact maps in one
  genome-wide map, therefore creating a very big matrix requiring memory
  space. However, this method is useful for many genome-wide analysis.
  The \code{normPerExpected} method applied to HTClist object is only
  available with the 'mean' method (see \code{getExpectedCounts} for
  details). In this case, the mean counts per distance are calculated
  over all intra-chromosomal maps.
}

\seealso{
  \code{\link[GenomicRanges]{GRangesList-class}}, \code{\link{HTCexp-class}}
}

\examples{
exDir <- system.file("extdata", package="HiTC")
l <- sapply(list.files(exDir, pattern=paste("HIC_gm06690_"), full.names=TRUE),
            import.my5C)
hiC <- HTClist(l)
names(hiC)

## Methods
ranges(hiC)
range(hiC)
isComplete(hiC)
isPairwise(hiC)
isBinned(hiC)
isIntraChrom(hiC)
seqlevels(hiC)
}
\keyword{classes}

\name{exportC}
\Rdversion{1.1}
\alias{exportC}
\title{Export \code{HTCexp} object}
\description{
  Export \code{HTCexp} object to tab format
}
\usage{exportC(x, file, per.chromosome=FALSE, use.names=FALSE, header=FALSE)}
\arguments{
  \item{x}{object that inherits from class \code{HTCexp}}
  \item{file}{character; the basename of the output file}
  \item{per.chromosome}{logical; export each contact maps in a different files (i.e one per chromosome pair)}
  \item{use.names}{if TRUE, keep the original row/colnames of the contact matrix}
  \item{header}{if TRUE, add an header with the package version and the date}
}
\value{
 Three output files will be created ; 2 BED files for each genomic
 intervals, and one tab file.
 The standard format for 5C/Hi-C data is the following :\cr
 ** One list file (tab delimited)
 bin1 bin2 x12
 bin1 bin3 x13
 ...
 ** The BED file(s) describing the intervals ('xgi.bed' and 'ygi.bed'
 are usually the same for Hi-C but can be different for 5C data)
 chr1 1 1000000 bin1
 chr1 1000001 2000000 bin2
 ...
 Note that this format is particularly interesting for sparse
 data as only non null values are stored.
 If per.chromosome=FALSE, the data will be exported in one genome scaled file.
}
\author{N. Servant}
\seealso{\code{\link{export.my5C}}, \code{\link{importC}}}
\examples{
\dontrun{
data(Nora_5C)

## Data binning
E14.bin<-binningC(E14$chrXchrX)

## Export the new intervals definition
exportC(E14.bin, file="E14")
}
}
\keyword{IO}

\name{HiTC-internal}
\alias{colorC}
\alias{getDeltaRange}
\alias{tricube}
\alias{gdiag}
\alias{addImageTracks}
\alias{getData2Map}
\alias{heatmapC}
\alias{triViewC}
\alias{pair.chrom}
\alias{my5C2gr}

\title{Internal HiTC functions}
\description{
  Called internally by other \code{HiTC} functions.
  In theory, not called by the user.
}
\author{Nicolas Servant}
\seealso{\code{\link{HTCexp-class}}, \code{\link{HTClist-class}}}
\keyword{internal}

\name{getPearsonMap}
\Rdversion{1.1}
\alias{getPearsonMap}
\title{Pearson correlation map}
\description{
  Generate pearson correlation map, usually used to call
    chromosomal compartments
}
\usage{getPearsonMap(x, normPerExpected=TRUE, center=TRUE, ...)}
\arguments{
  \item{x}{object that inherits from class \code{HTCexp}}
  \item{normPerExpected}{normalized by expected interaction using the loess
    calculation of distance dependency. see normPerExpected}
  \item{center}{default=true. center the observed/expected map before calculating the
    Pearson correlation}
  \item{...}{additional parameters passed to \code{normPerExpected}}
}

\details{
  The function returns an \code{HTCexp} object with Pearson correlation
  map. This is usually the first step of the Principal Component
  Analysis (see \code{pca.hic}).
  Centering the rows of the observed/expected matrix allows to avoid
  bias to due to ranges of interaction counts.
  If true, the correlation of small values should be as valuable as
  correlation of large values
}

\value{
  A \code{HTCexp} object
}
\seealso{\code{\link{normPerExpected}}}
\author{N. Servant, B. Lajoie, R. McCord}
\examples{
## Get Lieberman-Aiden Hi-C data
exDir <- system.file("extdata", package="HiTC")
l <- sapply(list.files(exDir, pattern=paste("HIC_gm06690_"), full.names=TRUE),
import.my5C)
hiC <- HTClist(l)

## get Pearson correlation map
pm <- getPearsonMap(hiC$chr14chr14)
mapC(HTClist(pm), maxrange=1, col.pos=c("black","red"), col.neg=c("black","blue"))
}
\keyword{manip}

\name{mapC}
\Rdversion{1.1}
\alias{mapC}
\alias{mapC-methods}
\alias{mapC,HTClist,ANY-method}
\alias{mapC,HTCexp,ANY-method}
\alias{mapC,HTCexp,HTCexp-method}

\title{Visualize 'C' ontact map}
\description{
  Visualize 'C' contact map
}

\section{For \code{HTCexp} and \code{HTClist} objects}{
  \describe{
    \item{x}{object that inherits from class \code{HTCexp} or \code{HTClist}}
    \item{tracks}{List of GRanges objects of data to display as annotation track(s)}
    \item{minrange}{the minimum range of values used to define the color palette}
    \item{maxrange}{the maximum range of values used to define the color palette}
    \item{trim.range}{define the maxrange and minrange values using the
      percentile of the interaction matrix}
    \item{show.zero}{logical; plot the zero values}
    \item{show.na}{logical; show the NA values in gray}
    \item{log.data}{logical; do you want to log the data before plotting
      the heatmap}
    \item{col.pos}{color for (low,mid,high) positive contact counts. Must
      be a vectore of size 3. mid can be NA}
    \item{col.neg}{color for (low,mid,high) negative contact counts. Must
      be a vectore of size 3. mid can be NA}
    \item{col.na}{color for NA values}
      \item{grid}{logical; add a grid on the heatmap}
      \item{title}{character; add a title to the \code{HTCexp} plot(s)}
      \item{value}{logical; display the contact values on the
	matrix. Useful for small matrices}
    }
}
\section{For \code{HTCexp} objects only}{
\describe{
  \item{y}{optional. object that inherits from class \code{HTCexp}.}
 }
}
\section{For \code{HTClist} objects only}{
\describe{
  \item{names}{logical; display the names of the intervals. Useful for
      small matrices}
  }
}


\details{
  This function implements the \code{plot} method for objects
  of class \code{HTCexp} and \code{HTClist}.
  
  By default, the trim.range value is fixed so that the 98th
  percentile (resp. 2th percentile) of each interaction matrix is discarded. 
  It therefore allow to remove the extreme values from the matrix, but
  each map is plotted independently.
  If the maxrange argument is set, data higher that this threshold will
  be fixed to the maxrange value for all maps. In addition, color ranges
  are ajusted in a way that all maps are plotted within the same color
  range allowing visual maps comparison.
  

  The \code{HTCexp} and \code{HTClist} are not represented in the same
  way. The heatmap view is used to display the \code{HTClist} objects in
  two dimension. This view is mainly useful to have an overview of the
  data, as Hi-C data.
  The triangle view is used for \code{HTCexp} only and represent
  the top-right part the interaction matrix. If two \code{HTCexp}
  objects are specified, they will be displayed in order to compare both
  contact maps. The two maps have to be binned to ensure comparison
  between genomic ranges.

  Annotation tracks can be added to both views. In case of binned data,
  the exact genomic positions of each features are takken into
  account. Otherwise, the 'C' intervals which overlap with the
  annotation features are colored.
}
\value{
  Returns \code{NULL}; this function is called for the side-effect of
  creating the plot.
}
\author{N. Servant, B. Lajoie}
\seealso{\code{\link{HTCexp-class}}, \code{\link{HTClist-class}}}
\examples{
data(Nora_5C)

## Contact map
## HTClist view
mapC(E14)

## HTCexp view
mapC(E14$chrXchrX)

## Play with contrast and color
mapC(E14$chrXchrX, maxrange=100, col.pos=c("black","red","yellow"))

## Add annotation and change view
require(rtracklayer)
exDir <- system.file("extdata", package="HiTC")
gene <- import(file.path(exDir,"refseq_mm9_chrX_98831149_103425150.bed"), format="bed")
mapC(E14$chrXchrX, tracks=list(Refseq=gene))

## Compare two samples
mapC(binningC(E14$chrXchrX), binningC(MEF$chrXchrX), tracks=list(Refseq=gene))

}
\keyword{hplot}

\name{importC}
\Rdversion{1.1}
\alias{importC}
\title{Import high-htroughput 'C' data}
\description{
  Import 5C or Hi-C data from list file
}

\usage{importC(con, xgi.bed, ygi.bed=NULL, allPairwise=FALSE,
rm.trans=FALSE, lazyload=FALSE)}
\arguments{
  \item{con}{input csv file. See details}
  \item{xgi.bed}{BED file describing the 'x' Intervals (i.e. column
    names) of the contact map}
  \item{ygi.bed}{BED file describing the 'y' intervals (i.e. row names) of the
    contact map}
  \item{allPairwise}{logical; generate all pairwise chromosomal contact
    maps, i.e chr1-chr2, chr2-chr1}
  \item{rm.trans}{if true, returns only intra-chromosomal maps}
  \item{lazyload}{logical; see details}
}
\details{
  This function import high-throughput data from a tab list file.\cr
  The standard format for 5C/Hi-C data is the following :\cr
  ** One list file (tab delimited)
  bin1 bin2 x12
  bin1 bin3 x13
  ...
  ** The BED file(s) describing the intervals ('xgi.bed' and 'ygi.bed'
  are usually the same for Hi-C but can be different for 5C data)
  chr1 1 1000000 bin1
  chr1 1000001 2000000 bin2
  ...

  Note that this format is particularly interesting for sparse
  data as only non null values are stored.

  The lazyload option allow to reduce the memory size of imported
  object. Therefore, only half of inter-chromosomal maps are stored. And
  intra-chromosomal maps are stored as sparse triangular matrix.
  Note that even if the contact maps are stored as triangular matrix,
  the indata method always returns a symmetrical map.
}

\value{
  A \code{HTClist} object
}
\author{N. Servant}
\seealso{\code{\link{exportC}},\code{\link{import.my5C}}, \code{\link{HTCexp-class}}}
\examples{
\dontrun{
data(Nora_5C)

## Data binning
E14.bin<-binningC(E14$chrXchrX)

## Export the new intervals definition
exportC(E14.bin, file="E14")

##Import
importC(con="E14.count", xgi="E14_xgi.bed", ygi="E14_ygi.bed")
}
}
\keyword{IO}

\name{getExpectedCounts}
\Rdversion{1.1}
\alias{getExpectedCounts}
\title{Estimate expected interaction counts of a High-Throughput C
  intrachromsomal map based on the genomic distance between two loci
}
\description{
The expected interaction is defined as the linear relationship between
the interaction counts and the distance between two loci. See details for additional informations.
}
\usage{getExpectedCounts(x, method=c("mean","loess"), asList=FALSE, ...)}
\arguments{
  \item{x}{object that inherits from class \code{HTCexp}}
  \item{method}{method used to estimate the expected counts based on the
    genomic distance. See details}
  \item{asList}{return the results as a list. Otherwise, return an
    object of class \code{HTCexp}}
  \item{...}{arguments for mean or loess method, see below}
}

\section{For 'mean' method}{
  \describe{
    \item{logbin}{if true logarithm based bins are used. In practice, it
  means that the bin size will change as we move away from the diagonal}
    \item{step}{multiplicative factor between each bin. Use if logbin is
  true}
    \item{filter.low}{fraction of low counts to filter before
  normalizing. default: 0.05.}
  }
}
\section{For 'loess' method}{
  \describe{
    \item{span}{fraction of the data used for smoothing at each x point.}
    \item{bin}{interpolation parameter}
    \item{stdev}{logical, calculate the variance}
    \item{plot}{logical, display lowess smoothing and variance estimation points}
  }
}

\details{
The expected value is the interaction frequency between two loci that one would expect
based on a sole dependency on the genomic proximity of these fragments in the linear
genome. This can be estimated using two different methods, mean or loess.

The first method (default) is simply based on the mean counts of each
diagonal. If logbin is false, the expected counts will be estimated for
each bin of the contact maps. If logbin is true, the binsize will change
according to the distance to the diagonal. Short (resp. long) distance will be
estimated with smaller (larger) bins. This method works for all resolutions.

The second method is based on a Lowess regression model and works
usually fine with low resolution data (250Kb to 1Mb). At higher
resolution, the lowess regression might be difficult to fit.
The lowess smoothing has two parameters : span and bin.
The span corresponds to the fraction of the data used for smoothing.
Instead of computing the local polynomial fitting at each data point, a
window of size delta (bin parameter) is applied on the data and a linear interpolation
is used to fill in the fitted values within the window. 
The default is 1\% of the range of x. If delta=0 all but identical x values are estimated independently.
The variance is then estimated using the same span and bin parameter, at each interpolation points.
The points inside a window are weighted so that nearby points get the most weight (tricube weight function). 
}

\value{
  A list with the expected interaction map and the estimated variance
}
\seealso{\code{\link{HTCexp-class}},\code{\link{normPerExpected}},
  \code{\link{normPerExpected}}, \code{\link{lowess}}}
\author{N. Servant, B. Lajoie}
\examples{
data(Nora_5C)

## Estimate expected interaction from distance between intervals
E14.exp<-getExpectedCounts(E14$chrXchrX, method="loess", stdev=TRUE, plot=FALSE)
mapC(E14.exp)
}
\keyword{manip}

\name{discretize}
\Rdversion{1.1}
\alias{discretize}
\title{Transform matrix of counts data into discrete matrix}
\description{
  Transform matrix of counts data into discrete matrix
}
\usage{discretize(x, nb.lev=4, quant=TRUE)}
\arguments{
  \item{x}{data matrix}
  \item{nb.lev}{number of discretization level}
  \item{quant}{logical; use quantile distribution or split data into equals 'nb.lev' levels}
}
\value{
  A discrete matrix
}
\author{N. Servant}
\seealso{\code{quantile}}
\examples{
\dontrun{
data(Nora_5C)

## Data binning
E14bin<-binningC(E14$chrXchrX)

## Discretize matrix
dismat<-discretize(intdata(E14bin))
mapC(dismat)
}
}
\keyword{manip}

\name{removeIntervals}
\Rdversion{1.1}
\alias{removeIntervals}
\title{Remove intervals from HTCexp object}
\description{
  Remove primers intervals from HTCexp object
}
\usage{removeIntervals(x, ids)}
\arguments{
  \item{x}{object that inherits from class \code{HTCexp}}
  \item{ids}{character; vector of primers Ids to remove from the object}
}
\value{
  A \code{HTCexp} object without the discarded intervals
}
\author{N. Servant}
\seealso{\code{\link[GenomicRanges]{GRanges-class}}}
\examples{
data(Nora_5C)

## Remove intervals from a HTCexp object 
removeIntervals(E14$chrXchrX, ids=c("5C_938_XIC-3_REV_2", "5C_938_XIC-3_REV_4"))
}
\keyword{manip}

\name{HTCexp-class}
\Rdversion{1.1}
\docType{class}
\alias{HTCexp-class}
\alias{HTCexp}
\alias{c,HTCexp-method}
\alias{detail}
\alias{detail-methods}
\alias{detail,HTCexp-method}
\alias{divide}
\alias{divide-methods}
\alias{divide,HTCexp,HTCexp-method}
\alias{forceSymmetric,HTCexp,character-method}
\alias{forceSymmetric,HTCexp,missing-method}
\alias{forceTriangular}
\alias{forceTriangular-method}
\alias{forceTriangular,HTCexp-method}
\alias{intdata}
\alias{intdata-method}
\alias{intdata,HTCexp-method}
\alias{intdata<--methods}
\alias{intdata<-}
\alias{intdata<-,HTCexp,Matrix-method}
\alias{isBinned}
\alias{isBinned-methods}
\alias{isBinned,HTCexp-method}
\alias{isIntraChrom}
\alias{isIntraChrom-methods}
\alias{isIntraChrom,HTCexp-method}
\alias{isSymmetric}
\alias{isSymmetric-methods}
\alias{isSymmetric,HTCexp-method}
\alias{isTriangular-methods}
\alias{isTriangular,HTCexp-method}
\alias{id}
\alias{id-methods}
\alias{id,GRanges-method}
\alias{normPerReads}
\alias{normPerReads-methods}
\alias{normPerReads,HTCexp-method}
\alias{normPerExpected}
\alias{normPerExpected-methods}
\alias{normPerExpected,HTCexp-method}
\alias{normPerTrans}
\alias{normPerTrans-methods}
\alias{normPerTrans,HTCexp,HTCexp,HTCexp-method}
\alias{plot,HTCexp,ANY-method}
\alias{plot,HTCexp,HTCexp-method}
\alias{range}
\alias{range-method}
\alias{range,HTCexp-method}
\alias{seqlevels}
\alias{seqlevels-method}
\alias{seqlevels,HTCexp-method}
\alias{show,HTCexp-method}
\alias{substract}
\alias{substract-methods}
\alias{substract,HTCexp,HTCexp-method}
\alias{summary,HTCexp-method}
\alias{x_intervals}
\alias{x_intervals-methods}
\alias{x_intervals,HTCexp-method}
\alias{x_intervals<-}
\alias{x_intervals<--methods}
\alias{x_intervals<-,HTCexp,GRanges-method}
\alias{y_intervals,HTCexp-method}
\alias{y_intervals}
\alias{y_intervals-methods}
\alias{y_intervals,HTCexp-method}
\alias{y_intervals<-}
\alias{y_intervals<--methods}
\alias{y_intervals<-,HTCexp,GRanges-method}
\alias{xy_intervals,HTCexp-method}
\alias{xy_intervals}
\alias{xy_intervals-methods}

% Old stuff (deprecated or defunct).
\alias{export-defunct}
\alias{normPerZscore-defunct}
\alias{seq_name-deprecated}

\title{Class 'HTCexp'}
\description{
  A class for representing high throughput Chromosome Conformation Capture data from next-generation sequencing experiments.
}
\section{Objects from the Class}{
  Objects can be created either by:
  \enumerate{
    \item calls of the form
    \code{new("HTCexp", intdata, GRanges, GRanges)}.
    \item using the auxiliary function \code{HTCexp} and supplying
    contact Matrix with x and y intervals definition.
    The forceSymmetric option can used to force intra-chromosomal
    contact maps to be stored as symmetrical matrix.
  }
}
\section{Slots}{
	 \describe{
    \item{\code{intdata}:}{Dense or Sparse Matrix, holding the
      interaction level between each pairs of 'x-y' intervals. The 'y' intervals must be in rows, and the 'x' in columns.}
    \item{\code{ygi}:}{Genomic ranges of y intervals;
      see class \code{granges} for details}
    \item{\code{xgi}:}{Genomic ranges of x intervals;
      see class \code{granges} for details}
  }
}

\section{Methods}{
  \describe{
    \item{c(x, ...)}{Combines 'x' and the \code{signature("HTCexp")}
  objects in '...' together. The results is an object of class  \code{signature("HTCList")}}
    \item{detail(x)}{\code{signature("HTCexp")}: a more
      detailed output of the experiment than provided by \code{show}.}
    \item{divide(x)}{comparison of two \code{signature("HTCexp")}
      objects. Perform the division of the two contact matrices on the
      common 'x' and 'y' intervals. The operation is done only on the
      common intervals of both objects. If one of the two objects has a count
      to zero, the divided value will be NA}
    \item{intdata(x)}{return the \code{intdata} Matrix counts. Note that
  triangular matrices are always returned as symmetric matrices.}
    \item{export(x)}{Defunct. See exportC method}
    \item{isBinned}{return TRUE if the data are binned. The method tests
      if the 'x' and 'y' genome intervals are the same, if 90\% of the bins have the
      same size and if the full genomic range is covered}
    \item{forceSymmetric(x)}{force the interaction data to
      'symmetricMatrix'}
    \item{forceTriangular(x)}{force the interaction data to triangular,
  ie. symmetric. Lower triangle of the matrix is set to zero, therefore
  reducing the size of the data in memory}
    \item{isIntraChrom(x)}{return TRUE if the current
      \code{signature("HTCexp")} object contains intrachromosomal contact data}
    \item{isSymmetric(x)}{return TRUE if the contact map is
      symmetrical, i.e inherits the \code{symmetricMatrix} class}
    \item{normPerReads(x)}{normalize the contact matrix by the total
      number of reads of the matrix.}
    \item{normPerExpected(x, ...)}{normalize the contact matrix by the expected
      number of reads based on the distance between two loci. See details.}
    \item{normPerZscore(x)}{Defunct. See normPerExpected method}
    \item{normPerTrans(x, xtrans, ytrans, method="sum")}{Normalize cis contact map based on the trans
      interactions. See details}
    \item{plot(x)}{visualization method; Display an heatmap of the
      contact data. Refer to the documentation of
      \code{\link{mapC}} for more details of the plotting function}
    \item{range(x)}{return the genomic range of the
      \code{signature("HTCexp")} object} 
    \item{seq_name(x)}{Defunct. See seqlevels method} 
    \item{seqlevels(x)}{return the sequence levels of the \code{signature("HTCexp")} object} 
    \item{show(x)}{summarized output of the experiment, with informations
      about the data dimension and the genomic region studied}
    \item{substract(x)}{comparison of two \code{signature("HTCexp")}
      objects. Perform the substraction of the two contact matrices on the
      common 'x' and 'y' intervals. The operation is done only on the
      common intervals of both objects. If one of the two objects has a count
      to zero, the divided value will be NA}
    \item{summary(x)}{return descriptive summary statistics about the contact map}
    \item{x_intervals(x)}{return the \code{xgi} GRanges object defining the
      x intervals}
    \item{y_intervals(x)}{return the \code{ygi} GRanges object defining the
      y intervals}
    \item{xy_intervals(x)}{return both \code{xgi} and \code{ygi} objects as a
      \code{GRangesList} object}
  }
}
\author{Nicolas Servant}
\details{
  The \code{normPerExpected} method estimates the expected interactions based on a the dependency on the genomic
  proximity between two loci. See \code{\link{getExpectedCounts}} function for details.\cr
  
  The \code{normPerTrans} method is based on the assumption that all trans contacts should be the same. Thus, the cis contacts can be normalized by the interaction level of trans data.
  The \code{xtrans} trans map has to share its 'xgi' ranges with the
  cis map, and the \code{ytrans} has to share its 'ygi' ranges with
  the cismap.
  The \code{method} is used to combine the normalization factor from x
  and y ranges. Must be \sQuote{sum}, \sQuote{mult} or \sQuote{mean}.

}

\seealso{
  \code{\link[GenomicRanges]{GRanges-class}},\code{\link[GenomicRanges]{GRangesList-class}},\code{\link[Matrix]{Matrix-class}}
}

\examples{
data(Nora_5C)

## HTCexp descriptio
show(E14)
detail(E14)

## Is binned data ?
isBinned(E14$chrXchrX)

## Is a inter or intrachromsomal experiment ?
isIntraChrom(E14$chrXchrX)

## Bin the data
E14.bin <- binningC(E14$chrXchrX, binsize=100000, step=3)

## Divide by expected interaction counts
E14norm<-normPerExpected(E14.bin)

## Operation on HTCexp object
E14_d_MEF<-divide(normPerReads(E14$chrXchrX), normPerReads(MEF$chrXchrX))
E14_s_MEF<-substract(normPerReads(E14$chrXchrX), normPerReads(MEF$chrXchrX))

## Overlap with genomic annotation
require(rtracklayer)
gene <- import(file.path(system.file("extdata", package="HiTC"),"refseq_mm9_chrX_98831149_103425150.bed"), format="bed")
plot(E14$chrXchrX, tracks=list(RefSeqGene=gene))

\dontrun{
## normPerTrans data normalization applied on \href{http://genome.ucsc.edu/cgi-bin/hgFileUi?db=hg19&g=wgEncodeUmassDekker5C}{ENCODE data}.
ENCODE=import.my5C("./ENM-GM12878-R1.matrix")

## Look at raw contact map
mapC(ENCODE$chr7chr7)

## look at normalize by trans contact map
mapC(normPerTrans(ENCODE$chr7chr7, xtrans=ENCODE$chr7chr5, ytrans=ENCODE$chr5chr7))
}

\dontrun{
## Export
exportC(E14$chrXchrX, con="E14.csv")
}
}
\keyword{classes}

\name{CQC}
\Rdversion{1.1}
\alias{CQC}
\title{Quality Control for high-throughput 'C' experiment}
\description{
  Quality Control for high-throughput 'C' experiment
}
\usage{CQC(x,  cis.trans.ratio = TRUE, hist.interac=TRUE, scat.interac.dist=TRUE,
hist.dist=TRUE, trim.range=0.98, winsize=NA, dev.new=TRUE)}
\arguments{
  \item{x}{object that inherits from class
    \code{HTCexp} or \code{HTClist}}
  \item{cis.trans.ratio}{logical; barplot of percentage of
    inter-intrachromsomal interactions}
  \item{hist.interac}{logical; histogram of the interaction frequency}
  \item{scat.interac.dist}{logical; scatter plot of interaction count
    versus the genomic distance between two elements}
  \item{hist.dist}{logical; histogram of the distance between the 'x'
    and 'y' intervals}
  \item{trim.range}{remove the extreme values by trimming the counts. Only use for plotting functions. [0,1]}
  \item{winsize}{used for the scat.interac.dist. If specify, the data
    are windowed before plotting}
  \item{dev.new}{if true, each graphs is plotted in a new device}
}
\details{
If \code{x} is a \code{HTClist} object, all \code{HTCexp} objects are merged.
The zero values are not used to compute the descriptive statistics and to display the data.
If \code{trim.range} are lower than 1. The highest values (quantile probability is equal to \code{trim.range}) are discarded.
}
\value{
  Create quality plots and return a \code{matrix} with some simple statistics on all, cis and trans data.
}
\author{N. Servant, B. Lajoie}
\seealso{\code{\link{HTCexp-class}}}
\examples{
data(Nora_5C)

## Quality Control
CQC(E14)
}
\keyword{manip}

\name{Nora_5C}
\Rdversion{1.1}
\alias{Nora_5C}
\alias{E14}
\alias{MEF}
\docType{data}
\title{
HiTC - 5C data
}
\description{
  5C data described by Nora et al. (2012)
}
\usage{data(Nora_5C)}
\format{
  Contains two \code{HTClist} objects (E14 and MEF). Each of them
  containing the ChrX intrachromosomal maps as a \code{HTCexp} object.
}
\details{
This 5C dataset published by Nora et al
(\href{http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE35721}{GSE35721}),
contains two different samples, a male undifferentiated ES cells (E14,
GSM873935) and a mouse embryonic fibroblasts (MEF, GSM873924). This
dataset is mainly used to describe the available functionalities of the
\code{HiTC} package.
The data provided with the package are count data.
}
\source{
http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE35721
}
\references{
Nora EP, Lajoie BR, Schulz EG, Giorgetti L et al. Spatial partitioning of the regulatory landscape of the X-inactivation centre. Nature 2012 Apr 11;485(7398):381-5. PMID: 22495304
}
\examples{
data(Nora_5C)
show(E14)
show(MEF)
}
\keyword{datasets}

\name{binningC}
\Rdversion{1.1}
\alias{binningC}
\title{Windowing of high-throughput 'C' contact matrix}
\description{
  Binning of 'C' contact map
}
\usage{binningC(x, binsize=100000, bin.adjust=TRUE, upa=TRUE,
method=c("sum", "median","mean"), use.zero=TRUE, step=1, optimize.by = c("speed", "memory"))}
\arguments{
  \item{x}{object that inherits from class \code{HTCexp}}
  \item{binsize}{size of the bin to consider for windowing}
  \item{bin.adjust}{logical; adjust the size of the bin to the
   size of the genomic region}
  \item{upa}{logical; unique primer assignment. Allow one primer to
    belong to one or several bins}
  \item{method}{the method used to combine the counts. Must be
    \sQuote{mean}, \sQuote{median} or \sQuote{sum}}
  \item{use.zero}{logical; use the zero values in the \code{method} calculation}
  \item{step}{numeric; binning step size in \code{n} coverage
    \emph{i.e.} window step}
  \item{optimize.by}{"speed" will use faster methods but more RAM, and
          "memory" will be slower, but require less RAM}
}

\details{
 \code{bin.adjust} allows to work with bins of exactly the same size. Otherwise,
 the last bin is usually smaller than the others.\cr
 This function aims at changing the resolution of both 5C or Hi-C
 data. In case of 5C data (i.e. raw, not binned data), the contacts between all
 pairs of primers will be summarized per genomic bins (the median of all
 pairwise primers is usually used). In case of binned data
 (as Hi-C maps), the function can generate smaller resolution maps by
 aggregating bins. For instance, going from a 40kb resolution to a 1Mb
 resolution.\cr
 The \code{method} is used to combine the counts in a bin, must be \sQuote{mean}, \sQuote{median} or \sQuote{sum}.
 The \code{step} parameter allows to choose the overlap between the
 bins. A \code{step} of 2 means a 50\% overlap between two bins, a step
 of 3 means a 60\% overlap between two bins, \emph{etc.}
}

\value{
  An \code{HTCexp-class} object with binned intraction data. In this
  case, the genomic intervals are converted into bins of fixed size.
  The contact matrix is symetric.
}
\seealso{\code{\link{HTCexp-class}}}
\author{N. Servant, B. Lajoie}
\examples{
data(Nora_5C)

## Data binning 100kb, with a 1/3 overlap
E14.bin <- binningC(E14$chrXchrX, binsize=100000, step=3)
show(E14.bin)
## Move to a lower resolution map
E14.bin2 <- binningC(E14.bin, binsize=500000, step=1)
show(E14.bin2)
}
\keyword{manip}

\name{normICE}
\Rdversion{1.1}
\alias{normICE}

\title{Iterative Correction of Hi-C data (ICE)}
\description{
 Iterative correction leverages the unique pairwise genome-wide structure of Hi-C data to decompose the data into a set of biases and a map of relative contact probabilities between any two genomic loci, achieving equal visibility across all genomic regions.
}

\usage{normICE(x, max_iter=50, eps=1e-4, sparse.filter=0.02)}

\arguments{
    \item{x}{object that inherits from class \code{HTCexp}}
    \item{max_iter}{maximum number of iteration}
    \item{eps}{the relative increment in the results before declaring
      convergence}
    \item{sparse.filter}{Define which pourcentage of bins with high
    sparsity should be force to zero}
}

\details{
  The normalization of Hi-C data is based on matrix balancing algorithm which consists of iteratively
  estimating the matrix bias using the l1 norm. The method
  implemented here is the Sinkhorn-Knopp algorithm as used in the
  Imakaev et al. paper.
  Note that the original method is applied on the genome-wide Hi-C map,
  but that the method could be applied on intra-chromosomal maps at high
  resolution. 
}
\value{
  Returns a \code{HTCexp} object with a corrected contact map.
}
\references{Imakaev M, Fudenberg G, McCord RP, Naumova N, Goloborodko A, Lajoie BR, Dekker J, Mirny LA. Iterative correction of Hi-C data reveals hallmarks of chromosome organization.Nat Methods. 2012 Oct;9(10):999-1003.}
\author{N. Servant, N. Varoqaux}
\seealso{\code{\link{normLGF}}}
\examples{
\dontrun{
##Lieberman data
exDir <- system.file("extdata", package="HiTC")
l <- sapply(list.files(exDir, pattern=paste("HIC_gm06690_"), full.names=TRUE),
            import.my5C)
hiC <- HTClist(l)
hiC <- hiC[isIntraChrom(hiC)]

## Run ICE
hiC_iced <- HTClist(lapply(hiC, normICE))
}
}

\name{intervalsDist}
\Rdversion{1.1}
\alias{intervalsDist}
\title{intervalsDist}
\description{
   Compute the distance of intrachromosomal contacts of a 'C' experiment
}
\usage{intervalsDist(x, use.zero=TRUE)}
\arguments{
  \item{x}{object that inherits from class \code{HTCexp}}
  \item{use.zero}{if FALSE, the distance for non interacting
    regions (zero counts) are not reported}
}

\details{
  If \eqn{A} and \eqn{B} are the two sets of intervals and
  \eqn{s} and \eqn{e}, the start and end of an interval, the distance is calculated as :\cr
  \deqn{\min(|A_e - B_s|, |A_s - B_e|)}
    Only intrachromsomal contact maps can be use for this operation.
}

\value{
  A matrix of distances between genomic intervals
}
\seealso{\code{\link{HTCexp-class}}}
\author{N. Servant}
\examples{
data(Nora_5C)

## Calculate distances between primers/intervals
d<-intervalsDist(E14$chrXchrX)
}
\keyword{manip}

\name{export.my5C}
\Rdversion{1.1}
\alias{export.my5C}
\title{Export \code{HTCexp} object to my5C website format}
\description{
  Export \code{HTCexp} object to my5C website format
}
\usage{export.my5C(x, file, genome="mm9", per.chromosome=FALSE)}
\arguments{
  \item{x}{object that inherits from class \code{HTCexp}}
  \item{file}{character; the prefix of the output file}
  \item{genome}{The genome version. This information is only used for
    the 'mat' export format. See details}
  \item{per.chromosome}{logical; export each contact maps in a different
  file (i.e one per chromosome pair)}
}
\details{
  A tab-delimited matrix file is generated with the row and colnames
  defined as follow as in the my5C web tool :\cr
  REV_2|mm9|chrX:98831149-98834145
}
\author{N. Servant}
\seealso{\code{\link{exportC}}}
\examples{
\dontrun{
data(Nora_5C)

## Data binning
E14.bin<-binningC(E14$chrXchrX)

## Export the new intervals definition
export.my5C(E14.bin, file="E14my5C")
}
}
\keyword{plot}

\name{setIntervalScale}
\Rdversion{1.1}
\alias{setIntervalScale}
\title{Set x and y interval of the HTCexp object}
\description{Set x and y interval of the HTCexp object and update the
  contact map accordingly}
\usage{setIntervalScale(x, xgi, ygi, upa=TRUE, method=c("sum","median","mean"),
use.zero=TRUE, optimize.by = c("speed", "memory"))}
\arguments{
  \item{x}{object that inherits from class \code{HTCexp}}
  \item{ygi}{y intervals;
    see class \code{GRanges} for details}
  \item{xgi}{x intervals;
    see class \code{GRanges} for details}
   \item{upa}{logical; unique primer assignment. Allow one primer to
    belong to one or several bins}
  \item{method}{the method used to combine the counts. Must be
    \sQuote{mean}, \sQuote{median} or \sQuote{sum}}
  \item{use.zero}{logical; use the zero values in the \code{method}
    calculation}
  \item{optimize.by}{"speed" will use faster methods but more RAM, and
      "memory" will be slower, but require less RAM}
}

\details{
  Define new contact map based on the specified xgi and ygi
  intervals.

  This function has to be used carefully and can has important impact on
  the contact map. 
  It is important to note that the \code{setIntervalScale} function is different from the \code{binningC} function in the way that the output
  is not symetrical.
}

\value{
  A \code{HTCexp} object
}
\seealso{\code{\link{HTCexp-class}}}
\author{N. Servant}
\examples{
data(Nora_5C)

E14.bin<-binningC(E14$chrXchrX)

## I have two HTCexp samples defined with different intervals.
show(E14.bin)
show(MEF$chrXchrX)

## How to compare them ?
## One idea is to force the intervals definition of one object using the
## intervals of the other.

setIntervalScale(MEF$chrXchrX, xgi=x_intervals(E14.bin), ygi=y_intervals(E14.bin))
}
\keyword{manip}

\name{getRestrictionFragmentsPerChromosome}
\Rdversion{1.1}
\alias{getRestrictionFragmentsPerChromosome}

\title{Get a list of DNA restriction fragments}
\description{
  Performs the detection of restriction sites on a given genome and
  convert this information as a list of restriction fragments.}

\usage{getRestrictionFragmentsPerChromosome(resSite="AAGCTT", overhangs5=1,
chromosomes=NULL, genomePack="BSgenome.Mmusculus.UCSC.mm9")}

\arguments{
  \item{resSite}{the sequence of the restriction site to look
  for. Default is HindIII restriction site}
  \item{overhangs5}{5' overhangs on the DNA resulted from the cutting}
  \item{chromosomes}{vector of chromosome number to focus on. Default
  all the chromosomes for the specified genome.}
  \item{genomePack}{name of the genome package to access the DNA sequence}
}

\value{
  Returns a \code{GRanges} object with all restriction fragments for a
  given genome/chromosome.
}
\author{N. Servant}
\seealso{\code{\link{normLGF}}, \code{\link{setGenomicFeatures}}, \code{\link{getAnnotatedRestrictionSites}}}
\examples{

\dontrun{
## Mappability data From http://hgdownload.cse.ucsc.edu/goldenPath/hg18/encodeDCC/wgEncodeMapability/
map_hg18<- import("wgEncodeCrgMapabilityAlign100mer.bw",format="BigWig")

## 1- Get the list of restriction fragments for Human hg18 after HindIII digestion
resFrag <- getRestrictionFragmentsPerChromosome(resSite="AAGCTT", overhangs5=1, chromosomes="chrX", genomePack="BSgenome.Hsapiens.UCSC.hg18")
}

}
\keyword{hplot}

\name{extractRegion}
\Rdversion{1.1}
\alias{extractRegion}
\title{Extract a subset of the HTCexp object}
\description{
  Extract a subset of the \code{HTCexp} object based on genomic ranges
}
\usage{extractRegion(x, MARGIN, chr, from, to, exact=FALSE)}
\arguments{
  \item{x}{object that inherits from class \code{HTCexp}}
  \item{MARGIN}{a vector giving the subscripts which the function will be
          applied over as in 'apply' function.  E.g., '1' for the 'x'
	  intervals, and '2' for the 'y' intervals, 'c(1, 2)' indicates
	  'x' and 'y' intervals.}
  \item{chr}{character; the chromosome of the genomic region}
  \item{from}{numeric;  start of the genomic region}
  \item{to}{numeric; end of the genomic region}
  \item{exact}{logical; exact genomic region}
}
\details{
  By default, only the intervals fully included in the genomic ranges are returned.
  If exact is true, the overlapping intervals are also used, and
  forced to start/end at the specified position. If no intervals are
  overlapping, an interval with NA values is added.
}
\value{
  A \code{HTCexp} object
}
\author{N. Servant}
\seealso{\code{\link[GenomicRanges]{GRanges-class}}}
\examples{
data(Nora_5C)

## Focus on the genomic region chrX:98000000-100000000
E14sub<-extractRegion(E14$chrXchrX, c(1,2), chr="chrX", from=98000000, to=100000000)
show(E14sub)
}
\keyword{manip}

\name{import.my5C}
\Rdversion{1.1}
\alias{import.my5C}
\title{Import data from my5C webtool}
\description{
  Import data from my5C webtool
}

\usage{import.my5C(file, allPairwise=FALSE, rm.trans=FALSE, lazyload=FALSE)}

\arguments{
  \item{file}{input file from the my5C webtool}
  \item{allPairwise}{logical; generate all pairwise chromosomal
    contact maps, i.e chr1-chr2, chr2-chr1}
  \item{rm.trans}{if true, only intra-chromosomal maps are loaded}
  \item{lazyload}{logical; force the intra-chromosomal contact maps to
  be stored as triangular matrix}
}
\details{
  This function allows data import from the \href{http://my5c.umassmed.edu}{the my5C webtool}.\cr
  The matrix format is a tab-delimited format, corresponding to the
  contact map. The rownames and columnames are splitted to created
  the genome intervals (example : REV_2|mm9|chrX:98831149-98834145). 
  
  The \code{allPairwise} option is not necessary in case of symetric design. Otherwise, it will return all the pairwise contact maps.

  The matrix will be stored as a matrix inheriting from \code{Matrix}
  class. If forcesymmetrical=TRUE, the intrachromosomal matrix as coerced to
  \code{symmetricMatrix} class allowing a much more efficient memory usage.
}

\value{
  A \code{HTClist} object
}
\author{N. Servant}
\seealso{\code{\link{HTClist-class}},\code{\link{HTCexp-class}},\code{\link{importC}},
  \code{\link{Matrix-class}}, \code{\link[Matrix]{symmetricMatrix-class}}
}
\examples{
exDir <- system.file("extdata", package="HiTC")
## Load my5C matrix format
hiC<-import.my5C(file.path(exDir,"HIC_gm06690_chr14_chr14_1000000_obs.txt"))
detail(hiC)
}
\keyword{IO}

\name{directionalityIndex}
\Rdversion{1.1}
\alias{directionalityIndex}
\title{Directionality index calculation}
\description{
  Calculate the directionality index as proposed by Dixon et al. 2012
}

\usage{directionalityIndex(x, winup = 2e+06, windown = 2e+06)}
\arguments{
  \item{x}{\code{HTClist} object}
  \item{winup}{size of upstream window}
  \item{windown}{size of downstrean window}
}
\details{
  Calculate the directionality index as proposed by Dixon et al. This
  index is then used to call topological domains in Hi-C/5C data.
}

\value{
  A \code{numeric} vector
}
\author{N. Servant}
\seealso{\code{\link{HTClist-class}}}
\examples{
require(HiCDataHumanIMR90)
data(Dixon2012_IMR90)
hox <- extractRegion(hic_imr90_40$chr6chr6, chr="chr6", from=50e6, to=58e6)
di<-directionalityIndex(hox)
}
\keyword{IO}

\name{getAnnotatedRestrictionSites}
\Rdversion{1.1}
\alias{getAnnotatedRestrictionSites}

\title{Annotation of restriction sites}
\description{
  Performs the annotation of all restriction sites of a given genome
  (i.e. GC content, mappability, effective fragment length)
}

\usage{getAnnotatedRestrictionSites(resSite="AAGCTT", overhangs5=1,
chromosomes=NULL, genomePack="BSgenome.Mmusculus.UCSC.mm9", mappability=NULL, wingc=200, winmap=500)}

\arguments{
  \item{resSite}{the sequence of the restriction site to look
  for. Default is HindIII restriction site}
  \item{overhangs5}{5' overhangs on the DNA resulted from the cutting}
  \item{chromosomes}{vector of chromosome number to focus on. Default
  all the chromosomes for the specified genome.}
  \item{genomePack}{name of the genome package to access the DNA sequence}
  \item{wingc}{size of the window upstream and downstream the restriction site
  used to calculate the GC content}
  \item{mappability}{a \code{GRanges} with a 'score' describing the
  mappability of the genome}
  \item{winmap}{size of the window upstream and downstream the restriction site
  used to calculate the mappability}
}


\details{
  This function automatically annotate all the restriction sites of a
  given chromosome. The mappability is optional but strongly advice for
  Hi-C contact map normalization. This information can be easily
  download from public ressources like ftp://hgdownload.cse.ucsc.edu/gbdb/mm9/bbi/.
}
\value{
  Returns a \code{GRanges} object annotation data upstream (U) and
  downstream (D) the restriction sites.
}
\author{N. Servant}
\seealso{\code{\link{normLGF}}, \code{\link{setGenomicFeatures}}}
\examples{

\dontrun{
## Mappability data From http://hgdownload.cse.ucsc.edu/goldenPath/hg18/encodeDCC/wgEncodeMapability/
map_hg18<- import("wgEncodeCrgMapabilityAlign100mer.bw",format="BigWig")

## 1- Example of restriction sites annnotation
cutSites <- getAnnotatedRestrictionSites(resSite="AAGCTT", overhangs5=1, chromosomes="chr1", genomePack="BSgenome.Hsapiens.UCSC.hg18", wingc=200, mappability=map_hg18, winmap=500)
}

}
\keyword{hplot}

\name{pca.hic}
\Rdversion{1.1}
\alias{pca.hic}
\title{Perform Principle Component Analysis on Hi-C contact map}
\description{
  Perform Principle Component Analysis on Hi-C contact map
}
\usage{pca.hic(x, normPerExpected=TRUE, npc=2, asGRangesList=TRUE,
gene.gr=NULL, ...)}
\arguments{
  \item{x}{object that inherits from class \code{HTCexp}}
  \item{normPerExpected}{normalized by expected interaction using the loess
    calculation of distance dependency. see normPerExpected}
  \item{npc}{numeric; number of first principal component to return}
  \item{asGRangesList}{if TRUE a GRangesList object is returned where the scores represent the eigenvector}
  \item{gene.gr}{object of class \code{GenomicRanges} describing the
    genes position. If used, the A/B compartments classes are defined
    based on gene density}
  \item{...}{additional parameters passed to \code{normPerExpected} function}
}

\details{
 This method was apply by Lieberman-Aiden et al. 2009 to correlate the
 annotation profiles (genes, ChIP-seq, etc.) with the topological
 domains observed in Hi-C (see Fig3G of Lieberman-Aiden et al. 2009)
}

\value{
  A list with the eigen vector(s) of the \code{npc} first principal
  component(s), and their importance
}
\seealso{\code{\link{normPerExpected}}}
\author{N. Servant, B. Lajoie, R. McCord}
\examples{
## Get Lieberman-Aiden Hi-C data
exDir <- system.file("extdata", package="HiTC")
l <- sapply(list.files(exDir, pattern=paste("HIC_gm06690_"), full.names=TRUE),
import.my5C)
hiC <- HTClist(l)

## Performed PCA
pr<-pca.hic(hiC$chr14chr14, npc=1, asGRangesList=TRUE)

}
\keyword{manip}
